package es.upm.fi.dia.ontology.webode.wpbs;

// Java stuff
import java.util.Calendar;
import java.util.Date;

import es.upm.fi.dia.ontology.webode.service.ClassAttributeDescriptor;
import es.upm.fi.dia.ontology.webode.service.ConstantDescriptor;
import es.upm.fi.dia.ontology.webode.service.FormulaDescriptor;
import es.upm.fi.dia.ontology.webode.service.Group;
import es.upm.fi.dia.ontology.webode.service.Instance;
import es.upm.fi.dia.ontology.webode.service.InstanceAttributeDescriptor;
import es.upm.fi.dia.ontology.webode.service.ODEService;
import es.upm.fi.dia.ontology.webode.service.OntologyDescriptor;
import es.upm.fi.dia.ontology.webode.service.ReferenceDescriptor;
import es.upm.fi.dia.ontology.webode.service.SynonymAbbreviation;
import es.upm.fi.dia.ontology.webode.service.Term;
import es.upm.fi.dia.ontology.webode.service.TermRelation;
import es.upm.fi.dia.ontology.webode.service.TermRelationInstance;
import es.upm.fi.dia.ontology.webode.service.TermTypes;
import es.upm.fi.dia.ontology.webode.service.ValueTypes;
import es.upm.fi.dia.ontology.webode.service.WebODEException;

/** Implements a set of benchmarks for testing ODE service.
 *	All the benchmarks in this class test basic operations regarding
 *	inserting values in the database.
 *	All the benchmarks leave the system in the same state after executing.
 */
public class benchmarkBasicInsert
{

	/**
	 * Creates several ontologies with default general values.
	 */
	public static void benchmark1_1_01 (ODEService odeService) throws Exception
	{
		final int NEW_ONTOLOGIES = benchmarkConstants.TEST_ONTOLOGIES;
		final String ONTOLOGY_NAME = "newOntology";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_ONTOLOGIES);

		// Creates ontologies one by one
		System.out.println("Creating ontology: ");
		for (int i=1; i<=NEW_ONTOLOGIES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+ONTOLOGY_NAME+i);

			try{
				// Creates the ontology
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				Date now=Calendar.getInstance().getTime();
				OntologyDescriptor od = new OntologyDescriptor(ONTOLOGY_NAME+i,"webode://webode.dia.fi.upm.es/"+ONTOLOGY_NAME+i+"#","new ontology "+i,
																										 	 "admin","admin",now,now);
				odeService.createOntology(od);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				//Removes ontology
				odeService.removeOntology(ONTOLOGY_NAME+i);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating ontology: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_01.txt");
	}

	/**
	 *	Inserts several term references in an ontology.
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 */
	public static void benchmark1_1_02 (ODEService odeService) throws Exception
	{
		final int NEW_TERM_REFERENCES = benchmarkConstants.TEST_TERM_REFERENCES;
		final String ONTOLOGY_NAME = "ontology0";
		final String TERM_REFERENCE_NAME = "newTermReference";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_TERM_REFERENCES);

		// Creates term references one by one
		System.out.println("Creating term reference: ");
		for (int i=1; i<=NEW_TERM_REFERENCES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+TERM_REFERENCE_NAME+i);

			try{
				// Creates the term reference
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				ReferenceDescriptor rd = new ReferenceDescriptor(TERM_REFERENCE_NAME+i,
																												 "reference "+i);
				odeService.addReference(ONTOLOGY_NAME,rd);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the term reference
				odeService.removeReference(ONTOLOGY_NAME,TERM_REFERENCE_NAME+i);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating term reference: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_02.txt");
	}

	/**
	 * Inserts several term references in several ontologies.
	 *
	 *	The database must have NEW_TERM_REFERENCES ontologies named ONTOLOGY_NAMEx
	 */
	public static void benchmark1_1_03 (ODEService odeService) throws Exception
	{
		final int NEW_TERM_REFERENCES = benchmarkConstants.TEST_TERM_REFERENCES;
		final String ONTOLOGY_NAME = "ontology";
		final String TERM_REFERENCE_NAME = "newTermReference_1";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_TERM_REFERENCES);

		// Creates term references one by one
		System.out.println("Creating term reference: ");
		for (int i=1; i<=NEW_TERM_REFERENCES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+TERM_REFERENCE_NAME+"("+i+")");

			try{
				// Creates the term reference
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				ReferenceDescriptor rd = new ReferenceDescriptor(TERM_REFERENCE_NAME,
																												 "reference "+i);
				odeService.addReference(ONTOLOGY_NAME+i,rd);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the term reference
				odeService.removeReference(ONTOLOGY_NAME+i,TERM_REFERENCE_NAME);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating term reference: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_03.txt");
	}

	/**
	 * Inserts several ontology references in an ontology.
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 */
	public static void benchmark1_1_04 (ODEService odeService) throws Exception
	{
		final int NEW_ONTOLOGY_REFERENCES = benchmarkConstants.TEST_ONTOLOGY_REFERENCES;
		final String ONTOLOGY_NAME = "ontology0";
		final String ONTOLOGY_REFERENCE_NAME = "newOntologyReference";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_ONTOLOGY_REFERENCES);

		// Creates ontology references one by one
		System.out.println("Creating ontology reference: ");
		for (int i=1; i<=NEW_ONTOLOGY_REFERENCES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+ONTOLOGY_REFERENCE_NAME+i);

			try{
				// Creates the ontology reference
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				ReferenceDescriptor rd = new ReferenceDescriptor(ONTOLOGY_REFERENCE_NAME+i,
																												 "reference of "+ONTOLOGY_NAME);
				odeService.addReferenceToOntology(ONTOLOGY_NAME,rd);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the ontology reference
				odeService.removeReference(ONTOLOGY_NAME,ONTOLOGY_REFERENCE_NAME+i);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating ontology reference: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_04.txt");
	}

	/**
	 * Inserts several ontology references in several ontologies.
	 *
	 *	The database must have NEW_ONTOLOGY_REFERENCES ontologies named ONTOLOGY_NAMEx
	 */
	public static void benchmark1_1_05 (ODEService odeService) throws Exception
	{
		final int NEW_ONTOLOGY_REFERENCES = benchmarkConstants.TEST_ONTOLOGY_REFERENCES;
		final String ONTOLOGY_NAME = "ontology";
		final String ONTOLOGY_REFERENCE_NAME = "newOntologyReference_1";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_ONTOLOGY_REFERENCES);

		// Creates ontology references one by one
		System.out.println("Creating ontology reference: ");
		for (int i=1; i<=NEW_ONTOLOGY_REFERENCES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+ONTOLOGY_REFERENCE_NAME+"("+i+")");

			try{
				// Creates the ontology reference
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				ReferenceDescriptor rd = new ReferenceDescriptor(ONTOLOGY_REFERENCE_NAME,
																												 "reference of "+ONTOLOGY_NAME+i);
				odeService.addReferenceToOntology(ONTOLOGY_NAME+i,rd);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the ontology reference
				odeService.removeReference(ONTOLOGY_NAME+i,ONTOLOGY_REFERENCE_NAME);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating ontology reference: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_05.txt");
	}

	/**
	 *	Relates several term references with concepts in an ontology.
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 *	with NEW_TERM_REFERENCES concepts named CONCEPT_NAMEx and
	 *	NEW_TERM_REFERENCES term references names TERM_REFERENCE_NAMEx
	 */
	public static void benchmark1_1_06 (ODEService odeService) throws Exception
	{
		final int NEW_TERM_REFERENCES = benchmarkConstants.TEST_TERM_REFERENCE_RELATIONS;
		final String CONCEPT_NAME = "concept";
		final String TERM_REFERENCE_NAME = "termReference";
		final String ONTOLOGY_NAME = "ontology0";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_TERM_REFERENCES);

		// Relates term references one by one
		System.out.println("Relating: ");
		for (int i=1; i<=NEW_TERM_REFERENCES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+TERM_REFERENCE_NAME+i+" with "+CONCEPT_NAME+i);

			try{
				// Relates the term reference
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				odeService.relateReferenceToTerm(ONTOLOGY_NAME,CONCEPT_NAME+i,TERM_REFERENCE_NAME+i);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Unrelates the term reference
				odeService.unrelateReferenceToTerm(ONTOLOGY_NAME,CONCEPT_NAME+i,TERM_REFERENCE_NAME+i);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR relating term reference: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_06.txt");
	}

	/**
	 *	Relates several term references with concepts in several ontologies.
	 *
	 *	The database must have NEW_TERM_REFERENCES ontologies named ONTOLOGY_NAMEx
	 *	with a concept named CONCEPT_NAME and
	 *	a term reference named TERM_REFERENCE_NAME
	 */
	public static void benchmark1_1_07 (ODEService odeService) throws Exception
	{
		final int NEW_TERM_REFERENCES = benchmarkConstants.TEST_TERM_REFERENCE_RELATIONS;
		final String CONCEPT_NAME = "concept1";
		final String TERM_REFERENCE_NAME = "termReference1";
		final String ONTOLOGY_NAME = "ontology";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_TERM_REFERENCES);

		// Relates term references one by one
		System.out.println("Relating term reference: ");
		for (int i=1; i<=NEW_TERM_REFERENCES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+TERM_REFERENCE_NAME+" with "+CONCEPT_NAME+"("+i+")");

			try{
				// Relates the term reference
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				odeService.relateReferenceToTerm(ONTOLOGY_NAME+i,CONCEPT_NAME,TERM_REFERENCE_NAME);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Unrelates the term reference
				odeService.unrelateReferenceToTerm(ONTOLOGY_NAME+i,CONCEPT_NAME,TERM_REFERENCE_NAME);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR relating term reference: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_07.txt");
	}

	/**
	 *	Inserts several concepts in an ontology.
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 */
	public static void benchmark1_1_08 (ODEService odeService) throws Exception
	{
		final int NEW_CONCEPTS = benchmarkConstants.TEST_CONCEPTS;
		final String CONCEPT_NAME = "newConcept";
		final String ONTOLOGY_NAME = "ontology0";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_CONCEPTS);

		// Creates concepts one by one
		System.out.println("Creating concept: ");
		for (int i=1; i<=NEW_CONCEPTS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CONCEPT_NAME+i);

			try{
				// Creates the concept
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				odeService.insertTerm(ONTOLOGY_NAME,CONCEPT_NAME+i,
															"concept "+i+" of "+ONTOLOGY_NAME,Term.CONCEPT);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the concept
				odeService.removeTerm(ONTOLOGY_NAME,CONCEPT_NAME+i);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating concept: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_08.txt");
	}

	/**
	 *	Inserts several concepts in several ontologies.
	 *
	 *	The database must have NEW_CONCEPTS ontologies named ONTOLOGY_NAMEx
	 */
	public static void benchmark1_1_09 (ODEService odeService) throws Exception
	{
		final int NEW_CONCEPTS = benchmarkConstants.TEST_CONCEPTS;
		final String CONCEPT_NAME = "newConcept_1";
		final String ONTOLOGY_NAME = "ontology";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_CONCEPTS);

		// Creates concepts one by one
		System.out.println("Creating concept: ");
		for (int i=1; i<=NEW_CONCEPTS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CONCEPT_NAME+"("+i+")");

			try{
				// Creates the concept
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				odeService.insertTerm(ONTOLOGY_NAME+i,CONCEPT_NAME,
															"concept of "+ONTOLOGY_NAME+i,Term.CONCEPT);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the concept
				odeService.removeTerm(ONTOLOGY_NAME+i,CONCEPT_NAME);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating concept: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_09.txt");
	}

	/**
	 *	Inserts several term relations in an ontology.
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 *	with NEW_TERM_RELATIONS+1 concepts named CONCEPT_NAMEx
	 */
	public static void benchmark1_1_10 (ODEService odeService) throws Exception
	{
		final int NEW_TERM_RELATIONS = benchmarkConstants.TEST_TERM_RELATIONS;
		final String TERM_RELATION_NAME = "newTermRelation";
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology0";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_TERM_RELATIONS);

		// Creates term relations one by one
		System.out.println("Creating term relation: ");
		for (int i=1; i<=NEW_TERM_RELATIONS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+TERM_RELATION_NAME+i);

			try{
				// Creates the term relation
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				TermRelation tr = new TermRelation(ONTOLOGY_NAME,null,TERM_RELATION_NAME+i,
																					 CONCEPT_NAME+i,CONCEPT_NAME+(i+1));
				odeService.insertTermRelation(tr);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the term relation
				TermRelation[] trs = odeService.getTermRelations(ONTOLOGY_NAME,CONCEPT_NAME+i,false);
				int j=0;
				while (!(trs[j].name.equals(TERM_RELATION_NAME+i)) && (j<trs.length)){
					j++;
				}
				odeService.removeTermRelation(trs[j]);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating term relation: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_10.txt");
	}

	/**
	 *	Inserts several term relations in several ontologies.
	 *
	 *	The database must have NEW_TERM_RELATIONS ontologies named ONTOLOGY_NAMEx
	 *	with 2 concepts named CONCEPT_NAME1 and CONCEPT_NAME2
	 */
	public static void benchmark1_1_11 (ODEService odeService) throws Exception
	{
		final int NEW_TERM_RELATIONS = benchmarkConstants.TEST_TERM_RELATIONS;
		final String TERM_RELATION_NAME = "newTermRelation_1";
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_TERM_RELATIONS);

		// Creates term relations one by one
		System.out.println("Creating term relation: ");
		for (int i=1; i<=NEW_TERM_RELATIONS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+TERM_RELATION_NAME+"("+i+")");

			try{
				// Creates the term relation
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				TermRelation tr = new TermRelation(ONTOLOGY_NAME+i,null,TERM_RELATION_NAME,
																					 CONCEPT_NAME+"1",CONCEPT_NAME+"2");
				odeService.insertTermRelation(tr);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the term relation
				TermRelation[] trs = odeService.getTermRelations(ONTOLOGY_NAME+i,CONCEPT_NAME+"1",false);
				int j=0;
				while (!(trs[j].name.equals(TERM_RELATION_NAME)) && (j<trs.length)){
					j++;
				}
				odeService.removeTermRelation(trs[j]);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating term relation: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_11.txt");
	}

	/**
	 *	Inserts several class attributes in a term.
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 *	with a concept named CONCEPT_NAME
	 */
	public static void benchmark1_1_12 (ODEService odeService) throws Exception
	{
		final int NEW_CLASS_ATTRIBUTES = benchmarkConstants.TEST_CLASS_ATTRIBUTES;
		final String CLASS_ATTRIBUTE_NAME = "newClassAttribute";
		final String CONCEPT_NAME = "concept1";
		final String ONTOLOGY_NAME = "ontology0";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_CLASS_ATTRIBUTES);

		// Creates class attributes one by one
		System.out.println("Creating class attribute: ");
		for (int i=1; i<=NEW_CLASS_ATTRIBUTES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CLASS_ATTRIBUTE_NAME+i);

			try{
				// Creates the class attribute
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				ClassAttributeDescriptor cad = new ClassAttributeDescriptor(
					CLASS_ATTRIBUTE_NAME+i,CONCEPT_NAME,ValueTypes.STRING,ONTOLOGY_NAME,
					"measurement unit "+i,"precision"+i,1,100,
					"class attribute "+i+" of "+CONCEPT_NAME);
				odeService.insertClassAttribute(ONTOLOGY_NAME,cad);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the class attribute
				odeService.removeTerm(ONTOLOGY_NAME,CLASS_ATTRIBUTE_NAME+i,CONCEPT_NAME);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating class attribute: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_12.txt");
	}

	/**
	 *	Inserts several class attributes in several terms.
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 *	with NEW_CLASS_ATTRIBUTES concepts named CONCEPT_NAMEx
	 */
	public static void benchmark1_1_13 (ODEService odeService) throws Exception
	{
		final int NEW_CLASS_ATTRIBUTES = benchmarkConstants.TEST_CLASS_ATTRIBUTES;
		final String CLASS_ATTRIBUTE_NAME = "newClassAttribute_1";
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology0";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_CLASS_ATTRIBUTES);

		// Creates class attributes one by one
		System.out.println("Creating class attribute: ");
		for (int i=1; i<=NEW_CLASS_ATTRIBUTES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CLASS_ATTRIBUTE_NAME+"("+i+")");

			try{
				// Creates the class attribute
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				ClassAttributeDescriptor cad = new ClassAttributeDescriptor(
					CLASS_ATTRIBUTE_NAME,CONCEPT_NAME+i,ValueTypes.STRING,ONTOLOGY_NAME,
					"measurement unit "+i,"precision"+i,1,100,
					"class attribute "+i+" of "+CONCEPT_NAME+i);
				odeService.insertClassAttribute(ONTOLOGY_NAME,cad);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the class attribute
				odeService.removeTerm(ONTOLOGY_NAME,CLASS_ATTRIBUTE_NAME,CONCEPT_NAME+i);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating class attribute: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_13.txt");
	}

	/**
	 *	Inserts several values in a class attribute.
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 *	with a concept named CONCEPT_NAME
	 *	with a class attribute named CLASS_ATTRIBUTE_NAME
	 *	with a maximum cardinality of NEW_CLASS_ATTRIBUTE_VALUES
	 */
	public static void benchmark1_1_14 (ODEService odeService) throws Exception
	{
		final int NEW_CLASS_ATTRIBUTE_VALUES = benchmarkConstants.TEST_CLASS_ATTRIBUTE_VALUES;
		final String CLASS_ATTRIBUTE_VALUE_NAME = "newClassAttributeValue";
		final String CLASS_ATTRIBUTE_NAME = "classAttribute1";
		final String CONCEPT_NAME = "concept1";
		final String ONTOLOGY_NAME = "ontology0";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_CLASS_ATTRIBUTE_VALUES);

		// Creates class attribute values one by one
		System.out.println("Creating class attribute value: ");
		for (int i=1; i<=NEW_CLASS_ATTRIBUTE_VALUES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CLASS_ATTRIBUTE_VALUE_NAME+i);

			try{
				// Creates the class attribute value
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				odeService.addValueToClassAttribute(ONTOLOGY_NAME,CLASS_ATTRIBUTE_NAME,
																						CONCEPT_NAME,CLASS_ATTRIBUTE_VALUE_NAME+i);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the class attribute value
				odeService.removeValueFromClassAttribute(ONTOLOGY_NAME,CLASS_ATTRIBUTE_NAME,
																								 CONCEPT_NAME,CLASS_ATTRIBUTE_VALUE_NAME+i);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating class attribute value: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_14.txt");
	}

	/**
	 *	Inserts several values in several class attributes.
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 *	with a concept named CONCEPT_NAME
	 *	with NEW_CLASS_ATTRIBUTE_VALUES class attributes named CLASS_ATTRIBUTE_NAMEx
	 *	with a maximum cardinality of one
	 */
	public static void benchmark1_1_15 (ODEService odeService) throws Exception
	{
		final int NEW_CLASS_ATTRIBUTE_VALUES = benchmarkConstants.TEST_CLASS_ATTRIBUTE_VALUES;
		final String CLASS_ATTRIBUTE_VALUE_NAME = "newClassAttributeValue_1";
		final String CLASS_ATTRIBUTE_NAME = "classAttribute";
		final String CONCEPT_NAME = "concept1";
		final String ONTOLOGY_NAME = "ontology0";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_CLASS_ATTRIBUTE_VALUES);

		// Creates class attribute values one by one
		System.out.println("Creating class attribute value: ");
		for (int i=1; i<=NEW_CLASS_ATTRIBUTE_VALUES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CLASS_ATTRIBUTE_VALUE_NAME+"("+i+")");

			try{
				// Creates the class attribute value
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				odeService.addValueToClassAttribute(ONTOLOGY_NAME,CLASS_ATTRIBUTE_NAME+i,
																						CONCEPT_NAME,CLASS_ATTRIBUTE_VALUE_NAME);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the class attribute value
				odeService.removeValueFromClassAttribute(ONTOLOGY_NAME,CLASS_ATTRIBUTE_NAME+i,
																								 CONCEPT_NAME,CLASS_ATTRIBUTE_VALUE_NAME);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating class attribute value: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_15.txt");
	}

	/**
	 *	Inserts several instance attributes in a term.
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 *	with a concept named CONCEPT_NAME
	 */
	public static void benchmark1_1_16 (ODEService odeService) throws Exception
	{
		final int NEW_INSTANCE_ATTRIBUTES = benchmarkConstants.TEST_INSTANCE_ATTRIBUTES;
		final String INSTANCE_ATTRIBUTE_NAME = "newInstanceAttribute";
		final String CONCEPT_NAME = "concept1";
		final String ONTOLOGY_NAME = "ontology0";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_INSTANCE_ATTRIBUTES);

		// Creates instance attributes one by one
		System.out.println("Creating instance attribute: ");
		for (int i=1; i<=NEW_INSTANCE_ATTRIBUTES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+INSTANCE_ATTRIBUTE_NAME+i);

			try{
				// Creates the instance attribute
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				InstanceAttributeDescriptor iad = new InstanceAttributeDescriptor(
					INSTANCE_ATTRIBUTE_NAME+i,CONCEPT_NAME,ValueTypes.STRING,ONTOLOGY_NAME,
					"measurement unit "+i,"precision"+i,1,100,
					"instance attribute "+i+" of "+CONCEPT_NAME,"100","1");
				odeService.insertInstanceAttribute(ONTOLOGY_NAME,iad);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the instance attribute
				odeService.removeTerm(ONTOLOGY_NAME,INSTANCE_ATTRIBUTE_NAME+i,CONCEPT_NAME);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating instance attribute: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_16.txt");
	}

	/**
	 *	Inserts several instance attributes in several terms.
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 *	with NEW_INSTANCE_ATTRIBUTES concepts named CONCEPT_NAMEx
	 */
	public static void benchmark1_1_17 (ODEService odeService) throws Exception
	{
		final int NEW_INSTANCE_ATTRIBUTES = benchmarkConstants.TEST_INSTANCE_ATTRIBUTES;
		final String INSTANCE_ATTRIBUTE_NAME = "newInstanceAttribute_1";
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology0";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_INSTANCE_ATTRIBUTES);

		// Creates instance attributes one by one
		System.out.println("Creating instance attribute: ");
		for (int i=1; i<=NEW_INSTANCE_ATTRIBUTES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+INSTANCE_ATTRIBUTE_NAME+"("+i+")");

			try{
				// Creates the instance attribute
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				InstanceAttributeDescriptor iad = new InstanceAttributeDescriptor(
					INSTANCE_ATTRIBUTE_NAME,CONCEPT_NAME+i,ValueTypes.STRING,ONTOLOGY_NAME,
					"measurement unit "+i,"precision"+i,1,100,
					"instance attribute of "+CONCEPT_NAME+i,"100","1");
				odeService.insertInstanceAttribute(ONTOLOGY_NAME,iad);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the instance attribute
				odeService.removeTerm(ONTOLOGY_NAME,INSTANCE_ATTRIBUTE_NAME,CONCEPT_NAME+i);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating instance attribute: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_17.txt");
	}

	/**
	 *	Inserts several values in an instance attribute.
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 *	with a concept named CONCEPT_NAME
	 *	with an instance attribute named INSTANCE_ATTRIBUTE_NAME
	 *	with a maximum cardinality of NEW_INSTANCE_ATTRIBUTE_VALUES
	 */
	public static void benchmark1_1_18 (ODEService odeService) throws Exception
	{
		final int NEW_INSTANCE_ATTRIBUTE_VALUES = benchmarkConstants.TEST_INSTANCE_ATTRIBUTE_VALUES;
		final String INSTANCE_ATTRIBUTE_VALUE_NAME = "newInstanceAttributeValue";
		final String INSTANCE_ATTRIBUTE_NAME = "instanceAttribute1";
		final String CONCEPT_NAME = "concept1";
		final String ONTOLOGY_NAME = "ontology0";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_INSTANCE_ATTRIBUTE_VALUES);

		// Creates instance attribute values one by one
		System.out.println("Creating instance attribute value: ");
		for (int i=1; i<=NEW_INSTANCE_ATTRIBUTE_VALUES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+INSTANCE_ATTRIBUTE_VALUE_NAME+i);

			try{
				// Creates the instance attribute value
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				odeService.addValueToInstanceAttribute(ONTOLOGY_NAME,INSTANCE_ATTRIBUTE_NAME,
																						   CONCEPT_NAME,INSTANCE_ATTRIBUTE_VALUE_NAME+i);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the instance attribute value
				odeService.removeValueFromInstanceAttribute(ONTOLOGY_NAME,INSTANCE_ATTRIBUTE_NAME,
																									  CONCEPT_NAME,INSTANCE_ATTRIBUTE_VALUE_NAME+i);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating instance attribute value: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_18.txt");
	}

	/**
	 *	Inserts several values in several instance attributes.
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 *	with a concept named CONCEPT_NAME
	 *	with NEW_INSTANCE_ATTRIBUTE_VALUES instance attributes named INSTANCE_ATTRIBUTE_NAMEx
	 *	with a maximum cardinality of one
	 */
	public static void benchmark1_1_19 (ODEService odeService) throws Exception
	{
		final int NEW_INSTANCE_ATTRIBUTE_VALUES = benchmarkConstants.TEST_INSTANCE_ATTRIBUTE_VALUES;
		final String INSTANCE_ATTRIBUTE_VALUE_NAME = "newInstanceAttributeValue_1";
		final String INSTANCE_ATTRIBUTE_NAME = "instanceAttribute";
		final String CONCEPT_NAME = "concept1";
		final String ONTOLOGY_NAME = "ontology0";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_INSTANCE_ATTRIBUTE_VALUES);

		// Creates instance attribute values one by one
		System.out.println("Creating instance attribute value: ");
		for (int i=1; i<=NEW_INSTANCE_ATTRIBUTE_VALUES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+INSTANCE_ATTRIBUTE_VALUE_NAME+"("+i+")");

			try{
				// Creates the instance attribute value
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				odeService.addValueToInstanceAttribute(ONTOLOGY_NAME,INSTANCE_ATTRIBUTE_NAME+i,
																						   CONCEPT_NAME,INSTANCE_ATTRIBUTE_VALUE_NAME);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the instance attribute value
				odeService.removeValueFromInstanceAttribute(ONTOLOGY_NAME,INSTANCE_ATTRIBUTE_NAME+i,
																									  CONCEPT_NAME,INSTANCE_ATTRIBUTE_VALUE_NAME);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating instance attribute value: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_19.txt");
	}

	/**
	 *	Inserts several synonyms in a term.
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 *	with a concept named CONCEPT_NAME
	 */
	public static void benchmark1_1_20 (ODEService odeService) throws Exception
	{
		final int NEW_SYNONYMS = benchmarkConstants.TEST_SYNONYMS;
		final String SYNONYM_NAME = "newSynonym";
		final String CONCEPT_NAME = "concept1";
		final String ONTOLOGY_NAME = "ontology0";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_SYNONYMS);

		// Creates synonyms one by one
		System.out.println("Creating synonym: ");
		for (int i=1; i<=NEW_SYNONYMS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+SYNONYM_NAME+i);

			try{
				// Creates the synonym
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				SynonymAbbreviation sa = new SynonymAbbreviation(
																			SYNONYM_NAME+i, CONCEPT_NAME, null,
																			TermTypes.SYNONYM, "synonym of "+CONCEPT_NAME);
				odeService.addSynonymToTerm(ONTOLOGY_NAME,sa);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the synonym
				odeService.deleteSynonym(ONTOLOGY_NAME,SYNONYM_NAME+i,CONCEPT_NAME,null);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating synonym: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_20.txt");
	}

	/**
	 *	Inserts several syonyms in several terms.
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 *	with NEW_SYNONYMS concepts named CONCEPT_NAMEx
	 */
	public static void benchmark1_1_21 (ODEService odeService) throws Exception
	{
		final int NEW_SYNONYMS = benchmarkConstants.TEST_SYNONYMS;
		final String SYNONYM_NAME = "newSynonym_1";
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology0";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_SYNONYMS);

		// Creates synonyms one by one
		System.out.println("Creating synonym: ");
		for (int i=1; i<=NEW_SYNONYMS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+SYNONYM_NAME+"("+i+")");

			try{
				// Creates the synonym
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				SynonymAbbreviation sa = new SynonymAbbreviation(
																			SYNONYM_NAME, CONCEPT_NAME+i, null,
																			TermTypes.SYNONYM, "synonym of "+CONCEPT_NAME+i);
				odeService.addSynonymToTerm(ONTOLOGY_NAME,sa);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the synonym
				odeService.deleteSynonym(ONTOLOGY_NAME,SYNONYM_NAME,CONCEPT_NAME+i,null);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating synonym: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_21.txt");
	}

	/**
	 *	Inserts several imported terms in an ontology.
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 */
	public static void benchmark1_1_22 (ODEService odeService) throws Exception
	{
		final int NEW_IMPORTED_TERMS = benchmarkConstants.TEST_IMPORTED_TERMS;
		final String IMPORTED_TERM_NAME = "newImportedTerm";
		final String ONTOLOGY_NAME = "ontology0";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_IMPORTED_TERMS);

		// Creates imported terms one by one
		System.out.println("Creating imported term: ");
		for (int i=1; i<=NEW_IMPORTED_TERMS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+IMPORTED_TERM_NAME+i);

			try{
				// Creates the imported term
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				// Esto ha cambiado de la iteracin 3 a la 4.
				// Se introduce el namespace
				//odeService.importTerm(ONTOLOGY_NAME+i,"originalName"+i,IMPORTED_TERM_NAME,
				//											"http://www.noexists.com/originalName"+i);
				odeService.importTerm(ONTOLOGY_NAME,"http://www.noexists.com/#","noexists",
															IMPORTED_TERM_NAME+i,
															"http://www.noexists.com/#originalName"+i);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the imported term
				odeService.removeTerm(ONTOLOGY_NAME,IMPORTED_TERM_NAME+i);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating imported term: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_22.txt");
	}

	/**
	 *	Inserts several imported terms in several ontologies.
	 *
	 *	The database must have NEW_IMPORTED_TERMS ontologies named ONTOLOGY_NAMEx
	 */
	public static void benchmark1_1_23 (ODEService odeService) throws Exception
	{
		final int NEW_IMPORTED_TERMS = benchmarkConstants.TEST_IMPORTED_TERMS;
		final String IMPORTED_TERM_NAME = "newImportedTerm_1";
		final String ONTOLOGY_NAME = "ontology";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_IMPORTED_TERMS);

		// Creates imported terms one by one
		System.out.println("Creating imported term: ");
		for (int i=1; i<=NEW_IMPORTED_TERMS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+IMPORTED_TERM_NAME+"("+i+")");

			try{
				// Creates the imported term
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				// Esto ha cambiado de la iteracin 3 a la 4.
				// Se introduce el namespace
				//odeService.importTerm(ONTOLOGY_NAME+i,"originalName"+i,IMPORTED_TERM_NAME,
				//											"http://www.noexists.com/originalName"+i);
				odeService.importTerm(ONTOLOGY_NAME+i,"http://www.noexists.com/#","noexists",
															IMPORTED_TERM_NAME,
															"http://www.noexists.com/#originalName"+i);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the imported term
				odeService.removeTerm(ONTOLOGY_NAME+i,IMPORTED_TERM_NAME);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating imported term: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_23.txt");
	}

	/**
	 *	Inserts several constants in an ontology.
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 */
	public static void benchmark1_1_24 (ODEService odeService) throws Exception
	{
		final int NEW_CONSTANTS = benchmarkConstants.TEST_CONSTANTS;
		final String CONSTANT_NAME = "newConstant";
		final String ONTOLOGY_NAME = "ontology0";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_CONSTANTS);

		// Creates constants one by one
		System.out.println("Creating constant: ");
		for (int i=1; i<=NEW_CONSTANTS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CONSTANT_NAME+i);

			try{
				// Creates the constant
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				ConstantDescriptor cd = new ConstantDescriptor(
																			CONSTANT_NAME+i, "constant "+i+" of "+ONTOLOGY_NAME,
																			ValueTypes.STRING,"measurement unit "+i,
																			"value of "+CONSTANT_NAME+i);
				odeService.insertConstant(ONTOLOGY_NAME,cd);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the constant
				odeService.removeTerm(ONTOLOGY_NAME,CONSTANT_NAME+i);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating constant: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_24.txt");
	}

	/**
	 *	Inserts several constants in several ontologies.
	 *
	 *	The database must have NEW_CONSTANTS ontologies named ONTOLOGY_NAMEx
	 */
	public static void benchmark1_1_25 (ODEService odeService) throws Exception
	{
		final int NEW_CONSTANTS = benchmarkConstants.TEST_CONSTANTS;
		final String CONSTANT_NAME = "newConstant_1";
		final String ONTOLOGY_NAME = "ontology";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_CONSTANTS);

		// Creates constants one by one
		System.out.println("Creating constant: ");
		for (int i=1; i<=NEW_CONSTANTS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CONSTANT_NAME+"("+i+")");

			try{
				// Creates the constant
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				ConstantDescriptor cd = new ConstantDescriptor(
																			CONSTANT_NAME, "constant of "+ONTOLOGY_NAME+i,
																			ValueTypes.STRING,"measurement unit 1",
																			"value of "+CONSTANT_NAME);
				odeService.insertConstant(ONTOLOGY_NAME+i,cd);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the constant
				odeService.removeTerm(ONTOLOGY_NAME+i,CONSTANT_NAME);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating constant: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_25.txt");
	}

	/**
	 *	Inserts several formulas in an ontology.
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 */
	public static void benchmark1_1_26 (ODEService odeService) throws Exception
	{
		final int NEW_FORMULAS = benchmarkConstants.TEST_FORMULAS;
		final String FORMULA_NAME = "newFormula";
		final String ONTOLOGY_NAME = "ontology0";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_FORMULAS);

		// Creates formulas one by one
		System.out.println("Creating formula: ");
		for (int i=1; i<=NEW_FORMULAS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+FORMULA_NAME+i);

			try{
				// Creates the formula
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				FormulaDescriptor fd = new FormulaDescriptor(
													  ONTOLOGY_NAME,FORMULA_NAME+i, "formula "+i+" of "+ONTOLOGY_NAME,
														"Have friends and they will give you...",
														"person(?x), person(?y), friend(?x,?y) --> give(?y,?x)",
														Term.AXIOM);
				odeService.insertReasoningElement(fd);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the formula
				odeService.removeTerm(ONTOLOGY_NAME,FORMULA_NAME+i);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating formula: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_26.txt");
	}

	/**
	 *	Inserts several formulas in several ontologies.
	 *
	 *	The database must have NEW_FORMULAS ontologies named ONTOLOGY_NAMEx
	 */
	public static void benchmark1_1_27 (ODEService odeService) throws Exception
	{
		final int NEW_FORMULAS = benchmarkConstants.TEST_FORMULAS;
		final String FORMULA_NAME = "newFormula_1";
		final String ONTOLOGY_NAME = "ontology";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_FORMULAS);

		// Creates formulas one by one
		System.out.println("Creating formula: ");
		for (int i=1; i<=NEW_FORMULAS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+FORMULA_NAME+"("+i+")");

			try{
				// Creates the formula
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				FormulaDescriptor fd = new FormulaDescriptor(
													  ONTOLOGY_NAME+i,FORMULA_NAME, "formula of "+ONTOLOGY_NAME+i,
														"Have friends and they will give you...",
														"person(?x), person(?y), friend(?x,?y) --> give(?y,?x)",
														Term.AXIOM);
				odeService.insertReasoningElement(fd);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the formula
				odeService.removeTerm(ONTOLOGY_NAME+i,FORMULA_NAME);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating formula: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_27.txt");
	}

	/**
	 *	Relates several formulas to terms in an ontology.
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 *	with NEW_FORMULA_RELATIONS concepts named CONCEPT_NAMEx
	 *	with NEW_FORMULA_RELATIONS formulas named FORMULA_NAMEx
	 */
	public static void benchmark1_1_28 (ODEService odeService) throws Exception
	{
		final int NEW_FORMULA_RELATIONS = benchmarkConstants.TEST_FORMULA_RELATIONS;
		final String FORMULA_NAME = "formula";
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology0";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_FORMULA_RELATIONS);

		// Relate formulas one by one
		System.out.println("Relating formula: ");
		for (int i=1; i<=NEW_FORMULA_RELATIONS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+FORMULA_NAME+i);

			try{
				// Relates the formula
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				odeService.relateFormulaToTerm(ONTOLOGY_NAME,CONCEPT_NAME+i,FORMULA_NAME+i);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Unrelates the formula
				odeService.unrelateFormulaFromTerm(ONTOLOGY_NAME,CONCEPT_NAME+i,FORMULA_NAME+i);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR relating formula: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_28.txt");
	}

	/**
	 *	Relates several formulas to terms in several ontologies.
	 *
	 *	The database must have NEW_FORMULA_RELATIONS ontologies named ONTOLOGY_NAMEx
	 *	with a concept named CONCEPT_NAME
	 *	with a formula named FORMULA_NAME
	 */
	public static void benchmark1_1_29 (ODEService odeService) throws Exception
	{
		final int NEW_FORMULA_RELATIONS = benchmarkConstants.TEST_FORMULA_RELATIONS;
		final String FORMULA_NAME = "formula1";
		final String CONCEPT_NAME = "concept1";
		final String ONTOLOGY_NAME = "ontology";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_FORMULA_RELATIONS);

		// Relate formulas one by one
		System.out.println("Relating formula: ");
		for (int i=1; i<=NEW_FORMULA_RELATIONS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+FORMULA_NAME+"("+i+")");

			try{
				// Relates the formula
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				odeService.relateFormulaToTerm(ONTOLOGY_NAME+i,CONCEPT_NAME,FORMULA_NAME);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Unrelates the formula
				odeService.unrelateFormulaFromTerm(ONTOLOGY_NAME+i,CONCEPT_NAME,FORMULA_NAME);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR relating formula: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_29.txt");
	}

	/**
	 *	Inserts several instances in an ontology.
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 *	with an instance set named INSTANCE_SET_NAME
	 *	with a concept named CONCEPT_NAME
	 */
	public static void benchmark1_1_30 (ODEService odeService) throws Exception
	{
		final int NEW_INSTANCES = benchmarkConstants.TEST_INSTANCES;
		final String INSTANCE_NAME = "newInstance";
		final String INSTANCE_SET_NAME = "instanceSet1";
		final String CONCEPT_NAME = "concept1";
		final String ONTOLOGY_NAME = "ontology0";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_INSTANCES);

		// Creates instances one by one
		System.out.println("Creating instance: ");
		for (int i=1; i<=NEW_INSTANCES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+INSTANCE_NAME+i);

			try{
				// Creates the instance
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				Instance instance = new Instance(INSTANCE_NAME+i,CONCEPT_NAME,INSTANCE_SET_NAME,
																				 "instance "+i+" of "+ONTOLOGY_NAME);
				odeService.insertInstance(ONTOLOGY_NAME,instance);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the instance
				odeService.removeTerm(ONTOLOGY_NAME,INSTANCE_NAME+i,INSTANCE_SET_NAME);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating instance: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_30.txt");
	}

	/**
	 *	Inserts several instances in several ontologies.
	 *
	 *	The database must have NEW_INSTANCES ontologies named ONTOLOGY_NAMEx
	 *	with an instance set named INSTANCE_SET_NAME
	 *	with a concept named CONCEPT_NAME
	 */
	public static void benchmark1_1_31 (ODEService odeService) throws Exception
	{
		final int NEW_INSTANCES = benchmarkConstants.TEST_INSTANCES;
		final String INSTANCE_NAME = "newInstance_1";
		final String INSTANCE_SET_NAME = "instanceSet1";
		final String CONCEPT_NAME = "concept1";
		final String ONTOLOGY_NAME = "ontology";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_INSTANCES);

		// Creates instances one by one
		System.out.println("Creating instance: ");
		for (int i=1; i<=NEW_INSTANCES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+INSTANCE_NAME+"("+i+")");

			try{
				// Creates the instance
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				Instance instance = new Instance(INSTANCE_NAME,CONCEPT_NAME,INSTANCE_SET_NAME,
																				 "instance of "+ONTOLOGY_NAME+i);
				odeService.insertInstance(ONTOLOGY_NAME+i,instance);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the instance
				odeService.removeTerm(ONTOLOGY_NAME+i,INSTANCE_NAME,INSTANCE_SET_NAME);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating instance: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_31.txt");
	}

	/**
	 *	Inserts several relation instances in an ontology.
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 *	with an instance set named INSTANCE_SET_NAME
	 *	with NEW_RELATION_INSTANCES term relations named TERM_RELATION_NAMEx
	 *	with NEW_RELATION_INSTANCES+1 instances named INSTANCE_NAMEx
	 */
	public static void benchmark1_1_32 (ODEService odeService) throws Exception
	{
		final int NEW_RELATION_INSTANCES = benchmarkConstants.TEST_RELATION_INSTANCES;
		final String RELATION_INSTANCE_NAME = "newRelationInstance";
		final String TERM_RELATION_NAME = "termRelation";
		final String CONCEPT_NAME = "concept";
		final String INSTANCE_NAME = "instance";
		final String INSTANCE_SET_NAME = "instanceSet1";
		final String ONTOLOGY_NAME = "ontology0";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_RELATION_INSTANCES);

		// Creates relation instances one by one
		System.out.println("Creating relation instance:");
		for (int i=1; i<=NEW_RELATION_INSTANCES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+RELATION_INSTANCE_NAME+i+
												 "between "+INSTANCE_NAME+i+" and "+INSTANCE_NAME+(i+1));

			try{
				// Creates the relation instance
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				// Esto ha cambiado de la iteracin 3 a la 4
				//TermRelationInstance tri = new TermRelationInstance(
				//	RELATION_INSTANCE_NAME+i,TERM_RELATION_NAME+i,INSTANCE_SET_NAME,
				//	"instance of "+TERM_RELATION_NAME+i,
				//	INSTANCE_NAME+i,INSTANCE_NAME+(i+1));
				TermRelation tr = new TermRelation(ONTOLOGY_NAME,null,TERM_RELATION_NAME+i,
																					 CONCEPT_NAME+i,CONCEPT_NAME+(i+1));
				TermRelationInstance tri = new TermRelationInstance(
					RELATION_INSTANCE_NAME+i,tr,INSTANCE_SET_NAME,
					"instance of "+TERM_RELATION_NAME+i,
					INSTANCE_NAME+i,INSTANCE_NAME+(i+1));
				odeService.insertRelationInstance(ONTOLOGY_NAME,tri);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the relation instance
				odeService.removeTerm(ONTOLOGY_NAME,RELATION_INSTANCE_NAME+i,INSTANCE_SET_NAME);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating relation instance: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_32.txt");
	}

	/**
	 *	Inserts several relation instances in several ontologies.
	 *
	 *	The database must have NEW_RELATION_INSTANCES ontologies named ONTOLOGY_NAMEx
	 *	with an instance set named INSTANCE_SET_NAME
	 *	with a term relation named TERM_RELATION_NAME
	 *	with two instances named INSTANCE_NAME1 and INSTANCE_NAME2
	 */
	public static void benchmark1_1_33 (ODEService odeService) throws Exception
	{
		final int NEW_RELATION_INSTANCES = benchmarkConstants.TEST_RELATION_INSTANCES;
		final String RELATION_INSTANCE_NAME = "newRelationInstance_1";
		final String TERM_RELATION_NAME = "termRelation1";
		final String CONCEPT_NAME = "concept";
		final String INSTANCE_NAME = "instance";
		final String INSTANCE_SET_NAME = "instanceSet1";
		final String ONTOLOGY_NAME = "ontology";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_RELATION_INSTANCES);

		// Creates relation instances one by one
		System.out.println("Creating relation instance: ");
		for (int i=1; i<=NEW_RELATION_INSTANCES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+RELATION_INSTANCE_NAME+"("+i+")");

			try{
				// Creates the relation instance
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				// Esto ha cambiado de la iteracin 3 a la 4
				//TermRelationInstance tri = new TermRelationInstance(
				//	RELATION_INSTANCE_NAME,TERM_RELATION_NAME,INSTANCE_SET_NAME,
				//	"instance of "+TERM_RELATION_NAME,
				//	INSTANCE_NAME+"1",INSTANCE_NAME+"2");
				TermRelation tr = new TermRelation(ONTOLOGY_NAME+i,null,TERM_RELATION_NAME,
																					 CONCEPT_NAME+"1",CONCEPT_NAME+"2");
				TermRelationInstance tri = new TermRelationInstance(
					RELATION_INSTANCE_NAME,tr,INSTANCE_SET_NAME,
					"instance of "+TERM_RELATION_NAME,
					INSTANCE_NAME+"1",INSTANCE_NAME+"2");
				odeService.insertRelationInstance(ONTOLOGY_NAME+i,tri);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the relation instance
				odeService.removeTerm(ONTOLOGY_NAME+i,RELATION_INSTANCE_NAME,INSTANCE_SET_NAME);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating relation instance: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_33.txt");
	}

	/**
	 *	Inserts several instance values in an ontology.
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 *	with an instance set named INSTANCE_SET_NAME
	 *	with a concept named CONCEPT_NAME
	 *	with an instance named INSTANCE_NAME
	 *	with NEW_INSTANCE_VALUES instance attributes named INSTANCE_ATTRIBUTE_NAMEx
	 */
	public static void benchmark1_1_34 (ODEService odeService) throws Exception
	{
		final int NEW_INSTANCE_VALUES = benchmarkConstants.TEST_INSTANCE_VALUES;
		final String INSTANCE_VALUE_NAME = "newInstanceValue";
		final String INSTANCE_NAME = "instance1";
		final String INSTANCE_ATTRIBUTE_NAME = "instanceAttribute";
		final String INSTANCE_SET_NAME = "instanceSet1";
		final String CONCEPT_NAME = "concept1";
		final String ONTOLOGY_NAME = "ontology0";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_INSTANCE_VALUES);

		// Creates instance values one by one
		System.out.println("Creating instance value: ");
		for (int i=1; i<=NEW_INSTANCE_VALUES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+INSTANCE_VALUE_NAME+i+
												 " of attribute "+INSTANCE_ATTRIBUTE_NAME+i+
												 " of instance "+INSTANCE_NAME);

			try{
				// Creates the instance value
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				odeService.addValueToInstance(ONTOLOGY_NAME,INSTANCE_SET_NAME,
																			INSTANCE_NAME,CONCEPT_NAME,
																			INSTANCE_ATTRIBUTE_NAME+i,INSTANCE_VALUE_NAME+i);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the instance value
				odeService.removeValueFromInstance(ONTOLOGY_NAME,INSTANCE_SET_NAME,INSTANCE_NAME,
																					 CONCEPT_NAME,INSTANCE_ATTRIBUTE_NAME+i,
																					 INSTANCE_VALUE_NAME+i);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating instance value: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_34.txt");
	}

	/**
	 *	Inserts several instance values in several ontologies.
	 *
	 *	The database must have NEW_INSTANCE_VALUES ontologies named ONTOLOGY_NAMEx
	 *	with an instance set named INSTANCE_SET_NAME
	 *	with a concept named CONCEPT_NAME
	 *	with an instance named INSTANCE_NAME
	 *	with an instance attribute named INSTANCE_ATTRIBUTE_NAME
	 */
	public static void benchmark1_1_35 (ODEService odeService) throws Exception
	{
		final int NEW_INSTANCE_VALUES = benchmarkConstants.TEST_INSTANCE_VALUES;
		final String INSTANCE_VALUE_NAME = "newInstanceValue_1";
		final String INSTANCE_NAME = "instance0";
		final String INSTANCE_ATTRIBUTE_NAME = "instanceAttribute0";
		final String INSTANCE_SET_NAME = "instanceSet1";
		final String CONCEPT_NAME = "concept0";
		final String ONTOLOGY_NAME = "ontology";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_INSTANCE_VALUES);

		// Creates instance values one by one
		System.out.println("Creating instance value: ");
		for (int i=1; i<=NEW_INSTANCE_VALUES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+INSTANCE_VALUE_NAME+
												 " of attribute "+INSTANCE_ATTRIBUTE_NAME+
												 " of instance "+INSTANCE_NAME+"("+i+")");

			try{
				// Creates the instance value
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				odeService.addValueToInstance(ONTOLOGY_NAME+i,INSTANCE_SET_NAME,
																			INSTANCE_NAME,CONCEPT_NAME,
																			INSTANCE_ATTRIBUTE_NAME,INSTANCE_VALUE_NAME);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the instance value
				odeService.removeValueFromInstance(ONTOLOGY_NAME+i,INSTANCE_SET_NAME,
																					 INSTANCE_NAME,CONCEPT_NAME,
																					 INSTANCE_ATTRIBUTE_NAME,INSTANCE_VALUE_NAME);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating instance value: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_35.txt");
	}

	/**
	 *	Inserts several groups in an ontology.
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 *	with NEW_GROUPS concepts named CONCEPT_NAMEx
	 */
	public static void benchmark1_1_36 (ODEService odeService) throws Exception
	{
		final int NEW_GROUPS = benchmarkConstants.TEST_GROUPS;
		final String GROUP_NAME = "newGroup";
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology0";

		String[] concept_list = new String[1];

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_GROUPS);

		// Creates groups one by one
		System.out.println("Creating group: ");
		for (int i=1; i<=NEW_GROUPS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+GROUP_NAME+i);

			try{
				// Creates the group
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				concept_list[0] = CONCEPT_NAME+i;
				Group group = new Group(GROUP_NAME+i,"group "+i+" of "+ONTOLOGY_NAME,concept_list);
				odeService.addGroup(ONTOLOGY_NAME,group);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the group
				odeService.removeTerm(ONTOLOGY_NAME,GROUP_NAME+i);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating group: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_36.txt");
	}

	/**
	 *	Inserts several groups in several ontologies.
	 *
	 *	The database must have NEW_GROUPS ontologies named ONTOLOGY_NAMEx
	 *	with a concept named CONCEPT_NAME
	 */
	public static void benchmark1_1_37 (ODEService odeService) throws Exception
	{
		final int NEW_GROUPS = benchmarkConstants.TEST_GROUPS;
		final String GROUP_NAME = "newGroup_1";
		final String CONCEPT_NAME = "concept1";
		final String ONTOLOGY_NAME = "ontology";

		String[] concept_list = new String[1];

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_GROUPS);

		// Creates groups one by one
		System.out.println("Creating group: ");
		for (int i=1; i<=NEW_GROUPS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+GROUP_NAME+"("+i+")");

			try{
				// Creates the group
				if (i>benchmarkConstants.warming){
  				timer.start();
  			}
				concept_list[0] = CONCEPT_NAME;
				Group group = new Group(GROUP_NAME,"group of "+ONTOLOGY_NAME+i,concept_list);
				odeService.addGroup(ONTOLOGY_NAME+i,group);
				if (i>benchmarkConstants.warming){
  				timer.stop();
  			}

				if (benchmarkConstants.restore){
				// Removes the group
				odeService.removeTerm(ONTOLOGY_NAME+i,GROUP_NAME);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating group: "+e.getMessage());
			}
		}
		System.out.println("");

		benchmarkGeneral.save_timer_data(timer,"benchmark1_1_37.txt");
	}

}
