package es.upm.fi.dia.ontology.webode.wpbs;

// Java stuff
import java.util.Calendar;
import java.util.Date;

import es.upm.fi.dia.ontology.webode.service.ODEService;
import es.upm.fi.dia.ontology.webode.service.OntologyDescriptor;
import es.upm.fi.dia.ontology.webode.service.ReferenceDescriptor;
import es.upm.fi.dia.ontology.webode.service.SynonymAbbreviation;
import es.upm.fi.dia.ontology.webode.service.Term;
import es.upm.fi.dia.ontology.webode.service.TermRelation;
import es.upm.fi.dia.ontology.webode.service.TermTypes;
import es.upm.fi.dia.ontology.webode.service.WebODEException;

/** Implements a set of benchmarks for testing ODE service.
 *	All the benchmarks in this class test basic operations regarding 
 *	removing values in the database.
 */
public class benchmarkBasicRemove
{

	/**
	 *	Removes several ontologies
	 *
	 *	The database must have REMOVED_ONTOLOGIES ontologies named ONTOLOGY_NAMEx
	 */
	public static void benchmark1_3_01 (ODEService odeService) throws Exception
	{
		final int REMOVED_ONTOLOGIES = benchmarkConstants.TEST_ONTOLOGIES;
		String ONTOLOGY_NAME = "ontology";

		if (benchmarkConstants.restore){
			ONTOLOGY_NAME = "newOntology";
		}

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(REMOVED_ONTOLOGIES);
		
		// Removes ontologies one by one
		System.out.println("Removing ontology: ");
		for (int i=1; i<=REMOVED_ONTOLOGIES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+ONTOLOGY_NAME+i);
				
			try{
				if (benchmarkConstants.restore){
				// Creates the ontology
				Date now=Calendar.getInstance().getTime();
				OntologyDescriptor od = new OntologyDescriptor(ONTOLOGY_NAME+i,"webode://webode.dia.fi.upm.es/"+ONTOLOGY_NAME+i+"#","new ontology "+i,
																										 	 "admin","admin",now,now);
				odeService.createOntology(od);
				}

				// Removes the ontology
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				odeService.removeOntology(ONTOLOGY_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR removing ontology: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_3_01.txt");
	}

	/**
	 *	Unrelates several term references of concepts of an ontology
	 *
	 *	The database must have one ontology named ONTOLOGY_NAME
	 *	with REMOVED_TERM_REFERENCES term references named TERM_REFERENCE_NAMEx
	 *	related to REMOVED_TERM_REFERENCES concepts named CONCEPT_NAMEx
	 */
	public static void benchmark1_3_02 (ODEService odeService) throws Exception
	{
		final int REMOVED_TERM_REFERENCES = benchmarkConstants.TEST_TERM_REFERENCE_RELATIONS;
		final String TERM_REFERENCE_NAME = "termReference";
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology0";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(REMOVED_TERM_REFERENCES);
		
		// Unrelates term references one by one
		System.out.println("Unrelating term reference: ");
		for (int i=1; i<=REMOVED_TERM_REFERENCES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+TERM_REFERENCE_NAME+i);
				
			try{
				// Unrelates the term reference
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				odeService.unrelateReferenceToTerm(ONTOLOGY_NAME,CONCEPT_NAME+(i-1),TERM_REFERENCE_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }

				if (benchmarkConstants.restore){
				// Relates the term reference
				odeService.relateReferenceToTerm(ONTOLOGY_NAME,CONCEPT_NAME+(i-1),TERM_REFERENCE_NAME+i);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR unrelating term reference: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_3_02.txt");
	}

	/**
	 *	Unrelates several term references of concepts of several ontologies
	 *
	 *	The database must have REMOVED_TERM_REFERENCES ontologies named ONTOLOGY_NAMEx
	 *	with a term reference named TERM_REFERENCE_NAME
	 *	related to a concept named CONCEPT_NAME
	 */
	public static void benchmark1_3_03 (ODEService odeService) throws Exception
	{
		final int REMOVED_TERM_REFERENCES = benchmarkConstants.TEST_TERM_REFERENCE_RELATIONS;
		final String TERM_REFERENCE_NAME = "termReference2";
		final String CONCEPT_NAME = "concept2";
		final String ONTOLOGY_NAME = "ontology";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(REMOVED_TERM_REFERENCES);
		
		// Unrelates term references one by one
		System.out.println("Unrelating term reference: ");
		for (int i=1; i<=REMOVED_TERM_REFERENCES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+TERM_REFERENCE_NAME+"("+i+")");
				
			try{
				// Unrelates the term reference
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				odeService.unrelateReferenceToTerm(ONTOLOGY_NAME+i,CONCEPT_NAME,TERM_REFERENCE_NAME);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }

				if (benchmarkConstants.restore){
				// Relates the term reference
				odeService.relateReferenceToTerm(ONTOLOGY_NAME+i,CONCEPT_NAME,TERM_REFERENCE_NAME);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR unrelating term reference: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_3_03.txt");
	}

	/**
	 *	Removes several term references from an ontology
	 *
	 *	The database must have one ontology named ONTOLOGY_NAME
	 *	with REMOVED_TERM_REFERENCES term references named TERM_REFERENCE_NAMEx
	 */
	public static void benchmark1_3_04 (ODEService odeService) throws Exception
	{
		final int REMOVED_TERM_REFERENCES = benchmarkConstants.TEST_TERM_REFERENCES;
		String TERM_REFERENCE_NAME = "termReference";
		final String ONTOLOGY_NAME = "ontology0";

		if (benchmarkConstants.restore){
			TERM_REFERENCE_NAME = "newTermReference";
		}

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(REMOVED_TERM_REFERENCES);
		
		// Removes term references one by one
		System.out.println("Removing term reference: ");
		for (int i=1; i<=REMOVED_TERM_REFERENCES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+TERM_REFERENCE_NAME+i);
				
			try{
				if (benchmarkConstants.restore){
				// Inserts the term reference
				ReferenceDescriptor rd = new ReferenceDescriptor(TERM_REFERENCE_NAME+i,
																												 "reference "+i);
				odeService.addReference(ONTOLOGY_NAME,rd);
				}

				// Removes the term reference
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				odeService.removeReference(ONTOLOGY_NAME,TERM_REFERENCE_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR removing term reference: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_3_04.txt");
	}

	/**
	 *	Removes several term references from several ontologies
	 *
	 *	The database must have REMOVED_TERM_REFERENCES ontologies named ONTOLOGY_NAMEx
	 *	with a term reference named TERM_REFERENCE_NAME
	 */
	public static void benchmark1_3_05 (ODEService odeService) throws Exception
	{
		final int REMOVED_TERM_REFERENCES = benchmarkConstants.TEST_TERM_REFERENCES;
		String TERM_REFERENCE_NAME = "termReference1";
		final String ONTOLOGY_NAME = "ontology";

		if (benchmarkConstants.restore){
			TERM_REFERENCE_NAME = "newTermReference1";
		}

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(REMOVED_TERM_REFERENCES);
		
		// Removes term references one by one
		System.out.println("Removing term reference: ");
		for (int i=1; i<=REMOVED_TERM_REFERENCES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+TERM_REFERENCE_NAME+"("+i+")");
				
			try{
				if (benchmarkConstants.restore){
				// Inserts the term reference
				ReferenceDescriptor rd = new ReferenceDescriptor(TERM_REFERENCE_NAME,
																												 "reference "+i);
				odeService.addReference(ONTOLOGY_NAME+i,rd);
				}

				// Removes the term reference
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				odeService.removeReference(ONTOLOGY_NAME+i,TERM_REFERENCE_NAME);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR removing term reference: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_3_05.txt");
	}

	/**
	 *	Removes several ontology references from an ontology
	 *
	 *	The database must have one ontology named ONTOLOGY_NAME
	 *	with REMOVED_ONTOLOGY_REFERENCES ontology references named ONTOLOGY_REFERENCE_NAMEx
	 */
	public static void benchmark1_3_06 (ODEService odeService) throws Exception
	{
		final int REMOVED_ONTOLOGY_REFERENCES = benchmarkConstants.TEST_ONTOLOGY_REFERENCES;
		String ONTOLOGY_REFERENCE_NAME = "ontologyReference";
		final String ONTOLOGY_NAME = "ontology0";

		if (benchmarkConstants.restore){
			ONTOLOGY_REFERENCE_NAME = "newOntologyReference";
		}

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(REMOVED_ONTOLOGY_REFERENCES);
		
		// Removes ontology references one by one
		System.out.println("Removing ontology reference: ");
		for (int i=1; i<=REMOVED_ONTOLOGY_REFERENCES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+ONTOLOGY_REFERENCE_NAME+i);
				
			try{
				if (benchmarkConstants.restore){
				// Inserts the ontology reference
				ReferenceDescriptor rd = new ReferenceDescriptor(ONTOLOGY_REFERENCE_NAME+i,
																												 "reference of "+ONTOLOGY_NAME);
				odeService.addReferenceToOntology(ONTOLOGY_NAME,rd);
				}

				// Removes the ontology reference
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				odeService.removeReference(ONTOLOGY_NAME,ONTOLOGY_REFERENCE_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR removing ontology reference: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_3_06.txt");
	}

	/**
	 *	Removes several ontology references from several ontologies
	 *
	 *	The database must have REMOVED_ONTOLOGY_REFERENCES ontologies named ONTOLOGY_NAMEx
	 *	with an ontology reference named ONTOLOGY_REFERENCE_NAME
	 */
	public static void benchmark1_3_07 (ODEService odeService) throws Exception
	{
		final int REMOVED_ONTOLOGY_REFERENCES = benchmarkConstants.TEST_ONTOLOGY_REFERENCES;
		String ONTOLOGY_REFERENCE_NAME = "ontologyReference1";
		final String ONTOLOGY_NAME = "ontology";

		if (benchmarkConstants.restore){
			ONTOLOGY_REFERENCE_NAME = "newOntologyReference1";
		}

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(REMOVED_ONTOLOGY_REFERENCES);
		
		// Removes ontology references one by one
		System.out.println("Removing ontology reference: ");
		for (int i=1; i<=REMOVED_ONTOLOGY_REFERENCES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+ONTOLOGY_REFERENCE_NAME+"("+i+")");
				
			try{
				if (benchmarkConstants.restore){
				// Inserts the ontology reference
				ReferenceDescriptor rd = new ReferenceDescriptor(ONTOLOGY_REFERENCE_NAME,
																												 "reference of "+ONTOLOGY_NAME+i);
				odeService.addReferenceToOntology(ONTOLOGY_NAME+i,rd);
				}

				// Removes the ontology reference
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				odeService.removeReference(ONTOLOGY_NAME+i,ONTOLOGY_REFERENCE_NAME);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR removing ontology reference: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_3_07.txt");
	}

	/**
	 *	Removes several term relations from an ontology
	 *
	 *	The database must have one ontology named ONTOLOGY_NAME
	 *	with REMOVED_TERM_RELATIONS+1 concepts named CONCEPT_NAMEx
	 *	with REMOVED_TERM_RELATIONS term relations named TERM_RELATION_NAMEx
	 */
	public static void benchmark1_3_08 (ODEService odeService) throws Exception
	{
		final int REMOVED_TERM_RELATIONS = benchmarkConstants.TEST_TERM_RELATIONS;
		String TERM_RELATION_NAME = "termRelation";
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology0";

		if (benchmarkConstants.restore){
			TERM_RELATION_NAME = "newTermRelation";
		}

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(REMOVED_TERM_RELATIONS);
		
		// Removes term relations one by one
		System.out.println("Removing term relation: ");
		for (int i=1; i<=REMOVED_TERM_RELATIONS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+TERM_RELATION_NAME+i);
				
			try{
				if (benchmarkConstants.restore){
				// Inserts the term relation
				TermRelation tr = new TermRelation(ONTOLOGY_NAME,null,TERM_RELATION_NAME+i,
																					 CONCEPT_NAME+i,CONCEPT_NAME+(i+1));
				odeService.insertTermRelation(tr);
				}

				// Removes the term relation
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				TermRelation[] trs = odeService.getTermRelations(ONTOLOGY_NAME,CONCEPT_NAME+i,false);
				int j=0;
				while (!(trs[j].name.equals(TERM_RELATION_NAME+i)) && (j<trs.length)){
					j++;
				}
				odeService.removeTermRelation(trs[j]);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR removing term relation: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_3_08.txt");
	}

	/**
	 *	Removes several term relations from several ontologies
	 *
	 *	The database must have REMOVED_TERM_RELATIONS ontologies named ONTOLOGY_NAMEx
	 *	with two concepts named CONCEPT_NAME1 and CONCEPT_NAME2
	 *	with a term relation named TERM_RELATION_NAME
	 */
	public static void benchmark1_3_09 (ODEService odeService) throws Exception
	{
		final int REMOVED_TERM_RELATIONS = benchmarkConstants.TEST_TERM_RELATIONS;
		String TERM_RELATION_NAME = "termRelation1";
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology";

		if (benchmarkConstants.restore){
			TERM_RELATION_NAME = "newTermRelation1";
		}

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(REMOVED_TERM_RELATIONS);
		
		// Removes term relations one by one
		System.out.println("Removing term relation: ");
		for (int i=1; i<=REMOVED_TERM_RELATIONS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+TERM_RELATION_NAME);
				
			try{
				if (benchmarkConstants.restore){
				// Inserts the term relation
				TermRelation tr = new TermRelation(ONTOLOGY_NAME+i,null,TERM_RELATION_NAME,
																					 CONCEPT_NAME+"1",CONCEPT_NAME+"2");
				odeService.insertTermRelation(tr);
				}

				// Removes the term relation
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				TermRelation[] trs = odeService.getTermRelations(ONTOLOGY_NAME+i,CONCEPT_NAME+"1",false);
				int j=0;
				while (!(trs[j].name.equals(TERM_RELATION_NAME)) && (j<trs.length)){
					j++;
				}
				odeService.removeTermRelation(trs[j]);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR removing term relation: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_3_09.txt");
	}

	/**
	 *	Removes several values from a class attribute
	 *
	 *	The database must have one ontology named ONTOLOGY_NAME
	 *	with a concept named CONCEPT_NAME
	 *	with a class attribute named CLASS_ATTRIBUTE_NAME
	 *	with REMOVED_CLASS_ATTRIBUTE_VALUES class attribute values named CLASS_ATTRIBUTE_VALUE_NAMEx
	 */
	public static void benchmark1_3_10 (ODEService odeService) throws Exception
	{
		final int REMOVED_CLASS_ATTRIBUTE_VALUES = benchmarkConstants.TEST_CLASS_ATTRIBUTE_VALUES;
		String CLASS_ATTRIBUTE_VALUE_NAME = "classAttributeValue";
		final String CLASS_ATTRIBUTE_NAME = "classAttribute0";
		final String CONCEPT_NAME = "concept0";
		final String ONTOLOGY_NAME = "ontology0";

		if (benchmarkConstants.restore){
			CLASS_ATTRIBUTE_VALUE_NAME = "newClassAttributeValue";
		}

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(REMOVED_CLASS_ATTRIBUTE_VALUES);
		
		// Removes class attribute values one by one
		System.out.println("Removing class attribute value: ");
		for (int i=1; i<=REMOVED_CLASS_ATTRIBUTE_VALUES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CLASS_ATTRIBUTE_VALUE_NAME+i);
				
			try{
				if (benchmarkConstants.restore){
				// Inserts the class attribute value
				odeService.addValueToClassAttribute(ONTOLOGY_NAME,CLASS_ATTRIBUTE_NAME,
																						CONCEPT_NAME,CLASS_ATTRIBUTE_VALUE_NAME+i);
				}

				// Removes the class attribute value
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				odeService.removeValueFromClassAttribute(ONTOLOGY_NAME,CLASS_ATTRIBUTE_NAME,
																								 CONCEPT_NAME,CLASS_ATTRIBUTE_VALUE_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR removing class attribute value: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_3_10.txt");
	}

	/**
	 *	Removes several values from several class attributes
	 *
	 *	The database must have one ontology named ONTOLOGY_NAME
	 *	with a concept named CONCEPT_NAME
	 *	with REMOVED_CLASS_ATTRIBUTE_VALUES class attributes named CLASS_ATTRIBUTE_NAMEx
	 *	with a class attribute value named CLASS_ATTRIBUTE_VALUE_NAME
	 */
	public static void benchmark1_3_11 (ODEService odeService) throws Exception
	{
		final int REMOVED_CLASS_ATTRIBUTE_VALUES = benchmarkConstants.TEST_CLASS_ATTRIBUTE_VALUES;
		String CLASS_ATTRIBUTE_VALUE_NAME = "classAttributeValue1";
		final String CLASS_ATTRIBUTE_NAME = "classAttribute";
		final String CONCEPT_NAME = "concept0";
		final String ONTOLOGY_NAME = "ontology0";

		if (benchmarkConstants.restore){
			CLASS_ATTRIBUTE_VALUE_NAME = "newClassAttributeValue1";
		}

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(REMOVED_CLASS_ATTRIBUTE_VALUES);
		
		// Removes class attribute values one by one
		System.out.println("Removing class attribute value: ");
		for (int i=1; i<=REMOVED_CLASS_ATTRIBUTE_VALUES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CLASS_ATTRIBUTE_VALUE_NAME+"("+i+")");
				
			try{
				if (benchmarkConstants.restore){
				// Inserts the class attribute value
				odeService.addValueToClassAttribute(ONTOLOGY_NAME,CLASS_ATTRIBUTE_NAME+i,
																						CONCEPT_NAME,CLASS_ATTRIBUTE_VALUE_NAME);
				}

				// Removes the class attribute value
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				odeService.removeValueFromClassAttribute(ONTOLOGY_NAME,CLASS_ATTRIBUTE_NAME+i,
																								 CONCEPT_NAME,CLASS_ATTRIBUTE_VALUE_NAME);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR removing class attribute value: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_3_11.txt");
	}

	/**
	 *	Removes several values from an instance attribute
	 *
	 *	The database must have one ontology named ONTOLOGY_NAME
	 *	with a concept named CONCEPT_NAME
	 *	with an instance attribute named INSTANCE_ATTRIBUTE_NAME
	 *	with REMOVED_INSTANCE_ATTRIBUTE_VALUES instance attribute values named INSTANCE_ATTRIBUTE_VALUE_NAMEx
	 */
	public static void benchmark1_3_12 (ODEService odeService) throws Exception
	{
		final int REMOVED_INSTANCE_ATTRIBUTE_VALUES = benchmarkConstants.TEST_INSTANCE_ATTRIBUTE_VALUES;
		String INSTANCE_ATTRIBUTE_VALUE_NAME = "instanceAttributeValue";
		final String INSTANCE_ATTRIBUTE_NAME = "instanceAttribute0";
		final String CONCEPT_NAME = "concept0";
		final String ONTOLOGY_NAME = "ontology0";

		if (benchmarkConstants.restore){
			INSTANCE_ATTRIBUTE_VALUE_NAME = "newInstanceAttributeValue";
		}

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(REMOVED_INSTANCE_ATTRIBUTE_VALUES);
		
		// Removes instance attribute values one by one
		System.out.println("Removing instance attribute value: ");
		for (int i=1; i<=REMOVED_INSTANCE_ATTRIBUTE_VALUES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+INSTANCE_ATTRIBUTE_VALUE_NAME+i);
				
			try{
				if (benchmarkConstants.restore){
				// Inserts the instance attribute value
				odeService.addValueToInstanceAttribute(ONTOLOGY_NAME,INSTANCE_ATTRIBUTE_NAME,
																						   CONCEPT_NAME,INSTANCE_ATTRIBUTE_VALUE_NAME+i);
				}

				// Removes the instance attribute value
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				odeService.removeValueFromInstanceAttribute(ONTOLOGY_NAME,INSTANCE_ATTRIBUTE_NAME,
																									  CONCEPT_NAME,INSTANCE_ATTRIBUTE_VALUE_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR removing instance attribute value: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_3_12.txt");
	}

	/**
	 *	Removes several values from several instance attributes
	 *
	 *	The database must have one ontology named ONTOLOGY_NAME
	 *	with a concept named CONCEPT_NAME
	 *	with REMOVED_INSTANCE_ATTRIBUTE_VALUES instance attributes named INSTANCE_ATTRIBUTE_NAMEx
	 *	with an instance attribute value named INSTANCE_ATTRIBUTE_VALUE_NAME
	 */
	public static void benchmark1_3_13 (ODEService odeService) throws Exception
	{
		final int REMOVED_INSTANCE_ATTRIBUTE_VALUES = benchmarkConstants.TEST_INSTANCE_ATTRIBUTE_VALUES;
		String INSTANCE_ATTRIBUTE_VALUE_NAME = "instanceAttributeValue1";
		final String INSTANCE_ATTRIBUTE_NAME = "instanceAttribute";
		final String CONCEPT_NAME = "concept0";
		final String ONTOLOGY_NAME = "ontology0";

		if (benchmarkConstants.restore){
			INSTANCE_ATTRIBUTE_VALUE_NAME = "newInstanceAttributeValue1";
		}

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(REMOVED_INSTANCE_ATTRIBUTE_VALUES);
		
		// Removes instance attribute values one by one
		System.out.println("Removing instance attribute value: ");
		for (int i=1; i<=REMOVED_INSTANCE_ATTRIBUTE_VALUES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+INSTANCE_ATTRIBUTE_VALUE_NAME+"("+i+")");
					
			try{
				if (benchmarkConstants.restore){
				// Inserts the instance attribute value
				odeService.addValueToInstanceAttribute(ONTOLOGY_NAME,INSTANCE_ATTRIBUTE_NAME+i,
																						   CONCEPT_NAME,INSTANCE_ATTRIBUTE_VALUE_NAME);
				}

				// Removes the instance attribute value
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				odeService.removeValueFromInstanceAttribute(ONTOLOGY_NAME,INSTANCE_ATTRIBUTE_NAME+i,
																									  CONCEPT_NAME,INSTANCE_ATTRIBUTE_VALUE_NAME);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR removing instance attribute value: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_3_13.txt");
	}

	/**
	 *	Removes several synonyms from a term
	 *
	 *	The database must have one ontology named ONTOLOGY_NAME
	 *	with a concept named CONCEPT_NAME
	 *	with REMOVED_SYNONYMS synonyms named SYNONYM_NAMEx
	 */
	public static void benchmark1_3_14 (ODEService odeService) throws Exception
	{
		final int REMOVED_SYNONYMS = benchmarkConstants.TEST_SYNONYMS;
		String SYNONYM_NAME = "synonym";
		final String CONCEPT_NAME = "concept0";
		final String ONTOLOGY_NAME = "ontology0";

		if (benchmarkConstants.restore){
			SYNONYM_NAME = "newSynonym";
		}

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(REMOVED_SYNONYMS);
		
		// Removes synonyms one by one
		System.out.println("Removing synonym: ");
		for (int i=1; i<=REMOVED_SYNONYMS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+SYNONYM_NAME+i+" of "+CONCEPT_NAME);
				
			try{
				if (benchmarkConstants.restore){
				// Inserts the synonym
				SynonymAbbreviation sa = new SynonymAbbreviation(
																			SYNONYM_NAME+i, CONCEPT_NAME, null,    
																			TermTypes.SYNONYM, "synonym of "+CONCEPT_NAME);
				odeService.addSynonymToTerm(ONTOLOGY_NAME,sa);
				}
				
				// Removes the synonym
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				odeService.deleteSynonym(ONTOLOGY_NAME,SYNONYM_NAME+i,CONCEPT_NAME,null);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR removing synonym: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_3_14.txt");
	}

	/**
	 *	Removes several synonyms from several terms
	 *
	 *	The database must have one ontology named ONTOLOGY_NAME
	 *	with REMOVED_SYNONYMS concepts named CONCEPT_NAMEx
	 *	with a synonym named SYNONYM_NAME
	 */
	public static void benchmark1_3_15 (ODEService odeService) throws Exception
	{
		final int REMOVED_SYNONYMS = benchmarkConstants.TEST_SYNONYMS;
		String SYNONYM_NAME = "synonym1";
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology0";

		if (benchmarkConstants.restore){
			SYNONYM_NAME = "newSynonym1";
		}

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(REMOVED_SYNONYMS);
		
		// Removes synonyms one by one
		System.out.println("Removing synonym: ");
		for (int i=1; i<=REMOVED_SYNONYMS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+SYNONYM_NAME+" of "+CONCEPT_NAME+i);
				
			try{
				if (benchmarkConstants.restore){
				// Inserts the synonym
				SynonymAbbreviation sa = new SynonymAbbreviation(
																			SYNONYM_NAME, CONCEPT_NAME+i, null,    
																			TermTypes.SYNONYM, "synonym of "+CONCEPT_NAME+i);
				odeService.addSynonymToTerm(ONTOLOGY_NAME,sa);
				}

				// Removes the synonym
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				odeService.deleteSynonym(ONTOLOGY_NAME,SYNONYM_NAME,CONCEPT_NAME+i,null);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR removing synonym: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_3_15.txt");
	}

	/**
	 *	Unrelates several formulas with terms of an ontology
	 *
	 *	The database must have one ontology named ONTOLOGY_NAME
	 *	with REMOVED_FORMULAS formulas named FORMULA_NAMEx
	 *	related to REMOVED_FORMULAS concepts named CONCEPT_NAMEx
	 */
	public static void benchmark1_3_16 (ODEService odeService) throws Exception
	{
		final int REMOVED_FORMULAS = benchmarkConstants.TEST_FORMULA_RELATIONS;
		final String FORMULA_NAME = "formula";
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology0";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(REMOVED_FORMULAS);
		
		// Unrelates formulas one by one
		System.out.println("Unrelating formula: ");
		for (int i=1; i<=REMOVED_FORMULAS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+FORMULA_NAME+i+" with "+CONCEPT_NAME+i);
				
			try{
				// Unrelates the formula
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				odeService.unrelateFormulaFromTerm(ONTOLOGY_NAME,CONCEPT_NAME+(i-1),FORMULA_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }

				if (benchmarkConstants.restore){
				// Relates the formula
				odeService.relateFormulaToTerm(ONTOLOGY_NAME,CONCEPT_NAME+(i-1),FORMULA_NAME+i);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR unrelating formula: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_3_16.txt");
	}

	/**
	 *	Unrelates several formulas with terms of several ontologies
	 *
	 *	The database must have REMOVED_FORMULAS ontologies named ONTOLOGY_NAMEx
	 *	with a formula named FORMULA_NAME
	 *	related to a concept named CONCEPT_NAME
	 */
	public static void benchmark1_3_17 (ODEService odeService) throws Exception
	{
		final int REMOVED_FORMULAS = benchmarkConstants.TEST_FORMULA_RELATIONS;
		final String FORMULA_NAME = "formula2";
		final String CONCEPT_NAME = "concept2";
		final String ONTOLOGY_NAME = "ontology";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(REMOVED_FORMULAS);
		
		// Unrelates formulas one by one
		System.out.println("Unrelating formula: ");
		for (int i=1; i<=REMOVED_FORMULAS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+FORMULA_NAME+" with "+CONCEPT_NAME+"("+i+")");
				
			try{
				// Unrelates the formula
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				odeService.unrelateFormulaFromTerm(ONTOLOGY_NAME+i,CONCEPT_NAME,FORMULA_NAME);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }

				if (benchmarkConstants.restore){
				// Relates the formula
				odeService.relateFormulaToTerm(ONTOLOGY_NAME+i,CONCEPT_NAME,FORMULA_NAME);
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR unrelating formula: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_3_17.txt");
	}

	/**
	 *	Removes several instance values from an ontology
	 *
	 *	The database must have one ontology named ONTOLOGY_NAME
	 *	with an instance set named INSTANCE_SET_NAME
	 *	with a concept named CONCEPT_NAME
	 *	with an instance named INSTANCE_NAME
	 *	with REMOVED_INSTANCE_VALUES instance attributes named INSTANCE_ATTRIBUTRE_NAMEx
	 *	with REMOVED_INSTANCE_VALUES instance values named INSTANCE_VALUE_NAMEx
	 */
	public static void benchmark1_3_18 (ODEService odeService) throws Exception
	{
		final int REMOVED_INSTANCE_VALUES = benchmarkConstants.TEST_INSTANCE_VALUES;
		String INSTANCE_VALUE_NAME = "instanceValue";
		final String INSTANCE_NAME = "instance0";
		final String INSTANCE_ATTRIBUTE_NAME = "instanceAttribute";
		final String INSTANCE_SET_NAME = "instanceSet1";
		final String CONCEPT_NAME = "concept0";
		final String ONTOLOGY_NAME = "ontology0";

		if (benchmarkConstants.restore){
			INSTANCE_VALUE_NAME = "newInstanceValue";
		}

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(REMOVED_INSTANCE_VALUES);
		
		// Removes instance values one by one
		System.out.println("Removing instance value: ");
		for (int i=1; i<=REMOVED_INSTANCE_VALUES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+INSTANCE_VALUE_NAME+i);
				
			try{
				if (benchmarkConstants.restore){
				// Inserts the instance value
				odeService.addValueToInstance(ONTOLOGY_NAME,INSTANCE_SET_NAME,
																			INSTANCE_NAME,CONCEPT_NAME,
																			INSTANCE_ATTRIBUTE_NAME+i,INSTANCE_VALUE_NAME+i);
				}

				// Removes the instance value
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				odeService.removeValueFromInstance(ONTOLOGY_NAME,INSTANCE_SET_NAME,
																					 INSTANCE_NAME,CONCEPT_NAME,
																					 INSTANCE_ATTRIBUTE_NAME+i,INSTANCE_VALUE_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR removing instance value: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_3_18.txt");
	}

	/**
	 *	Removes several instance values from several ontologies
	 *
	 *	The database must have REMOVED_INSTANCE_VALUES ontologies named ONTOLOGY_NAMEx
	 *	with an instance set named INSTANCE_SET_NAME
	 *	with a concept named CONCEPT_NAME
	 *	with an instance named INSTANCE_NAME
	 *	with a instance attribute named INSTANCE_ATTRIBUTRE_NAME
	 *	with a instance value named INSTANCE_VALUE_NAME
	 */
	public static void benchmark1_3_19 (ODEService odeService) throws Exception
	{
		final int REMOVED_INSTANCE_VALUES = benchmarkConstants.TEST_INSTANCE_VALUES;
		String INSTANCE_VALUE_NAME = "instanceValue1";
		final String INSTANCE_NAME = "instance0";
		final String INSTANCE_ATTRIBUTE_NAME = "instanceAttribute0";
		final String INSTANCE_SET_NAME = "instanceSet1";
		final String CONCEPT_NAME = "concept0";
		final String ONTOLOGY_NAME = "ontology";

		if (benchmarkConstants.restore){
			INSTANCE_VALUE_NAME = "newInstanceValue1";
		}

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(REMOVED_INSTANCE_VALUES);
		
		// Removes instance values one by one
		System.out.println("Removing instance value: ");
		for (int i=1; i<=REMOVED_INSTANCE_VALUES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+INSTANCE_VALUE_NAME+"("+i+")");
				
			try{
				if (benchmarkConstants.restore){
				// Inserts the instance value
				odeService.addValueToInstance(ONTOLOGY_NAME+i,INSTANCE_SET_NAME,
																			INSTANCE_NAME,CONCEPT_NAME,
																			INSTANCE_ATTRIBUTE_NAME,INSTANCE_VALUE_NAME);
				}

				// Removes the instance value
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				odeService.removeValueFromInstance(ONTOLOGY_NAME+i,INSTANCE_SET_NAME,
																					 INSTANCE_NAME,CONCEPT_NAME,
																					 INSTANCE_ATTRIBUTE_NAME,INSTANCE_VALUE_NAME);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR removing instance value: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_3_19.txt");
	}

	/**
	 *	Removes several concepts from an ontology
	 *
	 *	The database must have one ontology named ONTOLOGY_NAME
	 *	with REMOVED_CONCEPTS concepts named CONCEPT_NAMEx
	 */
	public static void benchmark1_3_20 (ODEService odeService) throws Exception
	{
		final int REMOVED_CONCEPTS = benchmarkConstants.TEST_CONCEPTS;
		String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology0";

		if (benchmarkConstants.restore){
			CONCEPT_NAME = "newConcept";
		}

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(REMOVED_CONCEPTS);
		
		// Removes concepts one by one
		System.out.println("Removing concept: ");
		for (int i=1; i<=REMOVED_CONCEPTS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CONCEPT_NAME+i);
				
			try{
				if (benchmarkConstants.restore){
				// Inserts the concept
				odeService.insertTerm(ONTOLOGY_NAME,CONCEPT_NAME+i,
															"concept "+i+" of "+ONTOLOGY_NAME,Term.CONCEPT);
				}

				// Removes the concept
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				odeService.removeTerm(ONTOLOGY_NAME,CONCEPT_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR removing concept: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_3_20.txt");
	}

	/**
	 *	Removes several concepts from several ontologies
	 *
	 *	The database must have REMOVED_CONCEPTS ontologies named ONTOLOGY_NAMEx
	 *	with a concept named CONCEPT_NAME
	 */
	public static void benchmark1_3_21 (ODEService odeService) throws Exception
	{
		final int REMOVED_CONCEPTS = benchmarkConstants.TEST_CONCEPTS;
		String CONCEPT_NAME = "concept1";
		final String ONTOLOGY_NAME = "ontology";

		if (benchmarkConstants.restore){
			CONCEPT_NAME = "newConcept1";
		}

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(REMOVED_CONCEPTS);
		
		// Removes concepts one by one
		System.out.println("Removing concept: ");
		for (int i=1; i<=REMOVED_CONCEPTS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CONCEPT_NAME+"("+i+")");
				
			try{
				if (benchmarkConstants.restore){
				// Inserts the concept
				odeService.insertTerm(ONTOLOGY_NAME+i,CONCEPT_NAME,
															"concept of "+ONTOLOGY_NAME+i,Term.CONCEPT);
				}

				// Removes the concept
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				odeService.removeTerm(ONTOLOGY_NAME+i,CONCEPT_NAME);
				if (i>benchmarkConstants.warming){
					timer.stop();
				}
			}
			catch (WebODEException e){
				throw new Exception("ERROR removing concept: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_3_21.txt");
	}

}
