package es.upm.fi.dia.ontology.webode.wpbs;

// Java stuff
import java.util.HashMap;

import es.upm.fi.dia.ontology.webode.service.ClassAttributeDescriptor;
import es.upm.fi.dia.ontology.webode.service.ConstantDescriptor;
import es.upm.fi.dia.ontology.webode.service.FormulaDescriptor;
import es.upm.fi.dia.ontology.webode.service.Group;
import es.upm.fi.dia.ontology.webode.service.ImportedTerm;
import es.upm.fi.dia.ontology.webode.service.Instance;
import es.upm.fi.dia.ontology.webode.service.InstanceAttributeDescriptor;
import es.upm.fi.dia.ontology.webode.service.ODEService;
import es.upm.fi.dia.ontology.webode.service.OntologyDescriptor;
import es.upm.fi.dia.ontology.webode.service.ReferenceDescriptor;
import es.upm.fi.dia.ontology.webode.service.SynonymAbbreviation;
import es.upm.fi.dia.ontology.webode.service.Term;
import es.upm.fi.dia.ontology.webode.service.TermRelation;
import es.upm.fi.dia.ontology.webode.service.TermRelationInstance;
import es.upm.fi.dia.ontology.webode.service.TermTypes;
import es.upm.fi.dia.ontology.webode.service.WebODEException;

/** Implements a set of benchmarks for testing ODE service.
 *	All the benchmarks in this class test basic operations regarding 
 *	selecting values in the database.
 */
public class benchmarkBasicSelect
{

	/**
	 * Gets several times available ontologies in the database.
	 */
	public static void benchmark1_4_01 (ODEService odeService) throws Exception
	{
		final int GET_ONTOLOGIES = benchmarkConstants.TEST_ONTOLOGIES;
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_ONTOLOGIES);

		System.out.println("Getting available ontologies: ");
		for (int i=1; i<=GET_ONTOLOGIES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+i);
				
			// Gets available ontologies
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				OntologyDescriptor[] od = odeService.getAvailableOntologies();
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting available ontologies: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_01.txt");
	}

	/**
	 * Gets ontology references from several ontologies.
	 *
	 *	The database must have GET_ONTOLOGY_REFERENCES ontologies named ONTOLOGY_NAMEx
	 */
	public static void benchmark1_4_02 (ODEService odeService) throws Exception
	{
		final int GET_ONTOLOGY_REFERENCES = benchmarkConstants.TEST_ONTOLOGY_REFERENCES;
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_ONTOLOGY_REFERENCES);
		
		// Gets ontology references one by one
		System.out.println("Getting ontology references of: ");
		for (int i=1; i<=GET_ONTOLOGY_REFERENCES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+ONTOLOGY_NAME+i);
				
			// Gets ontology references
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				ReferenceDescriptor[] rd = odeService.getOntologyReferences(ONTOLOGY_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting ontology references: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_02.txt");
	}

	/**
	 * Gets several term references from an ontology.
	 *
	 *	The database must have a ontology named ONTOLOGY_NAME
	 *	with GET_TERM_REFERENCES term references named TERM_REFERENCE_NAMEx
	 */
	public static void benchmark1_4_03 (ODEService odeService) throws Exception
	{
		final int GET_TERM_REFERENCES = benchmarkConstants.TEST_TERM_REFERENCES;
		final String TERM_REFERENCE_NAME = "termReference";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_TERM_REFERENCES);
		
		// Gets term references one by one
		System.out.println("Getting term reference: "+TERM_REFERENCE_NAME);
		for (int i=1; i<=GET_TERM_REFERENCES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+TERM_REFERENCE_NAME+i);
				
			// Gets term references
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				ReferenceDescriptor rd = odeService.getReference(TERM_REFERENCE_NAME+i,ONTOLOGY_NAME);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting term references: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_03.txt");
	}

	/**
	 * Gets several term references from several ontologies.
	 *
	 *	The database must have GET_TERM_REFERENCES ontologies named ONTOLOGY_NAMEx
	 *	with a term reference named TERM_REFERENCE_NAME
	 */
	public static void benchmark1_4_04 (ODEService odeService) throws Exception
	{
		final int GET_TERM_REFERENCES = benchmarkConstants.TEST_TERM_REFERENCES;
		final String TERM_REFERENCE_NAME = "termReference1";
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_TERM_REFERENCES);
		
		// Gets term references one by one
		System.out.println("Getting term reference: ");
		for (int i=1; i<=GET_TERM_REFERENCES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+TERM_REFERENCE_NAME+"("+i+")");
				
			// Gets term references
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				ReferenceDescriptor rd = odeService.getReference(TERM_REFERENCE_NAME,ONTOLOGY_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting term references: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_04.txt");
	}

	/**
	 * Gets all term references from several ontologies.
	 *
	 *	The database must have GET_TERM_REFERENCES ontologies named ONTOLOGY_NAMEx
	 *	with term references
	 */
	public static void benchmark1_4_05 (ODEService odeService) throws Exception
	{
		final int GET_TERM_REFERENCES = benchmarkConstants.TEST_TERM_REFERENCES;
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_TERM_REFERENCES);
		
		System.out.println("Getting term references of ontology: ");
		for (int i=1; i<=GET_TERM_REFERENCES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+ONTOLOGY_NAME+i);
				
			// Gets term references
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				ReferenceDescriptor[] rd = odeService.getReferences(ONTOLOGY_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting term references of ontology: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_05.txt");
	}

	/**
	 * Gets all term references from several terms.
	 *
	 *	The database must have an ontology named ONTOLOGY_NAME 
	 *	with GET_TERM_REFERENCES concepts named CONCEPT_NAMEx
	 *	with term references
	 */
	public static void benchmark1_4_06 (ODEService odeService) throws Exception
	{
		final int GET_TERM_REFERENCES = benchmarkConstants.TEST_TERM_REFERENCES;
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_TERM_REFERENCES);
		
		System.out.println("Getting term references of concept: ");
		for (int i=1; i<=GET_TERM_REFERENCES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CONCEPT_NAME+i);
				
			// Gets term references
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				ReferenceDescriptor[] rd = odeService.getTermReferences(ONTOLOGY_NAME,CONCEPT_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting term references: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_06.txt");
	}

	/**
	 * Gets several terms from an ontology.
	 *
	 *	The database must have a ontology named ONTOLOGY_NAME
	 *	with GET_TERMS terms named TERM_NAMEx
	 */
	public static void benchmark1_4_07 (ODEService odeService) throws Exception
	{
		final int GET_TERMS = benchmarkConstants.TEST_CONCEPTS;
		final String TERM_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_TERMS);
		
		// Gets terms one by one
		System.out.println("Getting term: ");
		for (int i=1; i<=GET_TERMS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+TERM_NAME+i);
				
			// Gets terms
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				Term term = odeService.getTerm(ONTOLOGY_NAME,TERM_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting terms: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_07.txt");
	}

	/**
	 * Gets several terms from several ontologies.
	 *
	 *	The database must have GET_TERMS ontologies named ONTOLOGY_NAMEx
	 *	with a term named TERM_NAME
	 */
	public static void benchmark1_4_08 (ODEService odeService) throws Exception
	{
		final int GET_TERMS = benchmarkConstants.TEST_CONCEPTS;
		final String TERM_NAME = "concept1";
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_TERMS);
		
		// Gets terms one by one
		System.out.println("Getting term: ");
		for (int i=1; i<=GET_TERMS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+TERM_NAME+"("+i+")");
				
			// Gets terms
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				Term term = odeService.getTerm(ONTOLOGY_NAME+i,TERM_NAME);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting terms: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_08.txt");
	}

	/**
	 * Gets terms from several ontologies.
	 *
	 *	The database must have GET_TERMS ontologies named ONTOLOGY_NAMEx
	 *	with terms
	 */
	public static void benchmark1_4_09 (ODEService odeService) throws Exception
	{
		final int GET_TERMS = benchmarkConstants.TEST_CONCEPTS;
		final String ONTOLOGY_NAME = "ontology";
		
		int[] types = new int[1];
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_TERMS);
		
		System.out.println("Getting terms of: ");
		for (int i=1; i<=GET_TERMS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+ONTOLOGY_NAME+i);
				
			types[0] = TermTypes.CONCEPT;
				
			// Gets terms
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				Term[] term = odeService.getTerms(ONTOLOGY_NAME+i,types);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting terms: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_09.txt");
	}

	/**
	 * Gets term relations from several ontologies.
	 *
	 *	The database must have GET_TERMS ontologies named ONTOLOGY_NAMEx
	 *	with term relations
	 * MULTILINGUE*/
	public static void benchmark1_4_10 (ODEService odeService) throws Exception
	{
		final int GET_TERM_RELATIONS = benchmarkConstants.TEST_TERM_RELATIONS;
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_TERM_RELATIONS);
		
		System.out.println("Getting term relations of: ");
		for (int i=1; i<=GET_TERM_RELATIONS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+ONTOLOGY_NAME+i);
				
			// Gets terms
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				TermRelation[] tr = odeService.getTermRelations(ONTOLOGY_NAME+i,TermRelation.ADHOC);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting term relations: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_10.txt");
	}

	/**
	 * Gets several class attributes from a term.
	 *
	 *	The database must have a ontology named ONTOLOGY_NAME
	 *	with a concept named CONCEPT_NAME
	 *	with GET_CLASS_ATTRIBUTES class attributes named CLASS_ATTRIBUTE_NAMEx
	 */
	public static void benchmark1_4_11 (ODEService odeService) throws Exception
	{
		final int GET_CLASS_ATTRIBUTES = benchmarkConstants.TEST_CLASS_ATTRIBUTES;
		final String CLASS_ATTRIBUTE_NAME = "classAttribute";
		final String CONCEPT_NAME = "concept1";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_CLASS_ATTRIBUTES);
		
		// Gets class attributes one by one
		System.out.println("Getting class attribute: ");
		for (int i=1; i<=GET_CLASS_ATTRIBUTES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CLASS_ATTRIBUTE_NAME+i);
				
			// Gets class attributes
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				ClassAttributeDescriptor cad = odeService.getClassAttribute(ONTOLOGY_NAME,CONCEPT_NAME,
																																		CLASS_ATTRIBUTE_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting class attributes: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_11.txt");
	}

	/**
	 * Gets several class attributes from several terms.
	 *
	 *	The database must have a ontology named ONTOLOGY_NAME
	 *	with GET_CLASS_ATTRIBUTES concepts named CONCEPT_NAMEx
	 *	with a class attribute named CLASS_ATTRIBUTE_NAME
	 */
	public static void benchmark1_4_12 (ODEService odeService) throws Exception
	{
		final int GET_CLASS_ATTRIBUTES = benchmarkConstants.TEST_CLASS_ATTRIBUTES;
		final String CLASS_ATTRIBUTE_NAME = "classAttribute1";
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_CLASS_ATTRIBUTES);
		
		// Gets class attributes one by one
		System.out.println("Getting class attribute: ");
		for (int i=1; i<=GET_CLASS_ATTRIBUTES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CLASS_ATTRIBUTE_NAME+"("+i+")");
				
			// Gets class attributes
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				ClassAttributeDescriptor cad = odeService.getClassAttribute(ONTOLOGY_NAME,CONCEPT_NAME+i,
																																		CLASS_ATTRIBUTE_NAME);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting class attributes: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_12.txt");
	}

	/**
	 * Gets class attributes from several terms.
	 *
	 *	The database must have a ontology named ONTOLOGY_NAME
	 *	with GET_CLASS_ATTRIBUTES concepts named CONCEPT_NAMEx
	 *	with class attributes
	 */
	public static void benchmark1_4_13 (ODEService odeService) throws Exception
	{
		final int GET_CLASS_ATTRIBUTES = benchmarkConstants.TEST_CLASS_ATTRIBUTES;
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_CLASS_ATTRIBUTES);
		
		System.out.println("Getting class attributes of: ");
		for (int i=1; i<=GET_CLASS_ATTRIBUTES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CONCEPT_NAME+i);
				
			// Gets class attributes
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				ClassAttributeDescriptor[] cad = odeService.getClassAttributes(ONTOLOGY_NAME,CONCEPT_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting class attributes: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_13.txt");
	}

	/**
	 * Gets several instance attributes from a term.
	 *
	 *	The database must have a ontology named ONTOLOGY_NAME
	 *	with a concept named CONCEPT_NAME
	 *	with GET_INSTANCE_ATTRIBUTES instance attributes named INSTANCE_ATTRIBUTE_NAMEx
	 */
	public static void benchmark1_4_14 (ODEService odeService) throws Exception
	{
		final int GET_INSTANCE_ATTRIBUTES = benchmarkConstants.TEST_INSTANCE_ATTRIBUTES;
		final String INSTANCE_ATTRIBUTE_NAME = "instanceAttribute";
		final String CONCEPT_NAME = "concept1";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_INSTANCE_ATTRIBUTES);
		
		// Gets instance attributes one by one
		System.out.println("Getting instance attribute: ");
		for (int i=1; i<=GET_INSTANCE_ATTRIBUTES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+INSTANCE_ATTRIBUTE_NAME+i);
				
			// Gets instance attributes
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				InstanceAttributeDescriptor iad = odeService.getInstanceAttribute(ONTOLOGY_NAME,
																											CONCEPT_NAME,INSTANCE_ATTRIBUTE_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting instance attributes: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_14.txt");
	}

	/**
	 * Gets several instance attributes from several terms.
	 *
	 *	The database must have a ontology named ONTOLOGY_NAME
	 *	with GET_INSTANCE_ATTRIBUTES concepts named CONCEPT_NAMEx
	 *	with a instance attribute named INSTANCE_ATTRIBUTE_NAME
	 */
	public static void benchmark1_4_15 (ODEService odeService) throws Exception
	{
		final int GET_INSTANCE_ATTRIBUTES = benchmarkConstants.TEST_INSTANCE_ATTRIBUTES;
		final String INSTANCE_ATTRIBUTE_NAME = "instanceAttribute1";
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_INSTANCE_ATTRIBUTES);
		
		// Gets instance attributes one by one
		System.out.println("Getting instance attribute: ");
		for (int i=1; i<=GET_INSTANCE_ATTRIBUTES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+INSTANCE_ATTRIBUTE_NAME+"("+i+")");
				
			// Gets instance attributes
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				InstanceAttributeDescriptor iad = odeService.getInstanceAttribute(ONTOLOGY_NAME,
																											CONCEPT_NAME+i,INSTANCE_ATTRIBUTE_NAME);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting instance attributes: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_15.txt");
	}

	/**
	 * Gets instance attributes from several terms.
	 *
	 *	The database must have a ontology named ONTOLOGY_NAME
	 *	with GET_INSTANCE_ATTRIBUTES concepts named CONCEPT_NAMEx
	 *	with instance attributes
	 */
	public static void benchmark1_4_16 (ODEService odeService) throws Exception
	{
		final int GET_INSTANCE_ATTRIBUTES = benchmarkConstants.TEST_INSTANCE_ATTRIBUTES;
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_INSTANCE_ATTRIBUTES);
		
		System.out.println("Getting instance attributes of: ");
		for (int i=1; i<=GET_INSTANCE_ATTRIBUTES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CONCEPT_NAME+i);
				
			// Gets instance attributes
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				InstanceAttributeDescriptor[] iad = odeService.getInstanceAttributes(ONTOLOGY_NAME,
																																						 CONCEPT_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting instance attributes: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_16.txt");
	}

	/**
	 * Gets several synonyms from a term.
	 *
	 *	The database must have a ontology named ONTOLOGY_NAME
	 *	with a concept named CONCEPT_NAME
	 *	with GET_SYNONYMS synonyms named SYNONYM_NAMEx
	 */
	public static void benchmark1_4_17 (ODEService odeService) throws Exception
	{
		final int GET_SYNONYMS = benchmarkConstants.TEST_SYNONYMS;
		final String SYNONYM_NAME = "synonym";
		final String CONCEPT_NAME = "concept1";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_SYNONYMS);
		
		// Gets synonyms one by one
		System.out.println("Getting synonym: ");
		for (int i=1; i<=GET_SYNONYMS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+SYNONYM_NAME+i);
				
			// Gets synonyms
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				SynonymAbbreviation sa = odeService.getSynonym(ONTOLOGY_NAME,CONCEPT_NAME,null,
																											 SYNONYM_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting synonyms: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_17.txt");
	}

	/**
	 * Gets several synonyms from several terms.
	 *
	 *	The database must have a ontology named ONTOLOGY_NAME
	 *	with GET_SYNONYMS concepts named CONCEPT_NAMEx
	 *	with a synonym named SYNONYM_NAME
	 */
	public static void benchmark1_4_18 (ODEService odeService) throws Exception
	{
		final int GET_SYNONYMS = benchmarkConstants.TEST_SYNONYMS;
		final String SYNONYM_NAME = "synonym1";
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_SYNONYMS);
		
		// Gets synonyms one by one
		System.out.println("Getting synonym: ");
		for (int i=1; i<=GET_SYNONYMS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+SYNONYM_NAME+"("+i+")");
				
			// Gets synonyms
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				SynonymAbbreviation sa = odeService.getSynonym(ONTOLOGY_NAME,CONCEPT_NAME+i,null,
																											 SYNONYM_NAME);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting synonyms: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_18.txt");
	}

	/**
	 * Gets synonyms from several terms.
	 *
	 *	The database must have a ontology named ONTOLOGY_NAME
	 *	with GET_SYNONYMS concepts named CONCEPT_NAMEx
	 *	with synonyms
	 */
	public static void benchmark1_4_19 (ODEService odeService) throws Exception
	{
		final int GET_SYNONYMS = benchmarkConstants.TEST_SYNONYMS;
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_SYNONYMS);
		
		System.out.println("Getting synonyms of: ");
		for (int i=1; i<=GET_SYNONYMS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CONCEPT_NAME+i);
				
			// Gets synonyms
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				SynonymAbbreviation[] sa = odeService.getSynonyms(ONTOLOGY_NAME,CONCEPT_NAME+i,null);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting synonyms: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_19.txt");
	}

	/**
	 * Gets imported terms from several ontologies.
	 *
	 *	The database must have GET_IMPORTED_TERMS ontologies named ONTOLOGY_NAMEx
	 */
	public static void benchmark1_4_20 (ODEService odeService) throws Exception
	{
		final int GET_IMPORTED_TERMS = benchmarkConstants.TEST_IMPORTED_TERMS;
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_IMPORTED_TERMS);
		
		System.out.println("Getting imported terms of: ");
		for (int i=1; i<=GET_IMPORTED_TERMS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+ONTOLOGY_NAME+i);
				
			// Gets imported terms
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				ImportedTerm[] rd = odeService.getImportedTerms(ONTOLOGY_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting imported terms: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_20.txt");
	}

	/**
	 * Gets several constants from an ontology.
	 *
	 *	The database must have a ontology named ONTOLOGY_NAME
	 *	with GET_CONSTANTS constants named CONSTANT_NAMEx
	 */
	public static void benchmark1_4_21 (ODEService odeService) throws Exception
	{
		final int GET_CONSTANTS = benchmarkConstants.TEST_CONSTANTS;
		final String CONSTANT_NAME = "constant";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_CONSTANTS);
		
		// Gets constants one by one
		System.out.println("Getting constant: ");
		for (int i=1; i<=GET_CONSTANTS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CONSTANT_NAME+i);
				
			// Gets constants
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				ConstantDescriptor cd = odeService.getConstant(ONTOLOGY_NAME,CONSTANT_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting constants: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_21.txt");
	}

	/**
	 * Gets several constants from several ontologies.
	 *
	 *	The database must have GET_CONSTANTS ontologies named ONTOLOGY_NAMEx
	 *	with a constant named CONSTANT_NAME
	 */
	public static void benchmark1_4_22 (ODEService odeService) throws Exception
	{
		final int GET_CONSTANTS = benchmarkConstants.TEST_CONSTANTS;
		final String CONSTANT_NAME = "constant1";
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_CONSTANTS);
		
		// Gets constants one by one
		System.out.println("Getting constant: ");
		for (int i=1; i<=GET_CONSTANTS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CONSTANT_NAME+"("+i+")");
				
			// Gets constants
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				ConstantDescriptor cd = odeService.getConstant(ONTOLOGY_NAME+i,CONSTANT_NAME);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting constants: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_22.txt");
	}

	/**
	 * Gets several formulas from an ontology.
	 *
	 *	The database must have a ontology named ONTOLOGY_NAME
	 *	with GET_FORMULAS formulas named FORMULA_NAMEx
	 */
	public static void benchmark1_4_23 (ODEService odeService) throws Exception
	{
		final int GET_FORMULAS = benchmarkConstants.TEST_FORMULAS;
		final String FORMULA_NAME = "formula";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_FORMULAS);
		
		// Gets formulas one by one
		System.out.println("Getting formula: ");
		for (int i=1; i<=GET_FORMULAS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+FORMULA_NAME+i);
				
			// Gets formulas
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				FormulaDescriptor fd = odeService.getReasoningElement(ONTOLOGY_NAME,FORMULA_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting formulas: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_23.txt");
	}

	/**
	 * Gets several formulas from several ontologies.
	 *
	 *	The database must have GET_FORMULAS ontologies named ONTOLOGY_NAMEx
	 *	with a formula named FORMULA_NAME
	 */
	public static void benchmark1_4_24 (ODEService odeService) throws Exception
	{
		final int GET_FORMULAS = benchmarkConstants.TEST_FORMULAS;
		final String FORMULA_NAME = "formula1";
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_FORMULAS);
		
		// Gets formulas one by one
		System.out.println("Getting formula: ");
		for (int i=1; i<=GET_FORMULAS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+FORMULA_NAME+"("+i+")");
				
			// Gets formulas
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				FormulaDescriptor fd = odeService.getReasoningElement(ONTOLOGY_NAME+i,FORMULA_NAME);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting formulas: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_24.txt");
	}

	/**
	 * Gets formulas from several ontologies.
	 *
	 *	The database must have GET_FORMULAS ontologies named ONTOLOGY_NAMEx
	 *	with formulas
	 */
	public static void benchmark1_4_25 (ODEService odeService) throws Exception
	{
		final int GET_FORMULAS = benchmarkConstants.TEST_FORMULAS;
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_FORMULAS);
		
		System.out.println("Getting formulas of ontology: ");
		for (int i=1; i<=GET_FORMULAS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+ONTOLOGY_NAME+i);
				
			// Gets formulas
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				FormulaDescriptor[] rd = odeService.getReasoningElements(ONTOLOGY_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting formulas of ontology: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_25.txt");
	}

	/**
	 * Gets instances from several instance sets in an ontology.
	 *
	 *	The database must have a ontology named ONTOLOGY_NAME
	 *	with GET_INSTANCES instance sets named INSTANCE_SET_NAMEx
	 *	with instances
	 */
	public static void benchmark1_4_26 (ODEService odeService) throws Exception
	{
		final int GET_INSTANCES = benchmarkConstants.TEST_INSTANCES;
		final String INSTANCE_SET_NAME = "instanceSet";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_INSTANCES);
		
		System.out.println("Getting instances of instance set: ");
		for (int i=1; i<=GET_INSTANCES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+INSTANCE_SET_NAME+i);
				
			// Gets instances
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				Term[] term = odeService.getInstances(ONTOLOGY_NAME,INSTANCE_SET_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting instances: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_26.txt");
	}

	/**
	 * Gets instances from several instance sets in several ontologies.
	 *
	 *	The database must have GET_INSTANCES ontologies named ONTOLOGY_NAMEx
	 *	with a instance set named INSTANCE_SET_NAME
	 *	with instances
	 */
	public static void benchmark1_4_27 (ODEService odeService) throws Exception
	{
		final int GET_INSTANCES = benchmarkConstants.TEST_INSTANCES;
		final String INSTANCE_SET_NAME = "instanceSet1";
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_INSTANCES);
		
		System.out.println("Getting instances of instance set: ");
		for (int i=1; i<=GET_INSTANCES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+INSTANCE_SET_NAME+"("+i+")");
				
			// Gets instances
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				Term[] term = odeService.getInstances(ONTOLOGY_NAME+i,INSTANCE_SET_NAME);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting instances: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_27.txt");
	}

	/**
	 * Gets instances from several instance sets in an ontology.
	 *
	 *	The database must have a ontology named ONTOLOGY_NAME
	 *	with GET_INSTANCES instance sets named INSTANCE_SET_NAMEx
	 *	with instances
	 */
	public static void benchmark1_4_28 (ODEService odeService) throws Exception
	{
		final int GET_INSTANCES = benchmarkConstants.TEST_INSTANCES;
		final String INSTANCE_SET_NAME = "instanceSet";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_INSTANCES);
		
		System.out.println("Getting instances of instance set: ");
		for (int i=1; i<=GET_INSTANCES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+INSTANCE_SET_NAME+i);
				
			// Gets instances
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				Instance[] instance = odeService.getInstInstances(ONTOLOGY_NAME,INSTANCE_SET_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting instances: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_28.txt");
	}

	/**
	 * Gets instances from several instance sets in several ontologies.
	 *
	 *	The database must have GET_INSTANCES ontologies named ONTOLOGY_NAMEx
	 *	with a instance set named INSTANCE_SET_NAME
	 *	with instances
	 */
	public static void benchmark1_4_29 (ODEService odeService) throws Exception
	{
		final int GET_INSTANCES = benchmarkConstants.TEST_INSTANCES;
		final String INSTANCE_SET_NAME = "instanceSet1";
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_INSTANCES);
		
		System.out.println("Getting instances of instance set: ");
		for (int i=1; i<=GET_INSTANCES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+INSTANCE_SET_NAME+"("+i+")");
				
			// Gets instances
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				Instance[] instance = odeService.getInstInstances(ONTOLOGY_NAME+i,INSTANCE_SET_NAME);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting instances: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_29.txt");
	}

	/**
	 * Gets instance relations from several instance sets in an ontology.
	 *
	 *	The database must have a ontology named ONTOLOGY_NAME
	 *	with GET_INSTANCE_RELATIONS instance sets named INSTANCE_SET_NAMEx
	 */
	public static void benchmark1_4_30 (ODEService odeService) throws Exception
	{
		final int GET_INSTANCE_RELATIONS = benchmarkConstants.TEST_RELATION_INSTANCES;
		final String INSTANCE_SET_NAME = "instanceSet";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_INSTANCE_RELATIONS);
		
		// Gets instance relations one by one
		System.out.println("Getting instance relations of instance set: ");
		for (int i=1; i<=GET_INSTANCE_RELATIONS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+INSTANCE_SET_NAME+i);
				
			// Gets instance relations
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				TermRelationInstance[] tri = odeService.getRelationInstances(ONTOLOGY_NAME,INSTANCE_SET_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting instance relations: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_30.txt");
	}

	/**
	 * Gets instance relations from several instance sets in several ontologies.
	 *
	 *	The database must have GET_INSTANCE_RELATIONS ontologies named ONTOLOGY_NAMEx
	 *	with a instance set named INSTANCE_SET_NAME
	 */
	public static void benchmark1_4_31 (ODEService odeService) throws Exception
	{
		final int GET_INSTANCE_RELATIONS = benchmarkConstants.TEST_RELATION_INSTANCES;
		final String INSTANCE_SET_NAME = "instanceSet1";
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_INSTANCE_RELATIONS);
		
		// Gets instance relations one by one
		System.out.println("Getting instance relations of instance set: ");
		for (int i=1; i<=GET_INSTANCE_RELATIONS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+INSTANCE_SET_NAME+"("+i+")");
				
			// Gets instance relations
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				TermRelationInstance[] tri = odeService.getRelationInstances(ONTOLOGY_NAME+i,INSTANCE_SET_NAME);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting instance relations: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_31.txt");
	}

	/**
	 * Gets instance values from several instances in an ontology.
	 *
	 *	The database must have a ontology named ONTOLOGY_NAME
	 *	with a instance set named INSTANCE_SET_NAME
	 *	with GET_INSTANCE_VALUES instance values named INSTANCE_VALUE_NAMEx
	 */
	public static void benchmark1_4_32 (ODEService odeService) throws Exception
	{
		final int GET_INSTANCE_VALUES = benchmarkConstants.TEST_INSTANCE_VALUES;
		final String INSTANCE_NAME = "instance";
		final String INSTANCE_SET_NAME = "instanceSet1";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_INSTANCE_VALUES);
		
		System.out.println("Getting instance values of instance: ");
		for (int i=1; i<=GET_INSTANCE_VALUES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+INSTANCE_NAME+i);
				
			// Gets instance values
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				HashMap hm = odeService.getInstanceValues(ONTOLOGY_NAME,INSTANCE_SET_NAME,INSTANCE_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting instance values: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_32.txt");
	}

	/**
	 * Gets instance values from several instances in several ontologies.
	 *
	 *	The database must have GET_INSTANCE_VALUES ontologies named ONTOLOGY_NAMEx
	 *	with a instance set named INSTANCE_SET_NAME
	 *	with a instance value named INSTANCE_VALUE_NAME
	 */
	public static void benchmark1_4_33 (ODEService odeService) throws Exception
	{
		final int GET_INSTANCE_VALUES = benchmarkConstants.TEST_INSTANCE_VALUES;
		final String INSTANCE_NAME = "instance1";
		final String INSTANCE_SET_NAME = "instanceSet1";
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_INSTANCE_VALUES);
		
		System.out.println("Getting instance values of instance: ");
		for (int i=1; i<=GET_INSTANCE_VALUES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+INSTANCE_NAME+"("+i+")");
				
			// Gets instance values
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				HashMap hm = odeService.getInstanceValues(ONTOLOGY_NAME+i,INSTANCE_SET_NAME,INSTANCE_NAME);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting instance values: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_33.txt");
	}

	/**
	 * Gets logical instance values from several instances in an ontology.
	 *
	 *	The database must have a ontology named ONTOLOGY_NAME
	 *	with a instance set named INSTANCE_SET_NAME
	 *	with GET_INSTANCE_VALUES instance values named INSTANCE_VALUE_NAMEx
	 */
	public static void benchmark1_4_34 (ODEService odeService) throws Exception
	{
		final int GET_INSTANCE_VALUES = benchmarkConstants.TEST_INSTANCE_VALUES;
		final String INSTANCE_NAME = "instance";
		final String INSTANCE_SET_NAME = "instanceSet1";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_INSTANCE_VALUES);
		
		System.out.println("Getting logical instance values of instance: ");
		for (int i=1; i<=GET_INSTANCE_VALUES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+INSTANCE_NAME+i);
				
			// Gets logical instance values
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				HashMap hm = odeService.getLogicalInstanceValues(ONTOLOGY_NAME,INSTANCE_SET_NAME,INSTANCE_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting logical instance values: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_34.txt");
	}

	/**
	 * Gets logical instance values from several instances in several ontologies.
	 *
	 *	The database must have GET_INSTANCE_VALUES ontologies named ONTOLOGY_NAMEx
	 *	with a instance set named INSTANCE_SET_NAME
	 *	with a instance value named INSTANCE_VALUE_NAME
	 */
	public static void benchmark1_4_35 (ODEService odeService) throws Exception
	{
		final int GET_INSTANCE_VALUES = benchmarkConstants.TEST_INSTANCE_VALUES;
		final String INSTANCE_NAME = "instance1";
		final String INSTANCE_SET_NAME = "instanceSet1";
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_INSTANCE_VALUES);
		
		System.out.println("Getting logical instance values of instance: ");
		for (int i=1; i<=GET_INSTANCE_VALUES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+INSTANCE_NAME+"("+i+")");
				
			// Gets logical instance values
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				HashMap hm = odeService.getLogicalInstanceValues(ONTOLOGY_NAME+i,INSTANCE_SET_NAME,INSTANCE_NAME);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting logical instance values: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_35.txt");
	}

	/**
	 * Gets several groups from an ontology.
	 *
	 *	The database must have a ontology named ONTOLOGY_NAME
	 *	with GET_GROUPS groups named GROUP_NAMEx
	 */
	public static void benchmark1_4_36 (ODEService odeService) throws Exception
	{
		final int GET_GROUPS = benchmarkConstants.TEST_GROUPS;
		final String GROUP_NAME = "group";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_GROUPS);
		
		// Gets groups one by one
		System.out.println("Getting group: ");
		for (int i=1; i<=GET_GROUPS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+GROUP_NAME+i);
				
			// Gets groups
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				Group group = odeService.getGroup(ONTOLOGY_NAME,GROUP_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting groups: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_36.txt");
	}

	/**
	 * Gets several groups from several ontologies.
	 *
	 *	The database must have GET_GROUPS ontologies named ONTOLOGY_NAMEx
	 *	with a group named GROUP_NAME
	 */
	public static void benchmark1_4_37 (ODEService odeService) throws Exception
	{
		final int GET_GROUPS = benchmarkConstants.TEST_GROUPS;
		final String GROUP_NAME = "group1";
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_GROUPS);
		
		// Gets groups one by one
		System.out.println("Getting group: ");
		for (int i=1; i<=GET_GROUPS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+GROUP_NAME+"("+i+")");
				
			// Gets groups
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				Group group = odeService.getGroup(ONTOLOGY_NAME+i,GROUP_NAME);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting groups: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_37.txt");
	}

	/**
	 * Gets groups from several ontologies.
	 *
	 *	The database must have GET_GROUPS ontologies named ONTOLOGY_NAMEx
	 *	with groups
	 */
	public static void benchmark1_4_38 (ODEService odeService) throws Exception
	{
		final int GET_GROUPS = benchmarkConstants.TEST_GROUPS;
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_GROUPS);
		
		System.out.println("Getting groups of: ");
		for (int i=1; i<=GET_GROUPS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+ONTOLOGY_NAME+i);
				
			// Gets groups
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				Group[] group = odeService.getGroups(ONTOLOGY_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting groups: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_38.txt");
	}

	/**
	 * Gets concepts in which several attributes are defined in an ontology.
	 *
	 *	The database must have a ontology named ONTOLOGY_NAME
	 *	with concepts 
	 *	with GET_CONCEPTS attributes named ATTRIBUTE_NAMEx
	 */
	public static void benchmark1_4_39 (ODEService odeService) throws Exception
	{
		final int GET_CONCEPTS = benchmarkConstants.TEST_CLASS_ATTRIBUTES;
		final String ATTRIBUTE_NAME = "classAttribute";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_CONCEPTS);
		
		System.out.println("Getting concepts with attribute: ");
		for (int i=1; i<=GET_CONCEPTS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+ATTRIBUTE_NAME+i);
				
			// Gets concepts
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				String[] concepts = odeService.getConceptsInWhichAttributeIsDefined(ONTOLOGY_NAME,
																																						ATTRIBUTE_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting concepts: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_39.txt");
	}

	/**
	 * Gets concepts in which several attributes are defined in several ontologies.
	 *
	 *	The database must have GET_CONCEPTS ontologies named ONTOLOGY_NAMEx
	 *	with concepts 
	 *	with an attribute named ATTRIBUTE_NAME
	 */
	public static void benchmark1_4_40 (ODEService odeService) throws Exception
	{
		final int GET_CONCEPTS = benchmarkConstants.TEST_CLASS_ATTRIBUTES;
		final String ATTRIBUTE_NAME = "classAttribute1";
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_CONCEPTS);
		
		System.out.println("Getting concepts with attribute: ");
		for (int i=1; i<=GET_CONCEPTS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+ATTRIBUTE_NAME+"("+i+")");
				
			// Gets concepts
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				String[] concepts = odeService.getConceptsInWhichAttributeIsDefined(ONTOLOGY_NAME+i,
																																						ATTRIBUTE_NAME);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting concepts: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_40.txt");
	}

	/**
	 * Gets root concepts from several ontologies.
	 *
	 *	The database must have GET_ROOT_CONCEPTS ontologies named ONTOLOGY_NAMEx
	 */
	public static void benchmark1_4_41 (ODEService odeService) throws Exception
	{
		final int GET_ROOT_CONCEPTS = benchmarkConstants.TEST_CONCEPTS;
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_ROOT_CONCEPTS);
		
		System.out.println("Getting root concepts of: ");
		for (int i=1; i<=GET_ROOT_CONCEPTS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+ONTOLOGY_NAME+i);
				
			// Gets root concepts
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				Term[] rc = odeService.getRootConcepts(ONTOLOGY_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting root concepts: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_41.txt");
	}

	/**
	 * Gets parent concepts from several concepts in an ontology.
	 *
	 *	The database must have a ontology named ONTOLOGY_NAME
	 *	with GET_PARENT_CONCEPTS concepts named CONCEPT_NAMEx
	 */
	public static void benchmark1_4_42 (ODEService odeService) throws Exception
	{
		final int GET_PARENT_CONCEPTS = benchmarkConstants.TEST_CONCEPTS;
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_PARENT_CONCEPTS);
		
		System.out.println("Getting parent concepts of: ");
		for (int i=1; i<=GET_PARENT_CONCEPTS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CONCEPT_NAME+i);
				
			// Gets parent concepts
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				Term[] rc = odeService.getParentConcepts(ONTOLOGY_NAME,CONCEPT_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting parent concepts: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_42.txt");
	}

	/**
	 * Gets parent concepts from several concepts in several ontologies.
	 *
	 *	The database must have GET_PARENT_CONCEPTS ontologies named ONTOLOGY_NAMEx
	 *	with a concept named CONCEPT_NAME
	 */
	public static void benchmark1_4_43 (ODEService odeService) throws Exception
	{
		final int GET_PARENT_CONCEPTS = benchmarkConstants.TEST_CONCEPTS;
		final String CONCEPT_NAME = "concept1";
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_PARENT_CONCEPTS);
		
		System.out.println("Getting parent concepts of: ");
		for (int i=1; i<=GET_PARENT_CONCEPTS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CONCEPT_NAME+"("+i+")");
				
			// Gets parent concepts
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				Term[] rc = odeService.getParentConcepts(ONTOLOGY_NAME+i,CONCEPT_NAME);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting parent concepts: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_43.txt");
	}

	/**
	 * Gets child concepts from several concepts in an ontology.
	 *
	 *	The database must have a ontology named ONTOLOGY_NAME
	 *	with GET_CHILD_CONCEPTS concepts named CONCEPT_NAMEx
	 */
	public static void benchmark1_4_44 (ODEService odeService) throws Exception
	{
		final int GET_CHILD_CONCEPTS = benchmarkConstants.TEST_CONCEPTS;
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_CHILD_CONCEPTS);
		
		System.out.println("Getting child concepts of: ");
		for (int i=1; i<=GET_CHILD_CONCEPTS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CONCEPT_NAME+i);
				
			// Gets child concepts
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				Term[] rc = odeService.getChildConcepts(ONTOLOGY_NAME,CONCEPT_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting child concepts: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_44.txt");
	}

	/**
	 * Gets child concepts from several concepts in several ontologies.
	 *
	 *	The database must have GET_CHILD_CONCEPTS ontologies named ONTOLOGY_NAMEx
	 *	with a concept named CONCEPT_NAME
	 */
	public static void benchmark1_4_45 (ODEService odeService) throws Exception
	{
		final int GET_CHILD_CONCEPTS = benchmarkConstants.TEST_CONCEPTS;
		final String CONCEPT_NAME = "concept1";
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_CHILD_CONCEPTS);
		
		System.out.println("Getting child concepts of: ");
		for (int i=1; i<=GET_CHILD_CONCEPTS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CONCEPT_NAME+"("+i+")");
				
			// Gets child concepts
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				Term[] rc = odeService.getChildConcepts(ONTOLOGY_NAME+i,CONCEPT_NAME);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting child concepts: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_45.txt");
	}

	/**
	 * Gets ad-hoc term relations of several concepts in an ontology.
	 *
	 *	The database must have a ontology named ONTOLOGY_NAME
	 *	with GET_AD_HOC_RELATIONS concepts named CONCEPT_NAMEx
	 */
	public static void benchmark1_4_46 (ODEService odeService) throws Exception
	{
		final int GET_AD_HOC_RELATIONS = benchmarkConstants.TEST_CONCEPTS;
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_AD_HOC_RELATIONS);
		
		System.out.println("Getting ad-hoc relations of: ");
		for (int i=1; i<=GET_AD_HOC_RELATIONS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CONCEPT_NAME+i);
				
			// Gets ad-hoc relations
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				TermRelation[] tr = odeService.getAdHocTermRelationsAttachedToConcept(ONTOLOGY_NAME,CONCEPT_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting ad-hoc relations: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_46.txt");
	}

	/**
	 * Gets ad-hoc term relations of several concepts in several ontologies.
	 *
	 *	The database must have GET_AD_HOC_RELATIONS ontologies named ONTOLOGY_NAMEx
	 *	with a concept named CONCEPT_NAME
	 */
	public static void benchmark1_4_47 (ODEService odeService) throws Exception
	{
		final int GET_AD_HOC_RELATIONS = benchmarkConstants.TEST_CONCEPTS;
		final String CONCEPT_NAME = "concept1";
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_AD_HOC_RELATIONS);
		
		System.out.println("Getting ad-hoc relations of: ");
		for (int i=1; i<=GET_AD_HOC_RELATIONS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CONCEPT_NAME+"("+i+")");
				
			// Gets ad-hoc relations
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				TermRelation[] tr = odeService.getAdHocTermRelationsAttachedToConcept(ONTOLOGY_NAME+i,CONCEPT_NAME);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting ad-hoc relations: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_4_47.txt");
	}


}
