package es.upm.fi.dia.ontology.webode.wpbs;

// Java stuff
import es.upm.fi.dia.ontology.webode.service.ClassAttributeDescriptor;
import es.upm.fi.dia.ontology.webode.service.ConstantDescriptor;
import es.upm.fi.dia.ontology.webode.service.FormulaDescriptor;
import es.upm.fi.dia.ontology.webode.service.Group;
import es.upm.fi.dia.ontology.webode.service.InstanceAttributeDescriptor;
import es.upm.fi.dia.ontology.webode.service.ODEService;
import es.upm.fi.dia.ontology.webode.service.OntologyDescriptor;
import es.upm.fi.dia.ontology.webode.service.ReferenceDescriptor;
import es.upm.fi.dia.ontology.webode.service.SynonymAbbreviation;
import es.upm.fi.dia.ontology.webode.service.Term;
import es.upm.fi.dia.ontology.webode.service.TermTypes;
import es.upm.fi.dia.ontology.webode.service.ValueTypes;
import es.upm.fi.dia.ontology.webode.service.WebODEException;

/** Implements a set of benchmarks for testing ODE service.
 *	All the benchmarks in this class test basic operations regarding 
 *	updating values in the database.
 */
public class benchmarkBasicUpdate
{

	/**
	 *	Updates several times an ontology
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 */
	public static void benchmark1_2_01 (ODEService odeService) throws Exception
	{
		final int ONTOLOGY_UPDATES = benchmarkConstants.TEST_ONTOLOGIES;
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(ONTOLOGY_UPDATES);
		
		// Updates ontologies one by one
		System.out.println("Updating ontology: ");
		for (int i=1; i<=ONTOLOGY_UPDATES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+ONTOLOGY_NAME+"("+i+")");
				
			// Updates the ontology
			try{																									 	 
				if (i>benchmarkConstants.warming){
					timer.start();
				}
				OntologyDescriptor od = odeService.getOntologyDescriptor(ONTOLOGY_NAME);
				od.description = "ontology update "+i;
				odeService.updateOntology(ONTOLOGY_NAME,od);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR updating ontology: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_2_01.txt");
	}

	/**
	 *	Updates several ontologies
	 *
	 *	The database must have ONTOLOGY_UPDATES ontologies named ONTOLOGY_NAMEx
	 * MULTILINGUE*/
	public static void benchmark1_2_02 (ODEService odeService) throws Exception
	{
		final int ONTOLOGY_UPDATES = benchmarkConstants.TEST_ONTOLOGIES;
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(ONTOLOGY_UPDATES);
		
		// Updates ontologies one by one
		System.out.println("Updating ontology: ");
		for (int i=1; i<=ONTOLOGY_UPDATES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+ONTOLOGY_NAME+i);
				
			// Updates the ontology
			try{
				if (i>benchmarkConstants.warming){
      		timer.start();
    		}
				OntologyDescriptor od = odeService.getOntologyDescriptor(ONTOLOGY_NAME+i);
				od.description = "ontology update "+i;
				odeService.updateOntology(ONTOLOGY_NAME+i,od);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR updating ontology: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_2_02.txt");
	}

	/**
	 *	Updates several term references in an ontology
	 *
	 *	The database must have TERM_REFERENCE_UPDATES term references named TERM_REFERENCE_NAMEx
	 */
	public static void benchmark1_2_03 (ODEService odeService) throws Exception
	{
		final int TERM_REFERENCE_UPDATES = benchmarkConstants.TEST_TERM_REFERENCES;
		final String TERM_REFERENCE_NAME = "termReference";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(TERM_REFERENCE_UPDATES);
		
		// Updates term references one by one
		System.out.println("Updating term reference: ");
		for (int i=1; i<=TERM_REFERENCE_UPDATES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+TERM_REFERENCE_NAME+i);
				
			// Updates the term reference
			try{
				if (i>benchmarkConstants.warming){
					timer.start();
        }
				ReferenceDescriptor rd = new ReferenceDescriptor(TERM_REFERENCE_NAME+i,
																												 "term reference "+i,ONTOLOGY_NAME);
				odeService.updateReference(TERM_REFERENCE_NAME+i,rd);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR updating term reference: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_2_03.txt");
	}

	/**
	 *	Updates several term references in several ontologies
	 *
	 *	The database must have TERM_REFERENCE_UPDATES ontologies named ONTOLOGY_NAMEx
	 *	with one term reference named TERM_REFERENCE_NAME
	 */
	public static void benchmark1_2_04 (ODEService odeService) throws Exception
	{
		final int TERM_REFERENCE_UPDATES = benchmarkConstants.TEST_TERM_REFERENCES;
		final String TERM_REFERENCE_NAME = "termReference1";
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(TERM_REFERENCE_UPDATES);
		
		// Updates term references one by one
		System.out.println("Updating term reference: ");
		for (int i=1; i<=TERM_REFERENCE_UPDATES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+TERM_REFERENCE_NAME+"("+i+")");
				
			// Updates the term reference
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				ReferenceDescriptor rd = new ReferenceDescriptor(TERM_REFERENCE_NAME,
																												 "term reference "+i,ONTOLOGY_NAME+i);
				odeService.updateReference(TERM_REFERENCE_NAME,rd);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR updating term reference: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_2_04.txt");
	}

	/**
	 *	Updates several concepts in an ontology
	 *
	 *	The database must have one ontology named ONTOLOGY_NAME
	 *	with CONCEPT_UPDATES concepts named CONCEPT_NAMEx
	 * MULTILINGUE*/
	public static void benchmark1_2_05 (ODEService odeService) throws Exception
	{
		final int CONCEPT_UPDATES = benchmarkConstants.TEST_CONCEPTS;
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(CONCEPT_UPDATES);
		
		// Updates concepts one by one
		System.out.println("Updating concept: ");
		for (int i=1; i<=CONCEPT_UPDATES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CONCEPT_NAME+i);
				
			// Updates the concept
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				Term term = new Term(ONTOLOGY_NAME,CONCEPT_NAME+i,"concept updated "+i,Term.CONCEPT);
				odeService.updateTerm(CONCEPT_NAME+i,term);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR updating concept: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_2_05.txt");
	}

	/**
	 *	Updates several concepts in several ontologies
	 *
	 *	The database must have CONCEPT_UPDATES ontologies named ONTOLOGY_NAMEx
	 *	with one concept named CONCEPT_NAME
	 * MULTILINGUE*/
	public static void benchmark1_2_06 (ODEService odeService) throws Exception
	{
		final int CONCEPT_UPDATES = benchmarkConstants.TEST_CONCEPTS;
		final String CONCEPT_NAME = "concept1";
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(CONCEPT_UPDATES);
		
		// Updates concepts one by one
		System.out.println("Updating concept: ");
		for (int i=1; i<=CONCEPT_UPDATES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CONCEPT_NAME+"("+i+")");
				
			// Updates the concept
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				Term term = new Term(ONTOLOGY_NAME+i,CONCEPT_NAME,"concept updated "+i,Term.CONCEPT);
				odeService.updateTerm(CONCEPT_NAME,term);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR updating concept: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_2_06.txt");
	}

	/**
	 *	Updates several class attributes in a term
	 *
	 *	The database must have one ontology named ONTOLOGY_NAME
	 *	with a concept named CONCEPT_NAME
	 *	with CLASS_ATTRIBUTE_UPDATES class attributes named CLASS_ATTRIBUTE_NAMEx
	 */
	public static void benchmark1_2_07 (ODEService odeService) throws Exception
	{
		final int CLASS_ATTRIBUTE_UPDATES = benchmarkConstants.TEST_CLASS_ATTRIBUTES;
		final String CLASS_ATTRIBUTE_NAME = "classAttribute";
		final String CONCEPT_NAME = "concept1";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(CLASS_ATTRIBUTE_UPDATES);
		
		// Updates class attributes one by one
		System.out.println("Updating class attribute: ");
		for (int i=1; i<=CLASS_ATTRIBUTE_UPDATES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CLASS_ATTRIBUTE_NAME+i);
				
			// Updates the class attribute
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				ClassAttributeDescriptor cad = new ClassAttributeDescriptor(
					CLASS_ATTRIBUTE_NAME+i,CONCEPT_NAME,ValueTypes.STRING,ONTOLOGY_NAME,
					"measurement unit "+i,"precision"+i,1,2,
					"class attribute updated "+i+" of "+CONCEPT_NAME);
				odeService.updateClassAttribute(ONTOLOGY_NAME,CLASS_ATTRIBUTE_NAME+i,CONCEPT_NAME,cad);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR updating class attribute: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_2_07.txt");
	}

	/**
	 *	Updates several class attributes in several terms
	 *
	 *	The database must have one ontology named ONTOLOGY_NAME
	 *	with CLASS_ATTRIBUTE_UPDATES concepts named CONCEPT_NAMEx
	 *	with a class attribute named CLASS_ATTRIBUTE_NAME
	 */
	public static void benchmark1_2_08 (ODEService odeService) throws Exception
	{
		final int CLASS_ATTRIBUTE_UPDATES = benchmarkConstants.TEST_CLASS_ATTRIBUTES;
		final String CLASS_ATTRIBUTE_NAME = "classAttribute1";
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(CLASS_ATTRIBUTE_UPDATES);
		
		// Updates class attributes one by one
		System.out.println("Updating class attribute: ");
		for (int i=1; i<=CLASS_ATTRIBUTE_UPDATES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CLASS_ATTRIBUTE_NAME+"("+i+")");
				
			// Updates the class attribute
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				ClassAttributeDescriptor cad = new ClassAttributeDescriptor(
					CLASS_ATTRIBUTE_NAME,CONCEPT_NAME+i,ValueTypes.STRING,ONTOLOGY_NAME,
					"measurement unit "+i,"precision"+i,1,2,
					"class attribute updated "+i+" of "+CONCEPT_NAME+i);
				odeService.updateClassAttribute(ONTOLOGY_NAME,CLASS_ATTRIBUTE_NAME,CONCEPT_NAME+i,cad);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR updating class attribute: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_2_08.txt");
	}

	/**
	 *	Updates several instance attributes in a term
	 *
	 *	The database must have one ontology named ONTOLOGY_NAME
	 *	with a concept named CONCEPT_NAME
	 *	with INSTANCE_ATTRIBUTE_UPDATES instance attributes named INSTANCE_ATTRIBUTE_NAMEx
	 */
	public static void benchmark1_2_09 (ODEService odeService) throws Exception
	{
		final int INSTANCE_ATTRIBUTE_UPDATES = benchmarkConstants.TEST_INSTANCE_ATTRIBUTES;
		final String INSTANCE_ATTRIBUTE_NAME = "instanceAttribute";
		final String CONCEPT_NAME = "concept1";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(INSTANCE_ATTRIBUTE_UPDATES);
		
		// Updates instance attributes one by one
		System.out.println("Updating instance attribute: ");
		for (int i=1; i<=INSTANCE_ATTRIBUTE_UPDATES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+INSTANCE_ATTRIBUTE_NAME+i);
				
			// Updates the instance attribute
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				InstanceAttributeDescriptor iad = new InstanceAttributeDescriptor(
					INSTANCE_ATTRIBUTE_NAME+i,CONCEPT_NAME,ValueTypes.STRING,ONTOLOGY_NAME,
					"measurement unit "+i,"precision"+i,1,100,
					"instance attribute updated "+i+" of "+CONCEPT_NAME,"100","1");
				odeService.updateInstanceAttribute(ONTOLOGY_NAME,INSTANCE_ATTRIBUTE_NAME+i,CONCEPT_NAME,iad);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR updating instance attribute: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_2_09.txt");
	}

	/**
	 *	Updates several instance attributes in several terms
	 *
	 *	The database must have one ontology named ONTOLOGY_NAME
	 *	with INSTANCE_ATTRIBUTE_UPDATES concepts named CONCEPT_NAMEx
	 *	with an instance attribute named INSTANCE_ATTRIBUTE_NAME
	 */
	public static void benchmark1_2_10 (ODEService odeService) throws Exception
	{
		final int INSTANCE_ATTRIBUTE_UPDATES = benchmarkConstants.TEST_INSTANCE_ATTRIBUTES;
		final String INSTANCE_ATTRIBUTE_NAME = "instanceAttribute1";
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(INSTANCE_ATTRIBUTE_UPDATES);
		
		// Updates instance attributes one by one
		System.out.println("Updating instance attribute: ");
		for (int i=1; i<=INSTANCE_ATTRIBUTE_UPDATES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+INSTANCE_ATTRIBUTE_NAME+"("+i+")");
				
			// Updates the instance attribute
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				InstanceAttributeDescriptor iad = new InstanceAttributeDescriptor(
					INSTANCE_ATTRIBUTE_NAME,CONCEPT_NAME+i,ValueTypes.STRING,ONTOLOGY_NAME,
					"measurement unit "+i,"precision"+i,1,100,
					"instance attribute updated "+i+" of "+CONCEPT_NAME+i,"100","1");
				odeService.updateInstanceAttribute(ONTOLOGY_NAME,INSTANCE_ATTRIBUTE_NAME,CONCEPT_NAME+i,iad);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR updating instance attribute: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_2_10.txt");
	}

	/**
	 *	Updates several synonyms of a term
	 *
	 *	The database must have one ontology named ONTOLOGY_NAME
	 *	with a concept named CONCEPT_NAME
	 *	with SYNONYM_UPDATES synonyms named SYNONYM_NAMEx
	 */
	public static void benchmark1_2_11 (ODEService odeService) throws Exception
	{
		final int SYNONYM_UPDATES = benchmarkConstants.TEST_SYNONYMS;
		final String SYNONYM_NAME = "synonym";
		final String CONCEPT_NAME = "concept1";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(SYNONYM_UPDATES);
		
		// Updates synonyms one by one
		System.out.println("Updating synonym: ");
		for (int i=1; i<=SYNONYM_UPDATES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+SYNONYM_NAME+i);
				
			// Updates the synonym
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				SynonymAbbreviation sa = new SynonymAbbreviation(
																			SYNONYM_NAME+i, CONCEPT_NAME, null,    
																			TermTypes.SYNONYM, "synonym updated of "+CONCEPT_NAME);
				odeService.updateSynonym(ONTOLOGY_NAME,SYNONYM_NAME+i,sa);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR updating synonym: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_2_11.txt");
	}

	/**
	 *	Updates several synonyms of several terms
	 *
	 *	The database must have one ontology named ONTOLOGY_NAME
	 *	with SYNONYM_UPDATES concepts named CONCEPT_NAMEx
	 *	with a synonym named SYNONYM_NAME
	 */
	public static void benchmark1_2_12 (ODEService odeService) throws Exception
	{
		final int SYNONYM_UPDATES = benchmarkConstants.TEST_SYNONYMS;
		final String SYNONYM_NAME = "synonym1";
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(SYNONYM_UPDATES);
		
		// Updates synonyms one by one
		System.out.println("Updating synonym: ");
		for (int i=1; i<=SYNONYM_UPDATES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+SYNONYM_NAME+"("+i+")");
				
			// Updates the synonym
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				SynonymAbbreviation sa = new SynonymAbbreviation(
																			SYNONYM_NAME, CONCEPT_NAME+i, null,    
																			TermTypes.SYNONYM, "synonym updated of "+CONCEPT_NAME);
				odeService.updateSynonym(ONTOLOGY_NAME,SYNONYM_NAME,sa);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR updating synonym: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_2_12.txt");
	}

	/**
	 *	Updates several constants in an ontology
	 *
	 *	The database must have one ontology named ONTOLOGY_NAME
	 *	with CONSTANT_UPDATES constants named CONSTANT_NAMEx
	 */
	public static void benchmark1_2_13 (ODEService odeService) throws Exception
	{
		final int CONSTANT_UPDATES = benchmarkConstants.TEST_CONSTANTS;
		final String CONSTANT_NAME = "constant";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(CONSTANT_UPDATES);
		
		// Updates constants one by one
		System.out.println("Updating constant: ");
		for (int i=1; i<=CONSTANT_UPDATES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CONSTANT_NAME+i);
				
			// Updates the constant
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				ConstantDescriptor cd = new ConstantDescriptor(
																			CONSTANT_NAME+i, "constant updated "+i+" of "+ONTOLOGY_NAME,
																			ValueTypes.STRING,"measurement unit "+i,
																			"value of "+CONSTANT_NAME+i);
				odeService.updateConstant(ONTOLOGY_NAME,CONSTANT_NAME+i,cd);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR updating constant: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_2_13.txt");
	}

	/**
	 *	Updates several constants in several ontologies
	 *
	 *	The database must have CONSTANT_UPDATES ontologies named ONTOLOGY_NAMEx
	 *	with a constant named CONSTANT_NAME
	 */
	public static void benchmark1_2_14 (ODEService odeService) throws Exception
	{
		final int CONSTANT_UPDATES = benchmarkConstants.TEST_CONSTANTS;
		final String CONSTANT_NAME = "constant1";
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(CONSTANT_UPDATES);
		
		// Updates constants one by one
		System.out.println("Updating constant: ");
		for (int i=1; i<=CONSTANT_UPDATES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CONSTANT_NAME+"("+i+")");
				
			// Updates the constant
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				ConstantDescriptor cd = new ConstantDescriptor(
																			CONSTANT_NAME, "constant updated of "+ONTOLOGY_NAME+i,
																			ValueTypes.STRING,"measurement unit 1",
																			"value of "+CONSTANT_NAME);
				odeService.updateConstant(ONTOLOGY_NAME+i,CONSTANT_NAME,cd);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR updating constant: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_2_14.txt");
	}

	/**
	 *	Updates several formulas in an ontology
	 *
	 *	The database must have one ontology named ONTOLOGY_NAME
	 *	with FORMULA_UPDATES formulas named FORMULA_NAMEx
	 */
	public static void benchmark1_2_15 (ODEService odeService) throws Exception
	{
		final int FORMULA_UPDATES = benchmarkConstants.TEST_FORMULAS;
		final String FORMULA_NAME = "formula";
		final String ONTOLOGY_NAME = "ontology0";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(FORMULA_UPDATES);
		
		// Updates formulas one by one
		System.out.println("Updating formula: ");
		for (int i=1; i<=FORMULA_UPDATES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+FORMULA_NAME+i);
				
			// Updates the formula
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				FormulaDescriptor fd = new FormulaDescriptor(
													  ONTOLOGY_NAME,FORMULA_NAME+i, "formula updated "+i+" of "+ONTOLOGY_NAME,
														"Have friends and they will give you...",
														"person(?x), person(?y), friend(?x,?y) --> give(?y,?x)",
														Term.AXIOM);
				odeService.updateReasoningElement(FORMULA_NAME+i,fd);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR updating formula: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_2_15.txt");
	}

	/**
	 *	Updates several formulas in several ontologies
	 *
	 *	The database must have FORMULA_UPDATES ontologies named ONTOLOGY_NAMEx
	 *	with a formula named FORMULA_NAME
	 */
	public static void benchmark1_2_16 (ODEService odeService) throws Exception
	{
		final int FORMULA_UPDATES = benchmarkConstants.TEST_FORMULAS;
		final String FORMULA_NAME = "formula1";
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(FORMULA_UPDATES);
		
		// Updates formulas one by one
		System.out.println("Updating formula: ");
		for (int i=1; i<=FORMULA_UPDATES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+FORMULA_NAME+"("+i+")");
				
			// Updates the formula
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				FormulaDescriptor fd = new FormulaDescriptor(
													  ONTOLOGY_NAME+i,FORMULA_NAME, "formula updated "+i+" of "+ONTOLOGY_NAME+i,
														"Have friends and they will give you...",
														"person(?x), person(?y), friend(?x,?y) --> give(?y,?x)",
														Term.AXIOM);
				odeService.updateReasoningElement(FORMULA_NAME,fd);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR updating formula: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_2_16.txt");
	}

	/**
	 *	Updates several groups in an ontology
	 *
	 *	The database must have one ontology named ONTOLOGY_NAME
	 *	with GROUP_UPDATES groups named GROUP_NAMEx
	 *	with GROUP_UPDATES concepts names CONCEPT_NAMEx
	 */
	public static void benchmark1_2_17 (ODEService odeService) throws Exception
	{
		final int GROUP_UPDATES = benchmarkConstants.TEST_GROUPS;
		final String GROUP_NAME = "group";
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology0";
		
		String[] concept_list = new String[1];

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GROUP_UPDATES);
		
		// Updates groups one by one
		System.out.println("Updating group: ");
		for (int i=1; i<=GROUP_UPDATES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+GROUP_NAME+i);
				
			// Updates the group
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				concept_list[0] = CONCEPT_NAME+i;
				Group group = new Group(GROUP_NAME+i,"group updated "+i+" of "+ONTOLOGY_NAME,concept_list);
				odeService.updateGroup(ONTOLOGY_NAME,GROUP_NAME+i,group);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR updating group: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_2_17.txt");
	}

	/**
	 *	Updates several groups in several ontologies
	 *
	 *	The database must have GROUP_UPDATES ontologies named ONTOLOGY_NAMEx
	 *	with a group named GROUP_NAME
	 *	with a concept names CONCEPT_NAME
	 */
	public static void benchmark1_2_18 (ODEService odeService) throws Exception
	{
		final int GROUP_UPDATES = benchmarkConstants.TEST_GROUPS;
		final String GROUP_NAME = "group1";
		final String CONCEPT_NAME = "concept1";
		final String ONTOLOGY_NAME = "ontology";
		
		String[] concept_list = new String[1];

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GROUP_UPDATES);
		
		// Updates groups one by one
		System.out.println("Updating group: ");
		for (int i=1; i<=GROUP_UPDATES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+GROUP_NAME+"("+i+")");
				
			// Updates the group
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				concept_list[0] = CONCEPT_NAME;
				Group group = new Group(GROUP_NAME,"group updated of "+ONTOLOGY_NAME+i,concept_list);
				odeService.updateGroup(ONTOLOGY_NAME+i,GROUP_NAME,group);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR updating group: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark1_2_18.txt");
	}
}
