package es.upm.fi.dia.ontology.webode.wpbs;

// Java stuff
import java.util.Calendar;
import java.util.Date;

import es.upm.fi.dia.ontology.webode.Axiom.AxiomService;
import es.upm.fi.dia.ontology.webode.OntoClean.OntoCleanService;
import es.upm.fi.dia.ontology.webode.merge.MergeService;
import es.upm.fi.dia.ontology.webode.service.OntologyDescriptor;
import es.upm.fi.dia.ontology.webode.service.Term;
import es.upm.fi.dia.ontology.webode.service.WebODEException;
import es.upm.fi.dia.ontology.webode.translat.DAMLOIL.DAMLOILExportService;
import es.upm.fi.dia.ontology.webode.translat.OIL.OILExportService;
import es.upm.fi.dia.ontology.webode.translat.Prolog.PrologExportService;
import es.upm.fi.dia.ontology.webode.translat.XCARIN.XCARINExportService;
import es.upm.fi.dia.ontology.webode.translat.xml2java.JessExportService;
import es.upm.fi.dia.ontology.webode.xml.ExportService;

/** Implements a set of benchmarks for testing ODE service.
 *	All the benchmarks in this class test complex operations  
 */
public class benchmarkComplex
{

	/**
	 *	Parses several axioms of an ontology.
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 *	with PARSE_AXIOMS+1 concepts named CONCEPT_NAMEx
	 */
	public static void benchmark3_01 (AxiomService axiomService) throws Exception
	{
		final int PARSE_AXIOMS = benchmarkConstants.TEST_AXIOMS;
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology1";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(PARSE_AXIOMS);
		
		// Parses axioms one by one
		for (int i=1; i<=PARSE_AXIOMS; i++){
			String axiom = "forall ?x ("+CONCEPT_NAME+i+"(?x) -> "+CONCEPT_NAME+(i+1)+"(?x))";
			
			System.out.println("Parsing axiom: "+axiom);

			// Parses the axiom
			try{
				timer.start();
				axiomService.parseAxiom(ONTOLOGY_NAME,axiom);
				timer.stop();
			}
			catch (Exception e){
				throw new Exception("ERROR parsing axiom: "+e.getMessage());
			}
		}	
		
		timer.print();
		timer.save(benchmarkConstants.timer_destination_dir+"benchmark3_01.txt");		
	}	

	/**
	 *	Parses several axioms of several ontologies.
	 *
	 *	The database must have PARSE_AXIOMS ontologies named ONTOLOGY_NAMEx
	 *	with two concepts named CONCEPT_NAME1 and CONCEPT_NAME2
	 */
	public static void benchmark3_02 (AxiomService axiomService) throws Exception
	{
		final int PARSE_AXIOMS = benchmarkConstants.TEST_AXIOMS;
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(PARSE_AXIOMS);
		
		// Parses axioms one by one
		for (int i=1; i<=PARSE_AXIOMS; i++){
			String axiom = "forall ?x ("+CONCEPT_NAME+"1(?x) -> "+CONCEPT_NAME+"2(?x))";
			
			System.out.println("Parsing axiom: "+axiom);

			// Parses the axiom
			try{
				timer.start();
				axiomService.parseAxiom(ONTOLOGY_NAME+i,axiom);
				timer.stop();
			}
			catch (Exception e){
				throw new Exception("ERROR parsing axiom: "+e.getMessage());
			}
		}	
		
		timer.print();
		timer.save(benchmarkConstants.timer_destination_dir+"benchmark3_02.txt");		
	}	

	/**
	 *	Parses several rules of an ontology.
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 *	with PARSE_RULES+1 concepts named CONCEPT_NAMEx
	 */
	public static void benchmark3_03 (AxiomService axiomService) throws Exception
	{
		final int PARSE_RULES = benchmarkConstants.TEST_RULES;
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology1";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(PARSE_RULES);
		
		// Parses rules one by one
		for (int i=1; i<=PARSE_RULES; i++){
			String rule = "if "+CONCEPT_NAME+i+"(?x) then "+CONCEPT_NAME+(i+1)+"(?x)";
			
			System.out.println("Parsing rule: "+rule);

			// Parses the rule
			try{
				timer.start();
				axiomService.parseRule(ONTOLOGY_NAME,rule);
				timer.stop();
			}
			catch (Exception e){
				throw new Exception("ERROR parsing rule: "+e.getMessage());
			}
		}	
		
		timer.print();
		timer.save(benchmarkConstants.timer_destination_dir+"benchmark3_03.txt");		
	}	

	/**
	 *	Parses several rules of several ontologies.
	 *
	 *	The database must have PARSE_RULES ontologies named ONTOLOGY_NAMEx
	 *	with two concepts named CONCEPT_NAME1 and CONCEPT_NAME2
	 */
	public static void benchmark3_04 (AxiomService axiomService) throws Exception
	{
		final int PARSE_RULES = benchmarkConstants.TEST_RULES;
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(PARSE_RULES);
		
		// Parses rules one by one
		for (int i=1; i<=PARSE_RULES; i++){
			String rule = "if "+CONCEPT_NAME+"1(?x) then "+CONCEPT_NAME+"2(?x)";
			
			System.out.println("Parsing rule: "+rule);

			// Parses the rule
			try{
				timer.start();
				axiomService.parseRule(ONTOLOGY_NAME+i,rule);
				timer.stop();
			}
			catch (Exception e){
				throw new Exception("ERROR parsing rule: "+e.getMessage());
			}
		}	
		
		timer.print();
		timer.save(benchmarkConstants.timer_destination_dir+"benchmark3_04.txt");		
	}	

	/**
	 *	Merges several ontologies.
	 *
	 *	The database must have MERGE_ONTOLOGIES+1 ontologies named ONTOLOGY_NAMEx
	 */
	public static void benchmark3_05 (MergeService mergeService) throws Exception
	{
		final int MERGE_ONTOLOGIES = benchmarkConstants.TEST_MERGES;
		final String FILE_NAME = benchmarkConstants.merge_log_file; // El nombre del fichero de log
		final String TABLE_NAME1 = benchmarkConstants.merge_table1;	// Fichero con sinnimos (debe existir aunque este vaco)
		final String TABLE_NAME2 = benchmarkConstants.merge_table2;	// Fichero con hipernimos (debe existir aunque este vaco)
		final String MERGED_ONTOLOGY_NAME = "mergedOntology";
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(MERGE_ONTOLOGIES);

		// Merges ontologies two by two
		for (int i=0; i<MERGE_ONTOLOGIES; i++){
			System.out.println("merging ontology: "+MERGED_ONTOLOGY_NAME+i);

			// Merges ontologies
			try{
				timer.start();
				mergeService.mergeOntologies(FILE_NAME,ONTOLOGY_NAME+i,ONTOLOGY_NAME+(i+1),
																		 TABLE_NAME1,TABLE_NAME2,MERGED_ONTOLOGY_NAME+i,
																		 benchmarkConstants.MINERVA_USER,
																		 benchmarkConstants.MINERVA_GROUP);
				timer.stop();
			}
			catch (Exception e){
				throw new Exception("ERROR merging ontology: "+e.getMessage());
			}
		}	
		
		timer.print();
		timer.save(benchmarkConstants.timer_destination_dir+"benchmark3_05.txt");		
	}	

	/**
	 *  Exports several ontologies.
	 *
	 *	The dabatase must have EXPORT_ONTOLOGIES ontologies named ONTOLOGY_NAMEx
	 */
	public static void benchmark3_06 (ExportService exportService) throws Exception
	{
		final int EXPORT_ONTOLOGIES = benchmarkConstants.TEST_ONTOLOGIES;
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(EXPORT_ONTOLOGIES);

		for (int i=1; i<=EXPORT_ONTOLOGIES; i++){
			System.out.println("Exporting: "+ONTOLOGY_NAME+i);
				
			// Exports ontologies
			try{
				timer.start();
				exportService.export(ONTOLOGY_NAME+i,false,null,null,null);
				timer.stop();
			}
			catch (WebODEException e){
				throw new Exception("ERROR exporting ontologies: "+e.getMessage());
			}
		}	
		
		timer.print();
		timer.save(benchmarkConstants.timer_destination_dir+"benchmark3_06.txt");		
	}

	/**
	 *  Exports several ontologies to DAML+OIL.
	 *
	 *	The dabatase must have EXPORT_ONTOLOGIES ontologies named ONTOLOGY_NAMEx
	 */
	public static void benchmark3_07 (DAMLOILExportService exportService) throws Exception
	{
		final int EXPORT_ONTOLOGIES = benchmarkConstants.TEST_ONTOLOGIES;
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(EXPORT_ONTOLOGIES);

		for (int i=1; i<=EXPORT_ONTOLOGIES; i++){
			System.out.println("Exporting to DAML+OIL: "+ONTOLOGY_NAME+i);
				 
			// Exports ontologies to DAML+OIL
			try{
				timer.start();
				// Esto ha cambiado de la iteracin 3 a la 4
				// Se ha aadido el nombre del usuario que slo vale para crear el nombre del fichero destino
				//exportService.ExportOntologyDAMLOIL(ONTOLOGY_NAME+i,false,null,
				//																		"http://webode.dia.fi.upm.es/DAMLOIL/cosa#");
				exportService.ExportOntologyDAMLOIL(ONTOLOGY_NAME+i,"admin",false,null,
																						"http://webode.dia.fi.upm.es/DAMLOIL/cosa#");
				timer.stop();
			}
			catch (WebODEException e){
				throw new Exception("ERROR exporting ontologies to DAML+OIL: "+e.getMessage());
			}
		}	
		
		timer.print();
		timer.save(benchmarkConstants.timer_destination_dir+"benchmark3_07.txt");		
	}

	/**
	 *  Exports several ontologies to OIL.
	 *
	 *	The dabatase must have EXPORT_ONTOLOGIES ontologies named ONTOLOGY_NAMEx
	 */
	public static void benchmark3_08 (OILExportService exportService) throws Exception
	{
		final int EXPORT_ONTOLOGIES = benchmarkConstants.TEST_ONTOLOGIES;
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(EXPORT_ONTOLOGIES);

		for (int i=1; i<=EXPORT_ONTOLOGIES; i++){
			System.out.println("Exporting to OIL: "+ONTOLOGY_NAME+i);
				 
			// Exports ontologies to OIL
			try{
				timer.start();
				exportService.ExportOntologyOIL(ONTOLOGY_NAME+i,false,null);
				timer.stop();
			}
			catch (WebODEException e){
				throw new Exception("ERROR exporting ontologies to OIL: "+e.getMessage());
			}
		}	
		
		timer.print();
		timer.save(benchmarkConstants.timer_destination_dir+"benchmark3_08.txt");		
	}

	/**
	 *  Exports several ontologies to Prolog.
	 *
	 *	The dabatase must have EXPORT_ONTOLOGIES ontologies named ONTOLOGY_NAMEx
	 */
	public static void benchmark3_09 (PrologExportService exportService) throws Exception
	{
		final int EXPORT_ONTOLOGIES = benchmarkConstants.TEST_ONTOLOGIES;
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(EXPORT_ONTOLOGIES);

		for (int i=1; i<=EXPORT_ONTOLOGIES; i++){
			System.out.println("Exporting to Prolog: "+ONTOLOGY_NAME+i);
				 
			// Exports ontologies to Prolog
			try{
				timer.start();
				exportService.ExportOntologyProlog(ONTOLOGY_NAME+i,false,null);
				timer.stop();
			}
			catch (WebODEException e){
				throw new Exception("ERROR exporting ontologies to Prolog: "+e.getMessage());
			}
		}	
		
		timer.print();
		timer.save(benchmarkConstants.timer_destination_dir+"benchmark3_09.txt");		
	}

	/**
	 *  Exports several ontologies to XCARIN.
	 *
	 *	The dabatase must have EXPORT_ONTOLOGIES ontologies named ONTOLOGY_NAMEx
	 */
	public static void benchmark3_10 (XCARINExportService exportService) throws Exception
	{
		final int EXPORT_ONTOLOGIES = benchmarkConstants.TEST_ONTOLOGIES;
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(EXPORT_ONTOLOGIES);

		for (int i=1; i<=EXPORT_ONTOLOGIES; i++){
			System.out.println("Exporting to XCARIN: "+ONTOLOGY_NAME+i);
				 
			// Exports ontologies to XCARIN
			try{
				timer.start();
				exportService.ExportOntologyXCARIN(ONTOLOGY_NAME+i,false,null);
				timer.stop();
			}
			catch (WebODEException e){
				throw new Exception("ERROR exporting ontologies to XCARIN: "+e.getMessage());
			}
		}	
		
		timer.print();
		timer.save(benchmarkConstants.timer_destination_dir+"benchmark3_10.txt");		
	}

	/**
	 *  Exports several ontologies to Jess.
	 *
	 *	The dabatase must have EXPORT_ONTOLOGIES ontologies named ONTOLOGY_NAMEx
	 */
	public static void benchmark3_11 (JessExportService exportService) throws Exception
	{
		final int EXPORT_ONTOLOGIES = benchmarkConstants.TEST_ONTOLOGIES;
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(EXPORT_ONTOLOGIES);

		for (int i=1; i<=EXPORT_ONTOLOGIES; i++){
			System.out.println("Exporting to Jess: "+ONTOLOGY_NAME+i);
				 
			// Exports ontologies to Jess
			try{
				timer.start();
				exportService.ExportOntologyJess(ONTOLOGY_NAME+i,
																				 "D:\\rgarcia\\ont"+i+".xml","D:\\rgarcia\\ont"+i+".zip");
				timer.stop();
			}
			catch (WebODEException e){
				throw new Exception("ERROR exporting ontologies to Jess: "+e.getMessage());
			}
		}	
		
		timer.print();
		timer.save(benchmarkConstants.timer_destination_dir+"benchmark3_11.txt");		
	}

	/**
	 *  Creates several ontologies into OntoClean.
	 *
	 *	An ontology named 'top level ontology of universals' must exist.
	 * MULTILINGUE*/
	public static void benchmark3_12 (OntoCleanService ocService) throws Exception
	{
		final int NEW_ONTOLOGIES = benchmarkConstants.TEST_ONTOLOGIES;
		final String ONTOLOGY_NAME = "newOntologyOC";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_ONTOLOGIES);
		
		// Creates ontologies one by one
		for (int i=1; i<=NEW_ONTOLOGIES; i++){
			System.out.println("Creating ontology: "+ONTOLOGY_NAME+i);
				
			// Creates the ontology
			try{
				timer.start();
				Date now=Calendar.getInstance().getTime();
				OntologyDescriptor od = new OntologyDescriptor(ONTOLOGY_NAME+i,"new ontology OC"+i,
																										 	 "admin","admin",now,now);
				ocService.createOntology(od);
				timer.stop();
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating ontology: "+e.getMessage());
			}
		}	
		
		timer.print();
		timer.save(benchmarkConstants.timer_destination_dir+"benchmark3_12.txt");		
	}

	/**
	 *	Inserts several concepts in an ontology in OntoClean.
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 *	An ontology named 'top level ontology of universals' must exist.
	 */
	public static void benchmark3_13 (OntoCleanService ocService) throws Exception
	{
		final int NEW_CONCEPTS = benchmarkConstants.TEST_CONCEPTS;
		final String CONCEPT_NAME = "newConcept";
		final String ONTOLOGY_NAME = "newOntologyOC1";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(NEW_CONCEPTS);
		
		// Creates concepts one by one
		for (int i=1; i<=NEW_CONCEPTS; i++){
			System.out.println("Creating concept: "+CONCEPT_NAME+i);

			// Creates the concept
			try{
				timer.start();
				ocService.insertTerm(ONTOLOGY_NAME,CONCEPT_NAME+i,"concept "+i+" of "+ONTOLOGY_NAME,
														 Term.CONCEPT,1,1,1,1,1,1,1);
				timer.stop();
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating concept: "+e.getMessage());
			}
		}	
		
		timer.print();
		timer.save(benchmarkConstants.timer_destination_dir+"benchmark3_13.txt");		
	}	

	/**
	 *	Updates several concepts in an ontology in OntoClean
	 *
	 *	The database must have one ontology named ONTOLOGY_NAME
	 *	with CONCEPT_UPDATES concepts named CONCEPT_NAMEx
	 *	An ontology named 'top level ontology of universals' must exist.
	 * MULTILINGUE*/
	public static void benchmark3_14 (OntoCleanService ocService) throws Exception
	{
		final int CONCEPT_UPDATES = benchmarkConstants.TEST_CONCEPTS;
		final String CONCEPT_NAME = "newConcept";
		final String ONTOLOGY_NAME = "newOntologyOC1";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(CONCEPT_UPDATES);
		
		// Updates concepts one by one
		for (int i=1; i<=CONCEPT_UPDATES; i++){
			System.out.println("Updating concept: "+CONCEPT_NAME+i);
				
			// Updates the concept
			try{
				timer.start();
				Term term = new Term(ONTOLOGY_NAME,CONCEPT_NAME+i,"concept updated "+i,Term.CONCEPT);
				ocService.updateTerm(CONCEPT_NAME+i,null,term,1,1,1,1,1,1,1);
				timer.stop();
			}
			catch (WebODEException e){
				throw new Exception("ERROR updating concept: "+e.getMessage());
			}
		}	
		
		timer.print();
		timer.save(benchmarkConstants.timer_destination_dir+"benchmark3_14.txt");		
	}

	/**
	 *	Evaluates several ontologies from OntoClean
	 *
	 *	The database must have EVALUATED_ONTOLOGIES ontologies named ONTOLOGY_NAMEx
	 *	An ontology named 'top level ontology of universals' must exist.
	 */
	public static void benchmark3_15 (OntoCleanService ocService) throws Exception
	{
		final int EVALUATED_ONTOLOGIES = benchmarkConstants.TEST_ONTOLOGIES;
		final String ONTOLOGY_NAME = "newOntologyOC";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(EVALUATED_ONTOLOGIES);
		
		// Evaluates ontologies one by one
		for (int i=1; i<=EVALUATED_ONTOLOGIES; i++){
			System.out.println("Evaluating ontology: "+ONTOLOGY_NAME+i);
				
			// Evaluates the ontology
			try{
				timer.start();
				ocService.evaluationOntoClean(ONTOLOGY_NAME+i,benchmarkConstants.MINERVA_USER," ");
				timer.stop();
			}
			catch (WebODEException e){
				throw new Exception("ERROR evaluating ontology: "+e.getMessage());
			}
		}	
		
		timer.print();
		timer.save(benchmarkConstants.timer_destination_dir+"benchmark3_15.txt");		
	}

	/**
	 *	Removes several concepts from an ontology in OntoClean
	 *
	 *	The database must have one ontology named ONTOLOGY_NAME
	 *	with REMOVED_CONCEPTS concepts named CONCEPT_NAMEx
	 *	An ontology named 'top level ontology of universals' must exist.
	 */
	public static void benchmark3_16 (OntoCleanService ocService) throws Exception
	{
		final int REMOVED_CONCEPTS = benchmarkConstants.TEST_CONCEPTS;
		final String CONCEPT_NAME = "newConcept";
		final String ONTOLOGY_NAME = "newOntologyOC1";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(REMOVED_CONCEPTS);
		
		// Removes concepts one by one
		for (int i=1; i<=REMOVED_CONCEPTS; i++){
			System.out.println("Removing concept: "+CONCEPT_NAME+i);
				
			// Removes the concept
			try{
				timer.start();
				ocService.removeTerm(ONTOLOGY_NAME,CONCEPT_NAME+i,null,null);
				timer.stop();
			}
			catch (WebODEException e){
				throw new Exception("ERROR removing concept: "+e.getMessage());
			}
		}	
		
		timer.print();
		timer.save(benchmarkConstants.timer_destination_dir+"benchmark3_16.txt");		
	}

	/**
	 *	Removes several ontologies from OntoClean
	 *
	 *	The database must have REMOVED_ONTOLOGIES ontologies named ONTOLOGY_NAMEx
	 *	An ontology named 'top level ontology of universals' must exist.
	 */
	public static void benchmark3_17 (OntoCleanService ocService) throws Exception
	{
		final int REMOVED_ONTOLOGIES = benchmarkConstants.TEST_ONTOLOGIES;
		final String ONTOLOGY_NAME = "newOntologyOC";

		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(REMOVED_ONTOLOGIES);
		
		// Removes ontologies one by one
		for (int i=1; i<=REMOVED_ONTOLOGIES; i++){
			System.out.println("Removing ontology: "+ONTOLOGY_NAME+i);
				
			// Removes the ontology
			try{
				timer.start();
				ocService.removeOntology(benchmarkConstants.MINERVA_USER,ONTOLOGY_NAME+i);
				timer.stop();
			}
			catch (WebODEException e){
				throw new Exception("ERROR removing ontology: "+e.getMessage());
			}
		}	
		
		timer.print();
		timer.save(benchmarkConstants.timer_destination_dir+"benchmark3_17.txt");		
	}
}