package es.upm.fi.dia.ontology.webode.wpbs;

// Java stuff
import java.io.File;


public class benchmarkGeneral
{
	/**
	 * Saves the timer data according to a hierarchized structure
	 */
	public static void save_timer_data(benchmarkTimer timer, String filename) throws Exception
	{
		// timer.print();

		String f = Integer.toString(benchmarkConstants.initial_state_factor);
		if (f.length() <= 1) f = "000"+f;
		if (f.length() <= 2) f = "00"+f;
		if (f.length() <= 3) f = "0"+f;

		String n = Integer.toString(benchmarkConstants.n_times);
		if (n.length() <= 1) n = "000"+n;
		if (n.length() <= 2) n = "00"+n;
		if (n.length() <= 3) n = "0"+n;

		String restoredir;
		if (benchmarkConstants.restore){
			restoredir = "restore";
		} else {
			restoredir = "no_restore";
		}

		String timer_destination_dir = benchmarkConstants.destination_dir +
									 								 restoredir+java.io.File.separator+
																	 "f"+f+java.io.File.separator+
																	 "n"+n+java.io.File.separator;

		// Creates the destination dir if it doesn't exists
		File dir = new File(timer_destination_dir);
		if (!(dir.exists() && dir.isDirectory())) dir.mkdirs() ;

		timer.save(timer_destination_dir+filename);
	}

	/**
	 * Creates ontologies for benchmarking.
	 *
	 *  Creates x ontologies with:
	 *
	 *		- x concepts:										concept1..x
	 *		- x term references:						termReference1..x
	 *		- x ontology references:				ontologyReference1..x
	 *		- x class attributes:						classAttribute1..x
	 *		- x instance attributes:				instanceAttribute1..x
	 *		- x formulas:										formula1..x
	 *		- x instances:									instance1..x
	 *		- x synonyms:										synonym1..x
	 *		- x constants:									constant1..x
	 *		- x groups:											group1..x
	 *		- x term relations:							termRelation1..x
	 *		- x class attribute values:			classAttributeValue1..x
	 *		- x instance attribute values:	instanceAttributeValue1..x
	 *		- x imported terms:							importedTerm1..x
	 *		- x instance sets:							instanceSet1..x
	 *
	 *	where every x is a constant
	 */
	/* This method is not used actually, as ontologies are created in the initial states
	public static void createOntologies (ODEService odeService) throws Exception
	{
		final int COMMON_NUMBER = 20;

		final int ONTOLOGY_NUMBER = 								20;

		final int CONCEPT_NUMBER = 									COMMON_NUMBER;
		final int TERM_REFERENCE_NUMBER = 					COMMON_NUMBER;
		final int ONTOLOGY_REFERENCE_NUMBER = 			COMMON_NUMBER;
		final int FORMULA_NUMBER = 									COMMON_NUMBER;
		final int SYNONYM_NUMBER = 									COMMON_NUMBER;
		final int CONSTANT_NUMBER = 								COMMON_NUMBER;
		final int IMPORTED_TERM_NUMBER = 						COMMON_NUMBER;
		final int CLASS_ATTRIBUTE_NUMBER = 					COMMON_NUMBER;
		final int CLASS_ATTRIBUTE_VALUE_NUMBER = 		CLASS_ATTRIBUTE_NUMBER;
		final int INSTANCE_ATTRIBUTE_NUMBER = 			COMMON_NUMBER;
		final int INSTANCE_ATTRIBUTE_VALUE_NUMBER = INSTANCE_ATTRIBUTE_NUMBER;
		final int TERM_RELATION_NUMBER = 						CONCEPT_NUMBER-1;
		final int GROUP_NUMBER = 										COMMON_NUMBER;
		final int INSTANCE_SET_NUMBER = 						COMMON_NUMBER;
		final int INSTANCE_NUMBER = 								COMMON_NUMBER;

		// Creates the ontologies one by one
		for (int i=1; i<=ONTOLOGY_NUMBER; i++){
			System.out.println("Creating ontology: ontology"+i);

			// Creates ontology
			try{
				Date now=Calendar.getInstance().getTime();
				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"ontology "+i,
																										 "admin","admin",now,now);
				odeService.createOntology(od);
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating ontology: "+e.getMessage());
			}

			// Creates concepts
			if (CONCEPT_NUMBER>0){
				System.out.print("  - Creating concepts:                  ");
				for (int j=1; j<=CONCEPT_NUMBER;j++){
					System.out.print(".");
					odeService.insertTerm("ontology"+i,"concept"+j,
																"concept "+j+" of ontology "+i,Term.CONCEPT);
				}
				System.out.println();
			}

			// Creates term references and relates it to concepts
			if ((TERM_REFERENCE_NUMBER>0) &&
					(CONCEPT_NUMBER>0) &&
					(TERM_REFERENCE_NUMBER>=CONCEPT_NUMBER)){
				System.out.print("  - Creating term references:           ");
				for (int j=1; j<=TERM_REFERENCE_NUMBER;j++){
					System.out.print(".");
					ReferenceDescriptor rd = new ReferenceDescriptor("termReference"+j,
																													 "reference "+j+" of concept"+j);
					odeService.addReference("ontology"+i,rd);
					odeService.relateReferenceToTerm("ontology"+i,"concept"+j,"termReference"+j);
				}
				System.out.println();
			}

			// Creates ontology references
			if (ONTOLOGY_REFERENCE_NUMBER>0){
				System.out.print("  - Creating ontology references:       ");
				for (int j=1; j<=ONTOLOGY_REFERENCE_NUMBER;j++){
					System.out.print(".");
					ReferenceDescriptor rd = new ReferenceDescriptor("ontologyReference"+j,
																													 "reference "+j+" of ontology"+i);
					odeService.addReferenceToOntology("ontology"+i,rd);
				}
				System.out.println();
			}

			// Creates formulas
			if (FORMULA_NUMBER>0){
				System.out.print("  - Creating formulas:                  ");
				for (int j=1; j<=FORMULA_NUMBER;j++){
					System.out.print(".");
					FormulaDescriptor fd = new FormulaDescriptor(
														  "ontology"+i,"formula"+j, "formula "+j+" of ontology"+i,
															"Have friends and they will give you...",
															"person(?x), person(?y), friend(?x,?y) --> give(?y,?x)",
															Term.AXIOM);
					odeService.insertReasoningElement(fd);
				}
				System.out.println();
			}

			// Creates synonyms
			if (SYNONYM_NUMBER>0){
				System.out.print("  - Creating synonyms:                  ");
				for (int j=1; j<=SYNONYM_NUMBER;j++){
					System.out.print(".");
					SynonymAbbreviation sa = new SynonymAbbreviation(
																				"synonym"+j, "concept"+j, null,
																				TermTypes.SYNONYM, "synonym of concept"+j);
					odeService.addSynonymToTerm("ontology"+i,sa);
				}
				System.out.println();
			}

			// Creates constants
			if (CONSTANT_NUMBER>0){
				System.out.print("  - Creating constants:                 ");
				for (int j=1; j<=CONSTANT_NUMBER;j++){
					System.out.print(".");
					ConstantDescriptor cd = new ConstantDescriptor(
																				"constant"+j, "constant "+j+" of ontology "+i,
																				ValueTypes.STRING,"measurement unit "+j,
																				"value of constant"+j);
					odeService.insertConstant("ontology"+i,cd);
				}
				System.out.println();
			}

			// Creates imported terms
			if (IMPORTED_TERM_NUMBER>0){
				System.out.print("  - Creating imported terms:            ");
				for (int j=1; j<=IMPORTED_TERM_NUMBER;j++){
					System.out.print(".");
					odeService.importTerm("ontology"+i,"originalName"+j,"importedTerm"+j,
																"http://www.noexists.com/originalName"+j);
				}
				System.out.println();
			}

			// Creates class attributes
			if ((CLASS_ATTRIBUTE_NUMBER>0) && (CONCEPT_NUMBER>0)){
				System.out.print("  - Creating class attributes:          ");
				for (int j=1; j<=CONCEPT_NUMBER;j++){
					for (int k=1; k<=CLASS_ATTRIBUTE_NUMBER; k++){
						System.out.print(".");
						ClassAttributeDescriptor cad = new ClassAttributeDescriptor(
							"classAttribute"+k,"concept"+j,ValueTypes.STRING,"ontology"+i,
							"measurement unit "+k,"precision"+k,1,100,
							"class attribute "+k+" of concept "+j);
						odeService.insertClassAttribute("ontology"+i,cad);
					}
				}
				System.out.println();
			}

			// Creates class attribute values
			if ((CONCEPT_NUMBER>0) &&
					(CLASS_ATTRIBUTE_NUMBER>0) &&
					(CLASS_ATTRIBUTE_NUMBER>=CLASS_ATTRIBUTE_VALUE_NUMBER)){
				System.out.print("  - Creating class attribute values:    ");
				for (int j=1; j<=CONCEPT_NUMBER;j++){
					for (int k=1; k<=CLASS_ATTRIBUTE_VALUE_NUMBER; k++){
						System.out.print(".");
						odeService.addValueToClassAttribute("ontology"+i,"classAttribute"+k,
																								"concept"+j,"value"+k);
					}
				}
				System.out.println();
			}

			// Creates instance attributes
			if ((INSTANCE_ATTRIBUTE_NUMBER>0) && (CONCEPT_NUMBER>0)){
				System.out.print("  - Creating instance attributes:       ");
				for (int j=1; j<=CONCEPT_NUMBER;j++){
					for (int k=1; k<=INSTANCE_ATTRIBUTE_NUMBER; k++){
						System.out.print(".");
						InstanceAttributeDescriptor iad = new InstanceAttributeDescriptor(
							"instanceAttribute"+k,"concept"+j,ValueTypes.STRING,"ontology"+i,
							"measurement unit "+k,"precision"+k,1,100,
							"instance attribute "+k+" of concept "+j,"100","1");
						odeService.insertInstanceAttribute("ontology"+i,iad);
					}
				}
				System.out.println();
			}

			// Creates instance attribute values
			if ((CONCEPT_NUMBER>0) &&
					(INSTANCE_ATTRIBUTE_NUMBER>0) &&
					(INSTANCE_ATTRIBUTE_NUMBER>=INSTANCE_ATTRIBUTE_VALUE_NUMBER)){
				System.out.print("  - Creating instance attribute values: ");
				for (int j=1; j<=CONCEPT_NUMBER;j++){
					for (int k=1; k<=INSTANCE_ATTRIBUTE_VALUE_NUMBER; k++){
						System.out.print(".");
						odeService.addValueToInstanceAttribute("ontology"+i,"instanceAttribute"+k,
																								   "concept"+j,"value"+k);
					}
				}
				System.out.println();
			}

			// Creates term relations
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>TERM_RELATION_NUMBER)){
				System.out.print("  - Creating term relations:            ");
				for (int j=1; j<=TERM_RELATION_NUMBER;j++){
					System.out.print(".");
					TermRelation tr = new TermRelation("ontology"+i,null,"termRelation"+j,
																						 "concept"+j,"concept"+(j+1));
					odeService.insertTermRelation(tr);
				}
				System.out.println();
			}

			// Creates groups
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>=GROUP_NUMBER)){
				System.out.print("  - Creating groups:                    ");
				String[] concept_list = new String[1];
				for (int j=1; j<=GROUP_NUMBER;j++){
					System.out.print(".");
					concept_list[0] = "concept"+j;
					Group group = new Group("group"+j,"group "+j+" of ontology"+i,concept_list);
					odeService.addGroup("ontology"+i,group);
				}
				System.out.println();
			}

			// Creates instance sets
			if (INSTANCE_SET_NUMBER>0){
				System.out.print("  - Creating instance sets:             ");
				for (int j=1; j<=INSTANCE_SET_NUMBER;j++){
					System.out.print(".");
					odeService.insertTerm("ontology"+i,"instanceSet"+j,
																"instance set "+j+" of ontology"+i,TermTypes.INSTANCE_SET);
				}
				System.out.println();
			}

			// Creates instances
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>=INSTANCE_NUMBER) &&
					(INSTANCE_SET_NUMBER>0)){
				System.out.print("  - Creating instances:                 ");
				for (int j=1; j<=INSTANCE_NUMBER;j++){
					System.out.print(".");
					Instance instance = new Instance("instance"+j,"concept"+j,"instanceSet1",
																					 "instance "+j+" of ontology"+i);
					odeService.insertInstance("ontology"+i,instance);
				}
				System.out.println();
			}
		}
	}
	*/
}
