package es.upm.fi.dia.ontology.webode.wpbs;

// Java stuff
import java.util.Calendar;
import java.util.Date;

import es.upm.fi.dia.ontology.webode.service.ClassAttributeDescriptor;
import es.upm.fi.dia.ontology.webode.service.ConstantDescriptor;
import es.upm.fi.dia.ontology.webode.service.FormulaDescriptor;
import es.upm.fi.dia.ontology.webode.service.Group;
import es.upm.fi.dia.ontology.webode.service.Instance;
import es.upm.fi.dia.ontology.webode.service.InstanceAttributeDescriptor;
import es.upm.fi.dia.ontology.webode.service.ODEService;
import es.upm.fi.dia.ontology.webode.service.OntologyDescriptor;
import es.upm.fi.dia.ontology.webode.service.ReferenceDescriptor;
import es.upm.fi.dia.ontology.webode.service.SynonymAbbreviation;
import es.upm.fi.dia.ontology.webode.service.Term;
import es.upm.fi.dia.ontology.webode.service.TermRelation;
import es.upm.fi.dia.ontology.webode.service.TermRelationInstance;
import es.upm.fi.dia.ontology.webode.service.TermTypes;
import es.upm.fi.dia.ontology.webode.service.ValueTypes;
import es.upm.fi.dia.ontology.webode.service.WebODEException;

/**
 *	Creates the database initial state for the benchmarks
 */
public class benchmarkInitialState
{

  /**
   * Deletes all ontologies in the database
   */
  public static void deleteOntologies (ODEService odeService) throws Exception
  {
          OntologyDescriptor[] ontologies = odeService.getAvailableOntologies();
          System.out.println("Deleting: ");
          for (int i=0; i<ontologies.length; i++){
                  System.out.print("\r  ("+i+")"+ontologies[i].name+"      ");
                  odeService.removeOntology(ontologies[i].name);
          }
          System.out.println("");

          System.out.println("All ontologies have been deleted");
  }

	/**
	 *	Creates the initial state for the basic insert group of benchmarks
	 */
	public static void initialStateBasicInsert (ODEService odeService) throws Exception
	{
		final int factor = benchmarkConstants.initial_state_factor;

		final int ONTOLOGY_NUMBER = 								factor;

		final int CONCEPT_NUMBER = 									factor+1;
		final int TERM_REFERENCE_NUMBER = 					factor;
		final int FORMULA_NUMBER = 									factor;
		final int CLASS_ATTRIBUTE_NUMBER = 					factor;
		final int INSTANCE_ATTRIBUTE_NUMBER = 			factor;
		final int TERM_RELATION_NUMBER = 						factor; // CONCEPT_NUMBER-1
		final int INSTANCE_SET_NUMBER = 						1;
		final int INSTANCE_NUMBER = 								factor+1;

		// Creates the ontologies one by one
		for (int i=0; i<=ONTOLOGY_NUMBER; i++){
			System.out.println("Creating ontology: ontology"+i);

			// Creates ontology
			try{
				Date now=Calendar.getInstance().getTime();
				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 	 "admin","admin",now,now);
/* MULTILINGUE				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 	 "admin","admin",0,now,now);*/
				odeService.createOntology(od);
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating ontology: "+e.getMessage());
			}

			// Creates concepts
			if (CONCEPT_NUMBER>0){
				System.out.print("  - Creating concepts:                  ");
				for (int j=1; j<=CONCEPT_NUMBER;j++){
					System.out.print(".");
					odeService.insertTerm("ontology"+i,"concept"+j,
																"concept "+j+" of ontology "+i,Term.CONCEPT);
				}
				System.out.println();
			}

			// Creates term references and DOESN'T relates it to concepts
			if (TERM_REFERENCE_NUMBER>0){
				System.out.print("  - Creating term references:           ");
				for (int j=1; j<=TERM_REFERENCE_NUMBER;j++){
					System.out.print(".");
					ReferenceDescriptor rd = new ReferenceDescriptor("termReference"+j,
																													 "reference "+j+" of concept"+j);
					odeService.addReference("ontology"+i,rd);
				}
				System.out.println();
			}

			// Creates formulas
			if (FORMULA_NUMBER>0){
				System.out.print("  - Creating formulas:                  ");
				for (int j=1; j<=FORMULA_NUMBER;j++){
					System.out.print(".");
					FormulaDescriptor fd = new FormulaDescriptor(
														  "ontology"+i,"formula"+j, "formula "+j+" of ontology"+i,
															"Have friends and they will give you...",
															"person(?x), person(?y), friend(?x,?y) --> give(?y,?x)",
															Term.AXIOM);
					odeService.insertReasoningElement(fd);
				}
				System.out.println();
			}

			// Creates class attributes
			if ((CLASS_ATTRIBUTE_NUMBER>0) && (CONCEPT_NUMBER>0)){
				System.out.print("  - Creating class attributes:          ");
				for (int j=1; j<=CONCEPT_NUMBER;j++){
					for (int k=1; k<=CLASS_ATTRIBUTE_NUMBER; k++){
						System.out.print(".");
						ClassAttributeDescriptor cad = new ClassAttributeDescriptor(
							"classAttribute"+k,"concept"+j,ValueTypes.STRING,"ontology"+i,
							"measurement unit "+k,"precision"+k,1,100000,
							"class attribute "+k+" of concept "+j);
						odeService.insertClassAttribute("ontology"+i,cad);
					}
				}
				System.out.println();
			}

			// Creates instance attributes
			if ((INSTANCE_ATTRIBUTE_NUMBER>0) && (CONCEPT_NUMBER>0)){
				System.out.print("  - Creating instance attributes:       ");
				for (int j=1; j<=CONCEPT_NUMBER;j++){
					for (int k=1; k<=INSTANCE_ATTRIBUTE_NUMBER; k++){
						System.out.print(".");
						InstanceAttributeDescriptor iad = new InstanceAttributeDescriptor(
							"instanceAttribute"+k,"concept"+j,ValueTypes.STRING,"ontology"+i,
							"measurement unit "+k,"precision"+k,1,100000,
							"instance attribute "+k+" of concept "+j,"100","1");
						odeService.insertInstanceAttribute("ontology"+i,iad);
					}
				}
				System.out.println();
			}

			// Creates term relations
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>TERM_RELATION_NUMBER)){
				System.out.print("  - Creating term relations:            ");
				for (int j=1; j<=TERM_RELATION_NUMBER;j++){
					System.out.print(".");
					TermRelation tr = new TermRelation("ontology"+i,null,"termRelation"+j,
																						 "concept"+j,"concept"+(j+1));
					odeService.insertTermRelation(tr);
				}
				System.out.println();
			}

			// Creates instance sets
			if (INSTANCE_SET_NUMBER>0){
				System.out.print("  - Creating instance sets:             ");
				for (int j=1; j<=INSTANCE_SET_NUMBER;j++){
					System.out.print(".");
					odeService.insertTerm("ontology"+i,"instanceSet"+j,
																"instance set "+j+" of ontology"+i,TermTypes.INSTANCE_SET);
				}
				System.out.println();
			}

			// Creates instances
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>=INSTANCE_NUMBER) &&
					(INSTANCE_SET_NUMBER>0)){
				System.out.print("  - Creating instances:                 ");
				for (int j=1; j<=INSTANCE_NUMBER;j++){
					System.out.print(".");
					Instance instance = new Instance("instance"+j,"concept"+j,"instanceSet1",
																					 "instance "+j+" of ontology"+i);
					odeService.insertInstance("ontology"+i,instance);
				}
				System.out.println();
			}
		}
	}

	/**
	 *	Creates the minimal initial state for the basic insert group of benchmarks
	 */
	public static void initialStateBasicInsertMinimal (ODEService odeService) throws Exception
	{
		final int factor = benchmarkConstants.initial_state_factor;

		final int ONTOLOGY_NUMBER = 								factor;

		final int CONCEPT_NUMBER = 									factor+1;
		final int TERM_REFERENCE_NUMBER = 					factor;
		final int FORMULA_NUMBER = 									factor;
		final int CLASS_ATTRIBUTE_NUMBER = 					factor;
		final int INSTANCE_ATTRIBUTE_NUMBER = 			factor;
		final int TERM_RELATION_NUMBER = 						factor; // CONCEPT_NUMBER-1
		final int INSTANCE_SET_NUMBER = 						1;
		final int INSTANCE_NUMBER = 								factor+1;

		// Creates ontology0
		for (int i=0; i<=0; i++){
			System.out.println("Creating ontology: ontology"+i);

			// Creates ontology
			try{
				Date now=Calendar.getInstance().getTime();
				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 "admin","admin",now,now);
/* MULTILINGUE				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 	 "admin","admin",0,now,now);*/
				odeService.createOntology(od);
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating ontology: "+e.getMessage());
			}

			// Creates concepts
			if (CONCEPT_NUMBER>0){
				System.out.print("  - Creating concepts:                  ");
				for (int j=1; j<=CONCEPT_NUMBER;j++){
					System.out.print(".");
					odeService.insertTerm("ontology"+i,"concept"+j,
																"concept "+j+" of ontology "+i,Term.CONCEPT);
				}
				System.out.println();
			}

			// Creates term references and DOESN'T relates it to concepts
			if (TERM_REFERENCE_NUMBER>0){
				System.out.print("  - Creating term references:           ");
				for (int j=1; j<=TERM_REFERENCE_NUMBER;j++){
					System.out.print(".");
					ReferenceDescriptor rd = new ReferenceDescriptor("termReference"+j,
																													 "reference "+j+" of concept"+j);
					odeService.addReference("ontology"+i,rd);
				}
				System.out.println();
			}

			// Creates formulas
			if (FORMULA_NUMBER>0){
				System.out.print("  - Creating formulas:                  ");
				for (int j=1; j<=FORMULA_NUMBER;j++){
					System.out.print(".");
					FormulaDescriptor fd = new FormulaDescriptor(
														  "ontology"+i,"formula"+j, "formula "+j+" of ontology"+i,
															"Have friends and they will give you...",
															"person(?x), person(?y), friend(?x,?y) --> give(?y,?x)",
															Term.AXIOM);
					odeService.insertReasoningElement(fd);
				}
				System.out.println();
			}

			// Creates class attributes
			if ((CLASS_ATTRIBUTE_NUMBER>0) && (CONCEPT_NUMBER>0)){
				System.out.print("  - Creating class attributes:          ");
				for (int j=1; j<=1;j++){
					for (int k=1; k<=CLASS_ATTRIBUTE_NUMBER; k++){
						System.out.print(".");
						ClassAttributeDescriptor cad = new ClassAttributeDescriptor(
							"classAttribute"+k,"concept"+j,ValueTypes.STRING,"ontology"+i,
							"measurement unit "+k,"precision"+k,1,100000,
							"class attribute "+k+" of concept "+j);
						odeService.insertClassAttribute("ontology"+i,cad);
					}
				}
				System.out.println();
			}

			// Creates instance attributes
			if ((INSTANCE_ATTRIBUTE_NUMBER>0) && (CONCEPT_NUMBER>0)){
				System.out.print("  - Creating instance attributes:       ");
				for (int j=1; j<=1;j++){
					for (int k=1; k<=INSTANCE_ATTRIBUTE_NUMBER; k++){
						System.out.print(".");
						InstanceAttributeDescriptor iad = new InstanceAttributeDescriptor(
							"instanceAttribute"+k,"concept"+j,ValueTypes.STRING,"ontology"+i,
							"measurement unit "+k,"precision"+k,1,100000,
							"instance attribute "+k+" of concept "+j,"100","1");
						odeService.insertInstanceAttribute("ontology"+i,iad);
					}
				}
				System.out.println();
			}

			// Creates term relations
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>TERM_RELATION_NUMBER)){
				System.out.print("  - Creating term relations:            ");
				for (int j=1; j<=TERM_RELATION_NUMBER;j++){
					System.out.print(".");
					TermRelation tr = new TermRelation("ontology"+i,null,"termRelation"+j,
																						 "concept"+j,"concept"+(j+1));
					odeService.insertTermRelation(tr);
				}
				System.out.println();
			}

			// Creates instance sets
			if (INSTANCE_SET_NUMBER>0){
				System.out.print("  - Creating instance sets:             ");
				for (int j=1; j<=INSTANCE_SET_NUMBER;j++){
					System.out.print(".");
					odeService.insertTerm("ontology"+i,"instanceSet"+j,
																"instance set "+j+" of ontology"+i,TermTypes.INSTANCE_SET);
				}
				System.out.println();
			}

			// Creates instances
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>=INSTANCE_NUMBER) &&
					(INSTANCE_SET_NUMBER>0)){
				System.out.print("  - Creating instances:                 ");
				for (int j=1; j<=INSTANCE_NUMBER;j++){
					System.out.print(".");
					Instance instance = new Instance("instance"+j,"concept"+j,"instanceSet1",
																					 "instance "+j+" of ontology"+i);
					odeService.insertInstance("ontology"+i,instance);
				}
				System.out.println();
			}
		}

		// Creates the rest of ontologies
		for (int i=1; i<=ONTOLOGY_NUMBER; i++){
			System.out.println("Creating ontology: ontology"+i);

			// Creates ontology
			try{
				Date now=Calendar.getInstance().getTime();
				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 "admin","admin",now,now);
/* MULTILINGUE				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 	 "admin","admin",0,now,now);*/
				odeService.createOntology(od);
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating ontology: "+e.getMessage());
			}

			// Creates concepts
			if (CONCEPT_NUMBER>0){
				System.out.print("  - Creating concepts:                  ");
				for (int j=1; j<=2;j++){
					System.out.print(".");
					odeService.insertTerm("ontology"+i,"concept"+j,
																"concept "+j+" of ontology "+i,Term.CONCEPT);
				}
				System.out.println();
			}

			// Creates term references and DOESN'T relates it to concepts
			if (TERM_REFERENCE_NUMBER>0){
				System.out.print("  - Creating term references:           ");
				for (int j=1; j<=1;j++){
					System.out.print(".");
					ReferenceDescriptor rd = new ReferenceDescriptor("termReference"+j,
																													 "reference "+j+" of concept"+j);
					odeService.addReference("ontology"+i,rd);
				}
				System.out.println();
			}

			// Creates formulas
			if (FORMULA_NUMBER>0){
				System.out.print("  - Creating formulas:                  ");
				for (int j=1; j<=1;j++){
					System.out.print(".");
					FormulaDescriptor fd = new FormulaDescriptor(
														  "ontology"+i,"formula"+j, "formula "+j+" of ontology"+i,
															"Have friends and they will give you...",
															"person(?x), person(?y), friend(?x,?y) --> give(?y,?x)",
															Term.AXIOM);
					odeService.insertReasoningElement(fd);
				}
				System.out.println();
			}

			// Creates instance attributes
			if ((INSTANCE_ATTRIBUTE_NUMBER>0) && (CONCEPT_NUMBER>0)){
				System.out.print("  - Creating instance attributes:       ");
				for (int j=1; j<=2;j++){
					for (int k=1; k<=1; k++){
						System.out.print(".");
						InstanceAttributeDescriptor iad = new InstanceAttributeDescriptor(
							"instanceAttribute"+k,"concept"+j,ValueTypes.STRING,"ontology"+i,
							"measurement unit "+k,"precision"+k,1,100000,
							"instance attribute "+k+" of concept "+j,"100","1");
						odeService.insertInstanceAttribute("ontology"+i,iad);
					}
				}
				System.out.println();
			}

			// Creates term relations
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>TERM_RELATION_NUMBER)){
				System.out.print("  - Creating term relations:            ");
				for (int j=1; j<=1;j++){
					System.out.print(".");
					TermRelation tr = new TermRelation("ontology"+i,null,"termRelation"+j,
																						 "concept"+j,"concept"+(j+1));
					odeService.insertTermRelation(tr);
				}
				System.out.println();
			}

			// Creates instance sets
			if (INSTANCE_SET_NUMBER>0){
				System.out.print("  - Creating instance sets:             ");
				for (int j=1; j<=1;j++){
					System.out.print(".");
					odeService.insertTerm("ontology"+i,"instanceSet"+j,
																"instance set "+j+" of ontology"+i,TermTypes.INSTANCE_SET);
				}
				System.out.println();
			}

			// Creates instances
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>=INSTANCE_NUMBER) &&
					(INSTANCE_SET_NUMBER>0)){
				System.out.print("  - Creating instances:                 ");
				for (int j=1; j<=2;j++){
					System.out.print(".");
					Instance instance = new Instance("instance"+j,"concept"+j,"instanceSet1",
																					 "instance "+j+" of ontology"+i);
					odeService.insertInstance("ontology"+i,instance);
				}
				System.out.println();
			}
		}
	}

	/**
	 *	Creates the initial state for the basic update group of benchmarks
	 */
	public static void initialStateBasicUpdate (ODEService odeService) throws Exception
	{
		final int factor = benchmarkConstants.initial_state_factor;

		final int ONTOLOGY_NUMBER = 								factor;

		final int CONCEPT_NUMBER = 									factor;
		final int TERM_REFERENCE_NUMBER = 					factor;
		final int FORMULA_NUMBER = 									factor;
		final int SYNONYM_NUMBER = 									factor;
		final int CONSTANT_NUMBER = 								factor;
		final int CLASS_ATTRIBUTE_NUMBER = 					factor;
		final int INSTANCE_ATTRIBUTE_NUMBER = 			factor;
		final int GROUP_NUMBER = 										factor;

		// Creates the ontologies one by one
		for (int i=0; i<=ONTOLOGY_NUMBER; i++){
			System.out.println("Creating ontology: ontology"+i);

			// Creates ontology
			try{
				Date now=Calendar.getInstance().getTime();
				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 "admin","admin",now,now);
/* MULTILINGUE				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 	 "admin","admin",0,now,now);*/
				odeService.createOntology(od);
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating ontology: "+e.getMessage());
			}

			// Creates concepts
			if (CONCEPT_NUMBER>0){
				System.out.print("  - Creating concepts:                  ");
				for (int j=1; j<=CONCEPT_NUMBER;j++){
					System.out.print(".");
					odeService.insertTerm("ontology"+i,"concept"+j,
																"concept "+j+" of ontology "+i,Term.CONCEPT);
				}
				System.out.println();
			}

			// Creates term references and relates it to concepts
			if ((TERM_REFERENCE_NUMBER>0) &&
					(CONCEPT_NUMBER>0) &&
					(TERM_REFERENCE_NUMBER>=CONCEPT_NUMBER)){
				System.out.print("  - Creating term references:           ");
				for (int j=1; j<=TERM_REFERENCE_NUMBER;j++){
					System.out.print(".");
					ReferenceDescriptor rd = new ReferenceDescriptor("termReference"+j,
																													 "reference "+j+" of concept"+j);
					odeService.addReference("ontology"+i,rd);
					odeService.relateReferenceToTerm("ontology"+i,"concept"+j,"termReference"+j);
				}
				System.out.println();
			}

			// Creates formulas
			if (FORMULA_NUMBER>0){
				System.out.print("  - Creating formulas:                  ");
				for (int j=1; j<=FORMULA_NUMBER;j++){
					System.out.print(".");
					FormulaDescriptor fd = new FormulaDescriptor(
														  "ontology"+i,"formula"+j, "formula "+j+" of ontology"+i,
															"Have friends and they will give you...",
															"person(?x), person(?y), friend(?x,?y) --> give(?y,?x)",
															Term.AXIOM);
					odeService.insertReasoningElement(fd);
				}
				System.out.println();
			}

			// Creates synonyms
			if (SYNONYM_NUMBER>0){
				System.out.print("  - Creating synonyms:                  ");
				for (int j=1; j<=SYNONYM_NUMBER;j++){
					System.out.print(".");
					SynonymAbbreviation sa = new SynonymAbbreviation(
																				"synonym"+j, "concept"+j, null,
																				TermTypes.SYNONYM, "synonym of concept"+j);
					odeService.addSynonymToTerm("ontology"+i,sa);
				}
				System.out.println();
			}

			// Creates constants
			if (CONSTANT_NUMBER>0){
				System.out.print("  - Creating constants:                 ");
				for (int j=1; j<=CONSTANT_NUMBER;j++){
					System.out.print(".");
					ConstantDescriptor cd = new ConstantDescriptor(
																				"constant"+j, "constant "+j+" of ontology "+i,
																				ValueTypes.STRING,"measurement unit "+j,
																				"value of constant"+j);
					odeService.insertConstant("ontology"+i,cd);
				}
				System.out.println();
			}

			// Creates class attributes
			if ((CLASS_ATTRIBUTE_NUMBER>0) && (CONCEPT_NUMBER>0)){
				System.out.print("  - Creating class attributes:          ");
				for (int j=1; j<=CONCEPT_NUMBER;j++){
					for (int k=1; k<=CLASS_ATTRIBUTE_NUMBER; k++){
						System.out.print(".");
						ClassAttributeDescriptor cad = new ClassAttributeDescriptor(
							"classAttribute"+k,"concept"+j,ValueTypes.STRING,"ontology"+i,
							"measurement unit "+k,"precision"+k,1,100000,
							"class attribute "+k+" of concept "+j);
						odeService.insertClassAttribute("ontology"+i,cad);
					}
				}
				System.out.println();
			}

			// Creates instance attributes
			if ((INSTANCE_ATTRIBUTE_NUMBER>0) && (CONCEPT_NUMBER>0)){
				System.out.print("  - Creating instance attributes:       ");
				for (int j=1; j<=CONCEPT_NUMBER;j++){
					for (int k=1; k<=INSTANCE_ATTRIBUTE_NUMBER; k++){
						System.out.print(".");
						InstanceAttributeDescriptor iad = new InstanceAttributeDescriptor(
							"instanceAttribute"+k,"concept"+j,ValueTypes.STRING,"ontology"+i,
							"measurement unit "+k,"precision"+k,1,100000,
							"instance attribute "+k+" of concept "+j,"100","1");
						odeService.insertInstanceAttribute("ontology"+i,iad);
					}
				}
				System.out.println();
			}

			// Creates groups
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>=GROUP_NUMBER)){
				System.out.print("  - Creating groups:                    ");
				String[] concept_list = new String[1];
				for (int j=1; j<=GROUP_NUMBER;j++){
					System.out.print(".");
					concept_list[0] = "concept"+j;
					Group group = new Group("group"+j,"group "+j+" of ontology"+i,concept_list);
					odeService.addGroup("ontology"+i,group);
				}
				System.out.println();
			}
		}
	}

	/**
	 *	Creates the minimal initial state for the basic update group of benchmarks
	 */
	public static void initialStateBasicUpdateMinimal (ODEService odeService) throws Exception
	{
		final int factor = benchmarkConstants.initial_state_factor;

		final int ONTOLOGY_NUMBER = 								factor;

		final int CONCEPT_NUMBER = 									factor;
		final int TERM_REFERENCE_NUMBER = 					factor;
		final int FORMULA_NUMBER = 									factor;
		final int SYNONYM_NUMBER = 									factor;
		final int CONSTANT_NUMBER = 								factor;
		final int CLASS_ATTRIBUTE_NUMBER = 					factor;
		final int INSTANCE_ATTRIBUTE_NUMBER = 			factor;
		final int GROUP_NUMBER = 										factor;

		// Creates ontology0
		for (int i=0; i<=0; i++){
			System.out.println("Creating ontology: ontology"+i);

			// Creates ontology
			try{
				Date now=Calendar.getInstance().getTime();
				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 "admin","admin",now,now);
/* MULTILINGUE				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 	 "admin","admin",0,now,now);*/
				odeService.createOntology(od);
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating ontology: "+e.getMessage());
			}

			// Creates concepts
			if (CONCEPT_NUMBER>0){
				System.out.print("  - Creating concepts:                  ");
				for (int j=1; j<=CONCEPT_NUMBER;j++){
					System.out.print(".");
					odeService.insertTerm("ontology"+i,"concept"+j,
																"concept "+j+" of ontology "+i,Term.CONCEPT);
				}
				System.out.println();
			}

			// Creates term references and relates it to concepts
			if ((TERM_REFERENCE_NUMBER>0) &&
					(CONCEPT_NUMBER>0) &&
					(TERM_REFERENCE_NUMBER>=CONCEPT_NUMBER)){
				System.out.print("  - Creating term references:           ");
				for (int j=1; j<=TERM_REFERENCE_NUMBER;j++){
					System.out.print(".");
					ReferenceDescriptor rd = new ReferenceDescriptor("termReference"+j,
																													 "reference "+j+" of concept"+j);
					odeService.addReference("ontology"+i,rd);
					odeService.relateReferenceToTerm("ontology"+i,"concept"+j,"termReference"+j);
				}
				System.out.println();
			}

			// Creates formulas
			if (FORMULA_NUMBER>0){
				System.out.print("  - Creating formulas:                  ");
				for (int j=1; j<=FORMULA_NUMBER;j++){
					System.out.print(".");
					FormulaDescriptor fd = new FormulaDescriptor(
														  "ontology"+i,"formula"+j, "formula "+j+" of ontology"+i,
															"Have friends and they will give you...",
															"person(?x), person(?y), friend(?x,?y) --> give(?y,?x)",
															Term.AXIOM);
					odeService.insertReasoningElement(fd);
				}
				System.out.println();
			}

			// Creates synonyms
			if (SYNONYM_NUMBER>0){
				System.out.print("  - Creating synonyms:                  ");
				// First creates synonyms for concept 1
				for (int j=1; j<=SYNONYM_NUMBER;j++){
					System.out.print(".");
					SynonymAbbreviation sa = new SynonymAbbreviation(
																				"synonym"+j, "concept1", null,
																				TermTypes.SYNONYM, "synonym of concept1");
					odeService.addSynonymToTerm("ontology"+i,sa);
				}
				// Creates synonyms for the rest of concepts
				for (int j=2; j<=SYNONYM_NUMBER;j++){
					System.out.print(".");
					SynonymAbbreviation sa = new SynonymAbbreviation(
																				"synonym1", "concept"+j, null,
																				TermTypes.SYNONYM, "synonym of concept"+j);
					odeService.addSynonymToTerm("ontology"+i,sa);
				}
				System.out.println();
			}

			// Creates constants
			if (CONSTANT_NUMBER>0){
				System.out.print("  - Creating constants:                 ");
				for (int j=1; j<=CONSTANT_NUMBER;j++){
					System.out.print(".");
					ConstantDescriptor cd = new ConstantDescriptor(
																				"constant"+j, "constant "+j+" of ontology "+i,
																				ValueTypes.STRING,"measurement unit "+j,
																				"value of constant"+j);
					odeService.insertConstant("ontology"+i,cd);
				}
				System.out.println();
			}

			// Creates class attributes
			if ((CLASS_ATTRIBUTE_NUMBER>0) && (CONCEPT_NUMBER>0)){
				System.out.print("  - Creating class attributes:          ");
				// First creates class attributes for concept1
				for (int j=1; j<=1;j++){
					for (int k=1; k<=CLASS_ATTRIBUTE_NUMBER; k++){
						System.out.print(".");
						ClassAttributeDescriptor cad = new ClassAttributeDescriptor(
							"classAttribute"+k,"concept"+j,ValueTypes.STRING,"ontology"+i,
							"measurement unit "+k,"precision"+k,1,100000,
							"class attribute "+k+" of concept "+j);
						odeService.insertClassAttribute("ontology"+i,cad);
					}
				}
				// Creates class attributes for the rest of concepts
				for (int j=2; j<=CONCEPT_NUMBER;j++){
					for (int k=1; k<=1; k++){
						System.out.print(".");
						ClassAttributeDescriptor cad = new ClassAttributeDescriptor(
							"classAttribute"+k,"concept"+j,ValueTypes.STRING,"ontology"+i,
							"measurement unit "+k,"precision"+k,1,100000,
							"class attribute "+k+" of concept "+j);
						odeService.insertClassAttribute("ontology"+i,cad);
					}
				}
				System.out.println();
			}

			// Creates instance attributes
			if ((INSTANCE_ATTRIBUTE_NUMBER>0) && (CONCEPT_NUMBER>0)){
				System.out.print("  - Creating instance attributes:       ");
				// First creates instance attributes for concept1
				for (int j=1; j<=1;j++){
					for (int k=1; k<=INSTANCE_ATTRIBUTE_NUMBER; k++){
						System.out.print(".");
						InstanceAttributeDescriptor iad = new InstanceAttributeDescriptor(
							"instanceAttribute"+k,"concept"+j,ValueTypes.STRING,"ontology"+i,
							"measurement unit "+k,"precision"+k,1,100000,
							"instance attribute "+k+" of concept "+j,"100","1");
						odeService.insertInstanceAttribute("ontology"+i,iad);
					}
				}
				// Creates instance attributes for the rest of concepts
				for (int j=2; j<=CONCEPT_NUMBER;j++){
					for (int k=1; k<=1; k++){
						System.out.print(".");
						InstanceAttributeDescriptor iad = new InstanceAttributeDescriptor(
							"instanceAttribute"+k,"concept"+j,ValueTypes.STRING,"ontology"+i,
							"measurement unit "+k,"precision"+k,1,100000,
							"instance attribute "+k+" of concept "+j,"100","1");
						odeService.insertInstanceAttribute("ontology"+i,iad);
					}
				}
				System.out.println();
			}

			// Creates groups
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>=GROUP_NUMBER)){
				System.out.print("  - Creating groups:                    ");
				String[] concept_list = new String[1];
				for (int j=1; j<=GROUP_NUMBER;j++){
					System.out.print(".");
					concept_list[0] = "concept"+j;
					Group group = new Group("group"+j,"group "+j+" of ontology"+i,concept_list);
					odeService.addGroup("ontology"+i,group);
				}
				System.out.println();
			}
		}

		// Creates the rest of ontologies
		for (int i=1; i<=ONTOLOGY_NUMBER; i++){
			System.out.println("Creating ontology: ontology"+i);

			// Creates ontology
			try{
				Date now=Calendar.getInstance().getTime();
				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 "admin","admin",now,now);
/* MULTILINGUE				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 	 "admin","admin",0,now,now);*/
				odeService.createOntology(od);
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating ontology: "+e.getMessage());
			}

			// Creates concepts
			if (CONCEPT_NUMBER>0){
				System.out.print("  - Creating concepts:                  ");
				for (int j=1; j<=1;j++){
					System.out.print(".");
					odeService.insertTerm("ontology"+i,"concept"+j,
																"concept "+j+" of ontology "+i,Term.CONCEPT);
				}
				System.out.println();
			}

			// Creates term references and relates it to concepts
			if ((TERM_REFERENCE_NUMBER>0) &&
					(CONCEPT_NUMBER>0) &&
					(TERM_REFERENCE_NUMBER>=CONCEPT_NUMBER)){
				System.out.print("  - Creating term references:           ");
				for (int j=1; j<=1;j++){
					System.out.print(".");
					ReferenceDescriptor rd = new ReferenceDescriptor("termReference"+j,
																													 "reference "+j+" of concept"+j);
					odeService.addReference("ontology"+i,rd);
					odeService.relateReferenceToTerm("ontology"+i,"concept"+j,"termReference"+j);
				}
				System.out.println();
			}

			// Creates formulas
			if (FORMULA_NUMBER>0){
				System.out.print("  - Creating formulas:                  ");
				for (int j=1; j<=1;j++){
					System.out.print(".");
					FormulaDescriptor fd = new FormulaDescriptor(
														  "ontology"+i,"formula"+j, "formula "+j+" of ontology"+i,
															"Have friends and they will give you...",
															"person(?x), person(?y), friend(?x,?y) --> give(?y,?x)",
															Term.AXIOM);
					odeService.insertReasoningElement(fd);
				}
				System.out.println();
			}

			// Creates constants
			if (CONSTANT_NUMBER>0){
				System.out.print("  - Creating constants:                 ");
				for (int j=1; j<=1;j++){
					System.out.print(".");
					ConstantDescriptor cd = new ConstantDescriptor(
																				"constant"+j, "constant "+j+" of ontology "+i,
																				ValueTypes.STRING,"measurement unit "+j,
																				"value of constant"+j);
					odeService.insertConstant("ontology"+i,cd);
				}
				System.out.println();
			}

			// Creates groups
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>=GROUP_NUMBER)){
				System.out.print("  - Creating groups:                    ");
				String[] concept_list = new String[1];
				for (int j=1; j<=1;j++){
					System.out.print(".");
					concept_list[0] = "concept"+j;
					Group group = new Group("group"+j,"group "+j+" of ontology"+i,concept_list);
					odeService.addGroup("ontology"+i,group);
				}
				System.out.println();
			}
		}
	}

	/**
	 *	Creates the initial state for the basic removal group of benchmarks
	 */
	public static void initialStateBasicRemove (ODEService odeService) throws Exception
	{
		final int factor = benchmarkConstants.initial_state_factor;

		final int ONTOLOGY_NUMBER = 								factor;

		final int CONCEPT_NUMBER = 									factor+1;
		final int TERM_REFERENCE_NUMBER = 					factor;
		final int ONTOLOGY_REFERENCE_NUMBER = 			factor;
		final int FORMULA_NUMBER = 									factor;
		final int SYNONYM_NUMBER = 									factor;
		final int CLASS_ATTRIBUTE_NUMBER = 					factor;
		final int CLASS_ATTRIBUTE_VALUE_NUMBER = 		CLASS_ATTRIBUTE_NUMBER;
		final int INSTANCE_ATTRIBUTE_NUMBER = 			factor;
		final int INSTANCE_ATTRIBUTE_VALUE_NUMBER = INSTANCE_ATTRIBUTE_NUMBER;
		final int TERM_RELATION_NUMBER = 						factor; // CONCEPT_NUMBER-1
		final int INSTANCE_SET_NUMBER = 						1;
		final int INSTANCE_VALUE_NUMBER = 					factor;

		// Creates the ontologies one by one
		for (int i=0; i<=ONTOLOGY_NUMBER; i++){
			System.out.println("Creating ontology: ontology"+i);

			// Creates ontology
			try{
				Date now=Calendar.getInstance().getTime();
				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 "admin","admin",now,now);
/* MULTILINGUE				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 	 "admin","admin",0,now,now);*/
				odeService.createOntology(od);
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating ontology: "+e.getMessage());
			}

			// Creates concepts
			if (CONCEPT_NUMBER>0){
				System.out.print("  - Creating concepts:                  ");
				for (int j=0; j<=CONCEPT_NUMBER;j++){
					System.out.print(".");
					odeService.insertTerm("ontology"+i,"concept"+j,
																"concept "+j+" of ontology "+i,Term.CONCEPT);
				}
				System.out.println();
			}

			// Creates term references and relates it to concepts
			if ((TERM_REFERENCE_NUMBER>0) &&
					(CONCEPT_NUMBER>0) &&
					(TERM_REFERENCE_NUMBER>=CONCEPT_NUMBER)){
				System.out.print("  - Creating term references:           ");
				for (int j=1; j<=TERM_REFERENCE_NUMBER;j++){
					System.out.print(".");
					ReferenceDescriptor rd = new ReferenceDescriptor("termReference"+j,
																													 "reference "+j+" of concept"+j);
					odeService.addReference("ontology"+i,rd);
					odeService.relateReferenceToTerm("ontology"+i,"concept"+j,"termReference"+j);
				}
				System.out.println();
			}

			// Creates ontology references
			if (ONTOLOGY_REFERENCE_NUMBER>0){
				System.out.print("  - Creating ontology references:       ");
				for (int j=1; j<=ONTOLOGY_REFERENCE_NUMBER;j++){
					System.out.print(".");
					ReferenceDescriptor rd = new ReferenceDescriptor("ontologyReference"+j,
																													 "reference "+j+" of ontology"+i);
					odeService.addReferenceToOntology("ontology"+i,rd);
				}
				System.out.println();
			}

			// Creates formulas
			if (FORMULA_NUMBER>0){
				System.out.print("  - Creating formulas:                  ");
				for (int j=1; j<=FORMULA_NUMBER;j++){
					System.out.print(".");
					FormulaDescriptor fd = new FormulaDescriptor(
														  "ontology"+i,"formula"+j, "formula "+j+" of ontology"+i,
															"Have friends and they will give you...",
															"person(?x), person(?y), friend(?x,?y) --> give(?y,?x)",
															Term.AXIOM);
					odeService.insertReasoningElement(fd);
				}
				System.out.println();
			}

			// Creates synonyms
			if (SYNONYM_NUMBER>0){
				System.out.print("  - Creating synonyms:                  ");
				// Creates first synonyms for concept0
				for (int j=1; j<=SYNONYM_NUMBER;j++){
					System.out.print(".");
					SynonymAbbreviation sa = new SynonymAbbreviation(
																				"synonym"+j, "concept0", null,
																				TermTypes.SYNONYM, "synonym of concept0");
					odeService.addSynonymToTerm("ontology"+i,sa);
				}
				// Creates synonyms for the rest of concepts
				for (int j=1; j<=SYNONYM_NUMBER;j++){
					System.out.print(".");
					SynonymAbbreviation sa = new SynonymAbbreviation(
																				"synonym1", "concept"+j, null,
																				TermTypes.SYNONYM, "synonym of concept"+j);
					odeService.addSynonymToTerm("ontology"+i,sa);
				}
				System.out.println();
			}

			// Creates class attributes for concept0
			if ((CLASS_ATTRIBUTE_NUMBER>0) && (CONCEPT_NUMBER>0)){
				System.out.print("  - Creating class attributes:          ");
				for (int k=0; k<=CLASS_ATTRIBUTE_NUMBER; k++){
					System.out.print(".");
					ClassAttributeDescriptor cad = new ClassAttributeDescriptor(
						"classAttribute"+k,"concept0",ValueTypes.STRING,"ontology"+i,
						"measurement unit "+k,"precision"+k,1,100000,
						"class attribute "+k+" of concept0");
					odeService.insertClassAttribute("ontology"+i,cad);
				}
				System.out.println();
			}

			// Creates class attribute values for concept0
			if ((CONCEPT_NUMBER>0) &&
					(CLASS_ATTRIBUTE_NUMBER>0) &&
					(CLASS_ATTRIBUTE_NUMBER>=CLASS_ATTRIBUTE_VALUE_NUMBER)){
				System.out.print("  - Creating class attribute values:    ");
				// Creates class attribute values for classAttribute0
				for (int k=1; k<=CLASS_ATTRIBUTE_VALUE_NUMBER; k++){
					System.out.print(".");
					odeService.addValueToClassAttribute("ontology"+i,"classAttribute0",
																							"concept0","classAttributeValue"+k);
				}
				// Creates class attribute values for the rest class attributes
				for (int k=1; k<=CLASS_ATTRIBUTE_VALUE_NUMBER; k++){
					System.out.print(".");
					odeService.addValueToClassAttribute("ontology"+i,"classAttribute"+k,
																							"concept0","classAttributeValue1");
				}
				System.out.println();
			}

			// Creates instance attributes for concept0
			if ((INSTANCE_ATTRIBUTE_NUMBER>0) && (CONCEPT_NUMBER>0)){
				System.out.print("  - Creating instance attributes:       ");
				for (int k=0; k<=INSTANCE_ATTRIBUTE_NUMBER; k++){
					System.out.print(".");
					InstanceAttributeDescriptor iad = new InstanceAttributeDescriptor(
						"instanceAttribute"+k,"concept0",ValueTypes.STRING,"ontology"+i,
						"measurement unit "+k,"precision"+k,1,100000,
						"instance attribute "+k+" of concept0","100","1");
					odeService.insertInstanceAttribute("ontology"+i,iad);
				}
				System.out.println();
			}

			// Creates instance attribute values for concept0
			if ((CONCEPT_NUMBER>0) &&
					(INSTANCE_ATTRIBUTE_NUMBER>0) &&
					(INSTANCE_ATTRIBUTE_NUMBER>=INSTANCE_ATTRIBUTE_VALUE_NUMBER)){
				System.out.print("  - Creating instance attribute values: ");
				// Creates instance attribute values for instanceAttribute0
				for (int k=1; k<=INSTANCE_ATTRIBUTE_VALUE_NUMBER; k++){
					System.out.print(".");
					odeService.addValueToInstanceAttribute("ontology"+i,"instanceAttribute0",
																							   "concept0","instanceAttributeValue"+k);
				}
				// Creates instance attribute values for the rest of instance attributes
				for (int k=1; k<=INSTANCE_ATTRIBUTE_VALUE_NUMBER; k++){
					System.out.print(".");
					odeService.addValueToInstanceAttribute("ontology"+i,"instanceAttribute"+k,
																							   "concept0","instanceAttributeValue1");
				}
				System.out.println();
			}

			// Creates term relations
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>TERM_RELATION_NUMBER)){
				System.out.print("  - Creating term relations:            ");
				for (int j=1; j<=TERM_RELATION_NUMBER;j++){
					System.out.print(".");
					TermRelation tr = new TermRelation("ontology"+i,null,"termRelation"+j,
																						 "concept"+j,"concept"+(j+1));
					odeService.insertTermRelation(tr);
				}
				System.out.println();
			}

			// Creates instance sets
			if (INSTANCE_SET_NUMBER>0){
				System.out.print("  - Creating instance sets:             ");
				for (int j=1; j<=INSTANCE_SET_NUMBER;j++){
					System.out.print(".");
					odeService.insertTerm("ontology"+i,"instanceSet"+j,
																"instance set "+j+" of ontology"+i,TermTypes.INSTANCE_SET);
				}
				System.out.println();
			}

			// Creates instance0
			if ((CONCEPT_NUMBER>0) &&
					(INSTANCE_SET_NUMBER>0)){
				System.out.print("  - Creating instances:                 ");
				System.out.print(".");
				Instance instance = new Instance("instance0","concept0","instanceSet1",
																				 "instance 0 of ontology"+i);
				odeService.insertInstance("ontology"+i,instance);
				System.out.println();
			}

			// Creates instance values
			if (INSTANCE_VALUE_NUMBER>0){
				System.out.print("  - Creating instance values:             ");
				for (int j=1; j<=INSTANCE_VALUE_NUMBER;j++){
					System.out.print(".");
					odeService.addValueToInstance("ontology"+i,"instanceSet1","instance0",
						"concept0","instanceAttribute"+j,"instanceValue"+j);
				}
				System.out.println();
			}
		}
	}

	/**
	 *	Creates the minimal initial state for the basic removal group of benchmarks
	 */
	public static void initialStateBasicRemoveMinimal (ODEService odeService) throws Exception
	{
		final int factor = benchmarkConstants.initial_state_factor;

		final int ONTOLOGY_NUMBER = 								factor;

		final int CONCEPT_NUMBER = 									factor+1;
		final int TERM_REFERENCE_NUMBER = 					factor;
		final int ONTOLOGY_REFERENCE_NUMBER = 			factor;
		final int FORMULA_NUMBER = 									factor;
		final int SYNONYM_NUMBER = 									factor;
		final int CLASS_ATTRIBUTE_NUMBER = 					factor;
		final int CLASS_ATTRIBUTE_VALUE_NUMBER = 		CLASS_ATTRIBUTE_NUMBER;
		final int INSTANCE_ATTRIBUTE_NUMBER = 			factor;
		final int INSTANCE_ATTRIBUTE_VALUE_NUMBER = INSTANCE_ATTRIBUTE_NUMBER;
		final int TERM_RELATION_NUMBER = 						factor; // CONCEPT_NUMBER-1
		final int INSTANCE_SET_NUMBER = 						1;
		final int INSTANCE_VALUE_NUMBER = 					factor;

		// Creates ontology0
		for (int i=0; i<=0; i++){
			System.out.println("Creating ontology: ontology"+i);

			// Creates ontology
			try{
				Date now=Calendar.getInstance().getTime();
				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 "admin","admin",now,now);
/* MULTILINGUE				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 	 "admin","admin",0,now,now);*/
				odeService.createOntology(od);
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating ontology: "+e.getMessage());
			}

			// Creates concepts
			if (CONCEPT_NUMBER>0){
				System.out.print("  - Creating concepts:                  ");
				for (int j=0; j<=CONCEPT_NUMBER;j++){
					System.out.print(".");
					odeService.insertTerm("ontology"+i,"concept"+j,
																"concept "+j+" of ontology "+i,Term.CONCEPT);
				}
				System.out.println();
			}

			// Creates term references and relates it to concepts
			if ((TERM_REFERENCE_NUMBER>0) &&
					(CONCEPT_NUMBER>0) &&
					(TERM_REFERENCE_NUMBER>=CONCEPT_NUMBER)){
				System.out.print("  - Creating term references:           ");
				for (int j=1; j<=TERM_REFERENCE_NUMBER;j++){
					System.out.print(".");
					ReferenceDescriptor rd = new ReferenceDescriptor("termReference"+j,
																													 "reference "+j+" of concept"+j);
					odeService.addReference("ontology"+i,rd);
					odeService.relateReferenceToTerm("ontology"+i,"concept"+j,"termReference"+j);
				}
				System.out.println();
			}

			// Creates ontology references
			if (ONTOLOGY_REFERENCE_NUMBER>0){
				System.out.print("  - Creating ontology references:       ");
				for (int j=1; j<=ONTOLOGY_REFERENCE_NUMBER;j++){
					System.out.print(".");
					ReferenceDescriptor rd = new ReferenceDescriptor("ontologyReference"+j,
																													 "reference "+j+" of ontology"+i);
					odeService.addReferenceToOntology("ontology"+i,rd);
				}
				System.out.println();
			}

			// Creates formulas
			if (FORMULA_NUMBER>0){
				System.out.print("  - Creating formulas:                  ");
				for (int j=1; j<=FORMULA_NUMBER;j++){
					System.out.print(".");
					FormulaDescriptor fd = new FormulaDescriptor(
														  "ontology"+i,"formula"+j, "formula "+j+" of ontology"+i,
															"Have friends and they will give you...",
															"person(?x), person(?y), friend(?x,?y) --> give(?y,?x)",
															Term.AXIOM);
					odeService.insertReasoningElement(fd);
				}
				System.out.println();
			}

			// Creates synonyms
			if (SYNONYM_NUMBER>0){
				System.out.print("  - Creating synonyms:                  ");
				// Creates first synonyms for concept0
				for (int j=1; j<=SYNONYM_NUMBER;j++){
					System.out.print(".");
					SynonymAbbreviation sa = new SynonymAbbreviation(
																				"synonym"+j, "concept0", null,
																				TermTypes.SYNONYM, "synonym of concept0");
					odeService.addSynonymToTerm("ontology"+i,sa);
				}
				// Creates synonyms for the rest of concepts
				for (int j=1; j<=SYNONYM_NUMBER;j++){
					System.out.print(".");
					SynonymAbbreviation sa = new SynonymAbbreviation(
																				"synonym1", "concept"+j, null,
																				TermTypes.SYNONYM, "synonym of concept"+j);
					odeService.addSynonymToTerm("ontology"+i,sa);
				}
				System.out.println();
			}

			// Creates class attributes for concept0
			if ((CLASS_ATTRIBUTE_NUMBER>0) && (CONCEPT_NUMBER>0)){
				System.out.print("  - Creating class attributes:          ");
				for (int k=0; k<=CLASS_ATTRIBUTE_NUMBER; k++){
					System.out.print(".");
					ClassAttributeDescriptor cad = new ClassAttributeDescriptor(
						"classAttribute"+k,"concept0",ValueTypes.STRING,"ontology"+i,
						"measurement unit "+k,"precision"+k,1,100000,
						"class attribute "+k+" of concept0");
					odeService.insertClassAttribute("ontology"+i,cad);
				}
				System.out.println();
			}

			// Creates class attribute values for concept0
			if ((CONCEPT_NUMBER>0) &&
					(CLASS_ATTRIBUTE_NUMBER>0) &&
					(CLASS_ATTRIBUTE_NUMBER>=CLASS_ATTRIBUTE_VALUE_NUMBER)){
				System.out.print("  - Creating class attribute values:    ");
				// Creates class attribute values for classAttribute0
				for (int k=1; k<=CLASS_ATTRIBUTE_VALUE_NUMBER; k++){
					System.out.print(".");
					odeService.addValueToClassAttribute("ontology"+i,"classAttribute0",
																							"concept0","classAttributeValue"+k);
				}
				// Creates class attribute values for the rest class attributes
				for (int k=1; k<=CLASS_ATTRIBUTE_VALUE_NUMBER; k++){
					System.out.print(".");
					odeService.addValueToClassAttribute("ontology"+i,"classAttribute"+k,
																							"concept0","classAttributeValue1");
				}
				System.out.println();
			}

			// Creates instance attributes for concept0
			if ((INSTANCE_ATTRIBUTE_NUMBER>0) && (CONCEPT_NUMBER>0)){
				System.out.print("  - Creating instance attributes:       ");
				for (int k=0; k<=INSTANCE_ATTRIBUTE_NUMBER; k++){
					System.out.print(".");
					InstanceAttributeDescriptor iad = new InstanceAttributeDescriptor(
						"instanceAttribute"+k,"concept0",ValueTypes.STRING,"ontology"+i,
						"measurement unit "+k,"precision"+k,1,100000,
						"instance attribute "+k+" of concept0","100","1");
					odeService.insertInstanceAttribute("ontology"+i,iad);
				}
				System.out.println();
			}

			// Creates instance attribute values for concept0
			if ((CONCEPT_NUMBER>0) &&
					(INSTANCE_ATTRIBUTE_NUMBER>0) &&
					(INSTANCE_ATTRIBUTE_NUMBER>=INSTANCE_ATTRIBUTE_VALUE_NUMBER)){
				System.out.print("  - Creating instance attribute values: ");
				// Creates instance attribute values for instanceAttribute0
				for (int k=1; k<=INSTANCE_ATTRIBUTE_VALUE_NUMBER; k++){
					System.out.print(".");
					odeService.addValueToInstanceAttribute("ontology"+i,"instanceAttribute0",
																							   "concept0","instanceAttributeValue"+k);
				}
				// Creates instance attribute values for the rest of instance attributes
				for (int k=1; k<=INSTANCE_ATTRIBUTE_VALUE_NUMBER; k++){
					System.out.print(".");
					odeService.addValueToInstanceAttribute("ontology"+i,"instanceAttribute"+k,
																							   "concept0","instanceAttributeValue1");
				}
				System.out.println();
			}

			// Creates term relations
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>TERM_RELATION_NUMBER)){
				System.out.print("  - Creating term relations:            ");
				for (int j=1; j<=TERM_RELATION_NUMBER;j++){
					System.out.print(".");
					TermRelation tr = new TermRelation("ontology"+i,null,"termRelation"+j,
																						 "concept"+j,"concept"+(j+1));
					odeService.insertTermRelation(tr);
				}
				System.out.println();
			}

			// Creates instance sets
			if (INSTANCE_SET_NUMBER>0){
				System.out.print("  - Creating instance sets:             ");
				for (int j=1; j<=INSTANCE_SET_NUMBER;j++){
					System.out.print(".");
					odeService.insertTerm("ontology"+i,"instanceSet"+j,
																"instance set "+j+" of ontology"+i,TermTypes.INSTANCE_SET);
				}
				System.out.println();
			}

			// Creates instance0
			if ((CONCEPT_NUMBER>0) &&
					(INSTANCE_SET_NUMBER>0)){
				System.out.print("  - Creating instances:                 ");
				System.out.print(".");
				Instance instance = new Instance("instance0","concept0","instanceSet1",
																				 "instance 0 of ontology"+i);
				odeService.insertInstance("ontology"+i,instance);
				System.out.println();
			}

			// Creates instance values
			if (INSTANCE_VALUE_NUMBER>0){
				System.out.print("  - Creating instance values:             ");
				for (int j=1; j<=INSTANCE_VALUE_NUMBER;j++){
					System.out.print(".");
					odeService.addValueToInstance("ontology"+i,"instanceSet1","instance0",
						"concept0","instanceAttribute"+j,"instanceValue"+j);
				}
				System.out.println();
			}
		}

		// Creates the rest of ontologies
		for (int i=1; i<=ONTOLOGY_NUMBER; i++){
			System.out.println("Creating ontology: ontology"+i);

			// Creates ontology
			try{
				Date now=Calendar.getInstance().getTime();
				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 "admin","admin",now,now);
/* MULTILINGUE				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 	 "admin","admin",0,now,now);*/
				odeService.createOntology(od);
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating ontology: "+e.getMessage());
			}

			// Creates concepts
			if (CONCEPT_NUMBER>0){
				System.out.print("  - Creating concepts:                  ");
				for (int j=0; j<=2;j++){
					System.out.print(".");
					odeService.insertTerm("ontology"+i,"concept"+j,
																"concept "+j+" of ontology "+i,Term.CONCEPT);
				}
				System.out.println();
			}

			// Creates term references and relates it to concepts
			if ((TERM_REFERENCE_NUMBER>0) &&
					(CONCEPT_NUMBER>0) &&
					(TERM_REFERENCE_NUMBER>=CONCEPT_NUMBER)){
				System.out.print("  - Creating term references:           ");
				for (int j=1; j<=1;j++){
					System.out.print(".");
					ReferenceDescriptor rd = new ReferenceDescriptor("termReference"+j,
																													 "reference "+j+" of concept"+j);
					odeService.addReference("ontology"+i,rd);
					odeService.relateReferenceToTerm("ontology"+i,"concept"+j,"termReference"+j);
				}
				System.out.println();
			}

			// Creates ontology references
			if (ONTOLOGY_REFERENCE_NUMBER>0){
				System.out.print("  - Creating ontology references:       ");
				for (int j=1; j<=1;j++){
					System.out.print(".");
					ReferenceDescriptor rd = new ReferenceDescriptor("ontologyReference"+j,
																													 "reference "+j+" of ontology"+i);
					odeService.addReferenceToOntology("ontology"+i,rd);
				}
				System.out.println();
			}

			// Creates formulas
			if (FORMULA_NUMBER>0){
				System.out.print("  - Creating formulas:                  ");
				for (int j=1; j<=1;j++){
					System.out.print(".");
					FormulaDescriptor fd = new FormulaDescriptor(
														  "ontology"+i,"formula"+j, "formula "+j+" of ontology"+i,
															"Have friends and they will give you...",
															"person(?x), person(?y), friend(?x,?y) --> give(?y,?x)",
															Term.AXIOM);
					odeService.insertReasoningElement(fd);
				}
				System.out.println();
			}

			// Creates instance attributes for concept0
			if ((INSTANCE_ATTRIBUTE_NUMBER>0) && (CONCEPT_NUMBER>0)){
				System.out.print("  - Creating instance attributes:       ");
				for (int k=0; k<=0; k++){
					System.out.print(".");
					InstanceAttributeDescriptor iad = new InstanceAttributeDescriptor(
						"instanceAttribute"+k,"concept0",ValueTypes.STRING,"ontology"+i,
						"measurement unit "+k,"precision"+k,1,100000,
						"instance attribute "+k+" of concept0","100","1");
					odeService.insertInstanceAttribute("ontology"+i,iad);
				}
				System.out.println();
			}

			// Creates term relations
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>TERM_RELATION_NUMBER)){
				System.out.print("  - Creating term relations:            ");
				for (int j=1; j<=1;j++){
					System.out.print(".");
					TermRelation tr = new TermRelation("ontology"+i,null,"termRelation"+j,
																						 "concept"+j,"concept"+(j+1));
					odeService.insertTermRelation(tr);
				}
				System.out.println();
			}

			// Creates instance sets
			if (INSTANCE_SET_NUMBER>0){
				System.out.print("  - Creating instance sets:             ");
				for (int j=1; j<=1;j++){
					System.out.print(".");
					odeService.insertTerm("ontology"+i,"instanceSet"+j,
																"instance set "+j+" of ontology"+i,TermTypes.INSTANCE_SET);
				}
				System.out.println();
			}

			// Creates instance0
			if ((CONCEPT_NUMBER>0) &&
					(INSTANCE_SET_NUMBER>0)){
				System.out.print("  - Creating instances:                 ");
				System.out.print(".");
				Instance instance = new Instance("instance0","concept0","instanceSet1",
																				 "instance 0 of ontology"+i);
				odeService.insertInstance("ontology"+i,instance);
				System.out.println();
			}

			// Creates instance values
			if (INSTANCE_VALUE_NUMBER>0){
				System.out.print("  - Creating instance values:           ");
				for (int j=1; j<=1;j++){
					System.out.print(".");
					odeService.addValueToInstance("ontology"+i,"instanceSet1","instance0",
						"concept0","instanceAttribute0","instanceValue"+j);
				}
				System.out.println();
			}
		}
	}

	/**
	 *	Creates the initial state for the basic select group of benchmarks
	 */
	public static void initialStateBasicSelect (ODEService odeService) throws Exception
	{
		final int factor = benchmarkConstants.initial_state_factor;

		final int ONTOLOGY_NUMBER = 								factor;

		final int CONCEPT_NUMBER = 									factor;
		final int TERM_REFERENCE_NUMBER = 					factor;
		final int FORMULA_NUMBER = 									factor;
		final int SYNONYM_NUMBER = 									factor;
		final int CLASS_ATTRIBUTE_NUMBER = 					factor;
		final int INSTANCE_ATTRIBUTE_NUMBER = 			factor;
		final int CONSTANT_NUMBER = 								factor;
		final int TERM_RELATION_NUMBER = 						1;
		final int INSTANCE_SET_NUMBER = 						factor;
		final int INSTANCE_NUMBER = 								factor;
		final int INSTANCE_VALUE_NUMBER = 					factor;
		final int RELATION_INSTANCE_NUMBER = 				factor;
		final int GROUP_NUMBER = 										factor;

		// Creates the ontologies one by one
		for (int i=0; i<=ONTOLOGY_NUMBER; i++){
			System.out.println("Creating ontology: ontology"+i);

			// Creates ontology
			try{
				Date now=Calendar.getInstance().getTime();
				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 "admin","admin",now,now);
/* MULTILINGUE				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 	 "admin","admin",0,now,now);*/
				odeService.createOntology(od);
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating ontology: "+e.getMessage());
			}

			// Creates concepts
			if (CONCEPT_NUMBER>0){
				System.out.print("  - Creating concepts:                  ");
				for (int j=0; j<=CONCEPT_NUMBER;j++){
					System.out.print(".");
					odeService.insertTerm("ontology"+i,"concept"+j,
																"concept "+j+" of ontology "+i,Term.CONCEPT);
				}
				System.out.println();
			}

			// Creates term references and relates it to concepts
			if ((TERM_REFERENCE_NUMBER>0) &&
					(CONCEPT_NUMBER>0) &&
					(TERM_REFERENCE_NUMBER>=CONCEPT_NUMBER)){
				System.out.print("  - Creating term references:           ");
				for (int j=1; j<=TERM_REFERENCE_NUMBER;j++){
					System.out.print(".");
					ReferenceDescriptor rd = new ReferenceDescriptor("termReference"+j,
																													 "reference "+j+" of concept"+j);
					odeService.addReference("ontology"+i,rd);
					odeService.relateReferenceToTerm("ontology"+i,"concept"+j,"termReference"+j);
				}
				System.out.println();
			}

			// Creates formulas
			if (FORMULA_NUMBER>0){
				System.out.print("  - Creating formulas:                  ");
				for (int j=1; j<=FORMULA_NUMBER;j++){
					System.out.print(".");
					FormulaDescriptor fd = new FormulaDescriptor(
														  "ontology"+i,"formula"+j, "formula "+j+" of ontology"+i,
															"Have friends and they will give you...",
															"person(?x), person(?y), friend(?x,?y) --> give(?y,?x)",
															Term.AXIOM);
					odeService.insertReasoningElement(fd);
				}
				System.out.println();
			}

			// Creates synonyms
			if (SYNONYM_NUMBER>0){
				System.out.print("  - Creating synonyms:                  ");
				// Creates first synonyms for concept0
				for (int j=1; j<=SYNONYM_NUMBER;j++){
					System.out.print(".");
					SynonymAbbreviation sa = new SynonymAbbreviation(
																				"synonym"+j, "concept0", null,
																				TermTypes.SYNONYM, "synonym of concept0");
					odeService.addSynonymToTerm("ontology"+i,sa);
				}
				// Creates synonyms for the rest of concepts
				for (int j=1; j<=SYNONYM_NUMBER;j++){
					System.out.print(".");
					SynonymAbbreviation sa = new SynonymAbbreviation(
																				"synonym1", "concept"+j, null,
																				TermTypes.SYNONYM, "synonym of concept"+j);
					odeService.addSynonymToTerm("ontology"+i,sa);
				}
				System.out.println();
			}

			// Creates constants
			if (CONSTANT_NUMBER>0){
				System.out.print("  - Creating constants:                 ");
				for (int j=1; j<=CONSTANT_NUMBER;j++){
					System.out.print(".");
					ConstantDescriptor cd = new ConstantDescriptor(
																				"constant"+j, "constant "+j+" of ontology "+i,
																				ValueTypes.STRING,"measurement unit "+j,
																				"value of constant"+j);
					odeService.insertConstant("ontology"+i,cd);
				}
				System.out.println();
			}

			// Creates class attributes
			if ((CLASS_ATTRIBUTE_NUMBER>0) && (CONCEPT_NUMBER>0)){
				System.out.print("  - Creating class attributes:          ");
				// Creates first class attributes for concept0
				for (int k=0; k<=CLASS_ATTRIBUTE_NUMBER; k++){
					System.out.print(".");
					ClassAttributeDescriptor cad = new ClassAttributeDescriptor(
						"classAttribute"+k,"concept0",ValueTypes.STRING,"ontology"+i,
						"measurement unit "+k,"precision"+k,1,100000,
						"class attribute "+k+" of concept0");
					odeService.insertClassAttribute("ontology"+i,cad);
				}
				// Creates class attributes for the rest of concepts
				for (int k=1; k<=CLASS_ATTRIBUTE_NUMBER; k++){
					System.out.print(".");
					ClassAttributeDescriptor cad = new ClassAttributeDescriptor(
						"classAttribute1","concept"+k,ValueTypes.STRING,"ontology"+i,
						"measurement unit "+k,"precision"+k,1,100000,
						"class attribute 1 of concept"+k);
					odeService.insertClassAttribute("ontology"+i,cad);
				}
				System.out.println();
			}

			// Creates instance attributes
			if ((INSTANCE_ATTRIBUTE_NUMBER>0) && (CONCEPT_NUMBER>0)){
				System.out.print("  - Creating instance attributes:       ");
				// Creates first instance attributes for concept0
				for (int k=0; k<=INSTANCE_ATTRIBUTE_NUMBER; k++){
					System.out.print(".");
					InstanceAttributeDescriptor iad = new InstanceAttributeDescriptor(
						"instanceAttribute"+k,"concept0",ValueTypes.STRING,"ontology"+i,
						"measurement unit "+k,"precision"+k,1,100000,
						"instance attribute "+k+" of concept0","100","1");
					odeService.insertInstanceAttribute("ontology"+i,iad);
				}
				// Creates instance attributes for the rest of concepts
				for (int k=1; k<=INSTANCE_ATTRIBUTE_NUMBER; k++){
					System.out.print(".");
					InstanceAttributeDescriptor iad = new InstanceAttributeDescriptor(
						"instanceAttribute1","concept"+k,ValueTypes.STRING,"ontology"+i,
						"measurement unit "+k,"precision"+k,1,100000,
						"instance attribute 1 of concept"+k,"100","1");
					odeService.insertInstanceAttribute("ontology"+i,iad);
				}
				System.out.println();
			}

			// Creates term relations
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>TERM_RELATION_NUMBER)){
				System.out.print("  - Creating term relations:            ");
				for (int j=1; j<=TERM_RELATION_NUMBER;j++){
					System.out.print(".");
					TermRelation tr = new TermRelation("ontology"+i,null,"termRelation"+j,
																						 "concept"+j,"concept"+(j+1));
					odeService.insertTermRelation(tr);
				}
				System.out.println();
			}

			// Creates groups
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>=GROUP_NUMBER)){
				System.out.print("  - Creating groups:                    ");
				String[] concept_list = new String[1];
				for (int j=1; j<=GROUP_NUMBER;j++){
					System.out.print(".");
					concept_list[0] = "concept"+j;
					Group group = new Group("group"+j,"group "+j+" of ontology"+i,concept_list);
					odeService.addGroup("ontology"+i,group);
				}
				System.out.println();
			}

			// Creates instance sets
			if (INSTANCE_SET_NUMBER>0){
				System.out.print("  - Creating instance sets:             ");
				for (int j=1; j<=INSTANCE_SET_NUMBER;j++){
					System.out.print(".");
					odeService.insertTerm("ontology"+i,"instanceSet"+j,
																"instance set "+j+" of ontology"+i,TermTypes.INSTANCE_SET);
				}
				System.out.println();
			}

			// Creates instances
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>=INSTANCE_NUMBER) &&
					(INSTANCE_SET_NUMBER>0)){
				System.out.print("  - Creating instances:                 ");
				// Creates first instances for instanceSet1
				for (int j=0; j<=INSTANCE_NUMBER;j++){
					System.out.print(".");
					Instance instance = new Instance("instance"+j,"concept"+j,"instanceSet1",
																					 "instance "+j+" of ontology"+i);
					odeService.insertInstance("ontology"+i,instance);
				}
				// Creates instances for the rest of instance sets
				for (int j=2; j<=INSTANCE_NUMBER;j++){
					System.out.print(".");
					Instance instance = new Instance("instance1","concept1","instanceSet"+j,
																					 "instance 1 of ontology"+i);
					odeService.insertInstance("ontology"+i,instance);
					instance = new Instance("instance2","concept2","instanceSet"+j,
																					 "instance 2 of ontology"+i);
					odeService.insertInstance("ontology"+i,instance);
				}
				System.out.println();
			}

			// Creates instance values for instanceSet1
			if (INSTANCE_VALUE_NUMBER>0){
				System.out.print("  - Creating instance values:           ");
				for (int j=1; j<=INSTANCE_VALUE_NUMBER;j++){
					System.out.print(".");
					odeService.addValueToInstance("ontology"+i,"instanceSet1","instance0",
						"concept0","instanceAttribute"+j,"instanceValue"+j);
				}
				System.out.println();
			}

			// Creates relation instances
			if (RELATION_INSTANCE_NUMBER>0){
				System.out.print("  - Creating relation instances:        ");
				for (int j=1; j<=RELATION_INSTANCE_NUMBER;j++){
					System.out.print(".");
					// Esto ha cambiado de la iteracin 3 a la 4
					//TermRelationInstance tri = new TermRelationInstance(
					//	"relationInstance1","termRelation1","instanceSet"+j,
					//	"instance of termRelation1",
					//	"instance1","instance2");
					TermRelation tr = new TermRelation("ontology"+i,null,"termRelation1",
																						 "concept1","concept2");
					TermRelationInstance tri = new TermRelationInstance(
						"relationInstance1",tr,"instanceSet"+j,
						"instance of termRelation1",
						"instance1","instance2");
					odeService.insertRelationInstance("ontology"+i,tri);
				}
				System.out.println();
			}
		}
	}

	/**
	 *	Creates the minimal initial state for the basic select group of benchmarks
	 */
	public static void initialStateBasicSelectMinimal (ODEService odeService) throws Exception
	{
		final int factor = benchmarkConstants.initial_state_factor;

		final int ONTOLOGY_NUMBER = 								factor;

		final int CONCEPT_NUMBER = 									factor;
		final int TERM_REFERENCE_NUMBER = 					factor;
		final int FORMULA_NUMBER = 									factor;
		final int SYNONYM_NUMBER = 									factor;
		final int CLASS_ATTRIBUTE_NUMBER = 					factor;
		final int INSTANCE_ATTRIBUTE_NUMBER = 			factor;
		final int CONSTANT_NUMBER = 								factor;
		final int TERM_RELATION_NUMBER = 						1;
		final int INSTANCE_SET_NUMBER = 						factor;
		final int INSTANCE_NUMBER = 								factor;
		final int INSTANCE_VALUE_NUMBER = 					factor;
		final int RELATION_INSTANCE_NUMBER = 				factor;
		final int GROUP_NUMBER = 										factor;

		// Creates ontology0
		for (int i=0; i<=0; i++){
			System.out.println("Creating ontology: ontology"+i);

			// Creates ontology
			try{
				Date now=Calendar.getInstance().getTime();
				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 "admin","admin",now,now);
/* MULTILINGUE				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 	 "admin","admin",0,now,now);*/
				odeService.createOntology(od);
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating ontology: "+e.getMessage());
			}

			// Creates concepts
			if (CONCEPT_NUMBER>0){
				System.out.print("  - Creating concepts:                  ");
				for (int j=0; j<=CONCEPT_NUMBER;j++){
					System.out.print(".");
					odeService.insertTerm("ontology"+i,"concept"+j,
																"concept "+j+" of ontology "+i,Term.CONCEPT);
				}
				System.out.println();
			}

			// Creates term references and relates it to concepts
			if ((TERM_REFERENCE_NUMBER>0) &&
					(CONCEPT_NUMBER>0) &&
					(TERM_REFERENCE_NUMBER>=CONCEPT_NUMBER)){
				System.out.print("  - Creating term references:           ");
				for (int j=1; j<=TERM_REFERENCE_NUMBER;j++){
					System.out.print(".");
					ReferenceDescriptor rd = new ReferenceDescriptor("termReference"+j,
																													 "reference "+j+" of concept"+j);
					odeService.addReference("ontology"+i,rd);
					odeService.relateReferenceToTerm("ontology"+i,"concept"+j,"termReference"+j);
				}
				System.out.println();
			}

			// Creates formulas
			if (FORMULA_NUMBER>0){
				System.out.print("  - Creating formulas:                  ");
				for (int j=1; j<=FORMULA_NUMBER;j++){
					System.out.print(".");
					FormulaDescriptor fd = new FormulaDescriptor(
														  "ontology"+i,"formula"+j, "formula "+j+" of ontology"+i,
															"Have friends and they will give you...",
															"person(?x), person(?y), friend(?x,?y) --> give(?y,?x)",
															Term.AXIOM);
					odeService.insertReasoningElement(fd);
				}
				System.out.println();
			}

			// Creates synonyms
			if (SYNONYM_NUMBER>0){
				System.out.print("  - Creating synonyms:                  ");
				// Creates first synonyms for concept0
				for (int j=1; j<=SYNONYM_NUMBER;j++){
					System.out.print(".");
					SynonymAbbreviation sa = new SynonymAbbreviation(
																				"synonym"+j, "concept0", null,
																				TermTypes.SYNONYM, "synonym of concept0");
					odeService.addSynonymToTerm("ontology"+i,sa);
				}
				// Creates synonyms for the rest of concepts
				for (int j=1; j<=SYNONYM_NUMBER;j++){
					System.out.print(".");
					SynonymAbbreviation sa = new SynonymAbbreviation(
																				"synonym1", "concept"+j, null,
																				TermTypes.SYNONYM, "synonym of concept"+j);
					odeService.addSynonymToTerm("ontology"+i,sa);
				}
				System.out.println();
			}

			// Creates constants
			if (CONSTANT_NUMBER>0){
				System.out.print("  - Creating constants:                 ");
				for (int j=1; j<=CONSTANT_NUMBER;j++){
					System.out.print(".");
					ConstantDescriptor cd = new ConstantDescriptor(
																				"constant"+j, "constant "+j+" of ontology "+i,
																				ValueTypes.STRING,"measurement unit "+j,
																				"value of constant"+j);
					odeService.insertConstant("ontology"+i,cd);
				}
				System.out.println();
			}

			// Creates class attributes
			if ((CLASS_ATTRIBUTE_NUMBER>0) && (CONCEPT_NUMBER>0)){
				System.out.print("  - Creating class attributes:          ");
				// Creates first class attributes for concept0
				for (int k=0; k<=CLASS_ATTRIBUTE_NUMBER; k++){
					System.out.print(".");
					ClassAttributeDescriptor cad = new ClassAttributeDescriptor(
						"classAttribute"+k,"concept0",ValueTypes.STRING,"ontology"+i,
						"measurement unit "+k,"precision"+k,1,100000,
						"class attribute "+k+" of concept0");
					odeService.insertClassAttribute("ontology"+i,cad);
				}
				// Creates class attributes for the rest of concepts
				for (int k=1; k<=CLASS_ATTRIBUTE_NUMBER; k++){
					System.out.print(".");
					ClassAttributeDescriptor cad = new ClassAttributeDescriptor(
						"classAttribute1","concept"+k,ValueTypes.STRING,"ontology"+i,
						"measurement unit "+k,"precision"+k,1,100000,
						"class attribute 1 of concept"+k);
					odeService.insertClassAttribute("ontology"+i,cad);
				}
				System.out.println();
			}

			// Creates instance attributes
			if ((INSTANCE_ATTRIBUTE_NUMBER>0) && (CONCEPT_NUMBER>0)){
				System.out.print("  - Creating instance attributes:       ");
				// Creates first instance attributes for concept0
				for (int k=0; k<=INSTANCE_ATTRIBUTE_NUMBER; k++){
					System.out.print(".");
					InstanceAttributeDescriptor iad = new InstanceAttributeDescriptor(
						"instanceAttribute"+k,"concept0",ValueTypes.STRING,"ontology"+i,
						"measurement unit "+k,"precision"+k,1,100000,
						"instance attribute "+k+" of concept0","100","1");
					odeService.insertInstanceAttribute("ontology"+i,iad);
				}
				// Creates instance attributes for the rest of concepts
				for (int k=1; k<=INSTANCE_ATTRIBUTE_NUMBER; k++){
					System.out.print(".");
					InstanceAttributeDescriptor iad = new InstanceAttributeDescriptor(
						"instanceAttribute1","concept"+k,ValueTypes.STRING,"ontology"+i,
						"measurement unit "+k,"precision"+k,1,100000,
						"instance attribute 1 of concept"+k,"100","1");
					odeService.insertInstanceAttribute("ontology"+i,iad);
				}
				System.out.println();
			}

			// Creates term relations
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>TERM_RELATION_NUMBER)){
				System.out.print("  - Creating term relations:            ");
				for (int j=1; j<=TERM_RELATION_NUMBER;j++){
					System.out.print(".");
					TermRelation tr = new TermRelation("ontology"+i,null,"termRelation"+j,
																						 "concept"+j,"concept"+(j+1));
					odeService.insertTermRelation(tr);
				}
				System.out.println();
			}

			// Creates groups
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>=GROUP_NUMBER)){
				System.out.print("  - Creating groups:                    ");
				String[] concept_list = new String[1];
				for (int j=1; j<=GROUP_NUMBER;j++){
					System.out.print(".");
					concept_list[0] = "concept"+j;
					Group group = new Group("group"+j,"group "+j+" of ontology"+i,concept_list);
					odeService.addGroup("ontology"+i,group);
				}
				System.out.println();
			}

			// Creates instance sets
			if (INSTANCE_SET_NUMBER>0){
				System.out.print("  - Creating instance sets:             ");
				for (int j=1; j<=INSTANCE_SET_NUMBER;j++){
					System.out.print(".");
					odeService.insertTerm("ontology"+i,"instanceSet"+j,
																"instance set "+j+" of ontology"+i,TermTypes.INSTANCE_SET);
				}
				System.out.println();
			}

			// Creates instances
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>=INSTANCE_NUMBER) &&
					(INSTANCE_SET_NUMBER>0)){
				System.out.print("  - Creating instances:                 ");
				// Creates first instances for instanceSet1
				for (int j=0; j<=INSTANCE_NUMBER;j++){
					System.out.print(".");
					Instance instance = new Instance("instance"+j,"concept"+j,"instanceSet1",
																					 "instance "+j+" of ontology"+i);
					odeService.insertInstance("ontology"+i,instance);
				}
				// Creates instances for the rest of instance sets
				for (int j=2; j<=INSTANCE_NUMBER;j++){
					System.out.print(".");
					Instance instance = new Instance("instance1","concept1","instanceSet"+j,
																					 "instance 1 of ontology"+i);
					odeService.insertInstance("ontology"+i,instance);
					instance = new Instance("instance2","concept2","instanceSet"+j,
																					 "instance 2 of ontology"+i);
					odeService.insertInstance("ontology"+i,instance);
				}
				System.out.println();
			}

			// Creates instance values for instanceSet1
			if (INSTANCE_VALUE_NUMBER>0){
				System.out.print("  - Creating instance values:           ");
				for (int j=1; j<=INSTANCE_VALUE_NUMBER;j++){
					System.out.print(".");
					odeService.addValueToInstance("ontology"+i,"instanceSet1","instance0",
						"concept0","instanceAttribute"+j,"instanceValue"+j);
				}
				System.out.println();
			}

			// Creates relation instances
			if (RELATION_INSTANCE_NUMBER>0){
				System.out.print("  - Creating relation instances:        ");
				for (int j=1; j<=RELATION_INSTANCE_NUMBER;j++){
					System.out.print(".");
					// Esto ha cambiado de la iteracin 3 a la 4
					//TermRelationInstance tri = new TermRelationInstance(
					//	"relationInstance1","termRelation1","instanceSet"+j,
					//	"instance of termRelation1",
					//	"instance1","instance2");
					TermRelation tr = new TermRelation("ontology"+i,null,"termRelation1",
																						 "concept1","concept2");
					TermRelationInstance tri = new TermRelationInstance(
						"relationInstance1",tr,"instanceSet"+j,
						"instance of termRelation1",
						"instance1","instance2");
					odeService.insertRelationInstance("ontology"+i,tri);
				}
				System.out.println();
			}
		}
		// Creates the rest of ontologies
		for (int i=1; i<=ONTOLOGY_NUMBER; i++){
			System.out.println("Creating ontology: ontology"+i);

			// Creates ontology
			try{
				Date now=Calendar.getInstance().getTime();
				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 "admin","admin",now,now);
/* MULTILINGUE				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 	 "admin","admin",0,now,now);*/
				odeService.createOntology(od);
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating ontology: "+e.getMessage());
			}

			// Creates concepts
			if (CONCEPT_NUMBER>0){
				System.out.print("  - Creating concepts:                  ");
				for (int j=1; j<=2;j++){
					System.out.print(".");
					odeService.insertTerm("ontology"+i,"concept"+j,
																"concept "+j+" of ontology "+i,Term.CONCEPT);
				}
				System.out.println();
			}

			// Creates term references and relates it to concepts
			if ((TERM_REFERENCE_NUMBER>0) &&
					(CONCEPT_NUMBER>0) &&
					(TERM_REFERENCE_NUMBER>=CONCEPT_NUMBER)){
				System.out.print("  - Creating term references:           ");
				for (int j=1; j<=1;j++){
					System.out.print(".");
					ReferenceDescriptor rd = new ReferenceDescriptor("termReference"+j,
																													 "reference "+j+" of concept"+j);
					odeService.addReference("ontology"+i,rd);
					odeService.relateReferenceToTerm("ontology"+i,"concept"+j,"termReference"+j);
				}
				System.out.println();
			}

			// Creates formulas
			if (FORMULA_NUMBER>0){
				System.out.print("  - Creating formulas:                  ");
				for (int j=1; j<=1;j++){
					System.out.print(".");
					FormulaDescriptor fd = new FormulaDescriptor(
														  "ontology"+i,"formula"+j, "formula "+j+" of ontology"+i,
															"Have friends and they will give you...",
															"person(?x), person(?y), friend(?x,?y) --> give(?y,?x)",
															Term.AXIOM);
					odeService.insertReasoningElement(fd);
				}
				System.out.println();
			}

			// Creates constants
			if (CONSTANT_NUMBER>0){
				System.out.print("  - Creating constants:                 ");
				for (int j=1; j<=1;j++){
					System.out.print(".");
					ConstantDescriptor cd = new ConstantDescriptor(
																				"constant"+j, "constant "+j+" of ontology "+i,
																				ValueTypes.STRING,"measurement unit "+j,
																				"value of constant"+j);
					odeService.insertConstant("ontology"+i,cd);
				}
				System.out.println();
			}

			// Creates class attributes
			if ((CLASS_ATTRIBUTE_NUMBER>0) && (CONCEPT_NUMBER>0)){
				System.out.print("  - Creating class attributes:          ");
				// Creates class attributes
				for (int k=1; k<=1; k++){
					System.out.print(".");
					ClassAttributeDescriptor cad = new ClassAttributeDescriptor(
						"classAttribute1","concept"+k,ValueTypes.STRING,"ontology"+i,
						"measurement unit "+k,"precision"+k,1,100000,
						"class attribute 1 of concept"+k);
					odeService.insertClassAttribute("ontology"+i,cad);
				}
				System.out.println();
			}

			// Creates term relations
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>TERM_RELATION_NUMBER)){
				System.out.print("  - Creating term relations:            ");
				for (int j=1; j<=1;j++){
					System.out.print(".");
					TermRelation tr = new TermRelation("ontology"+i,null,"termRelation"+j,
																						 "concept"+j,"concept"+(j+1));
					odeService.insertTermRelation(tr);
				}
				System.out.println();
			}

			// Creates groups
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>=GROUP_NUMBER)){
				System.out.print("  - Creating groups:                    ");
				String[] concept_list = new String[1];
				for (int j=1; j<=1;j++){
					System.out.print(".");
					concept_list[0] = "concept"+j;
					Group group = new Group("group"+j,"group "+j+" of ontology"+i,concept_list);
					odeService.addGroup("ontology"+i,group);
				}
				System.out.println();
			}

			// Creates instance sets
			if (INSTANCE_SET_NUMBER>0){
				System.out.print("  - Creating instance sets:             ");
				for (int j=1; j<=1;j++){
					System.out.print(".");
					odeService.insertTerm("ontology"+i,"instanceSet"+j,
																"instance set "+j+" of ontology"+i,TermTypes.INSTANCE_SET);
				}
				System.out.println();
			}

			// Creates instances
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>=INSTANCE_NUMBER) &&
					(INSTANCE_SET_NUMBER>0)){
				System.out.print("  - Creating instances:                 ");
				// Creates instances for instanceSet1
				for (int j=1; j<=2;j++){
					System.out.print(".");
					Instance instance = new Instance("instance"+j,"concept"+j,"instanceSet1",
																					 "instance "+j+" of ontology"+i);
					odeService.insertInstance("ontology"+i,instance);
				}
				System.out.println();
			}

			// Creates relation instances
			if (RELATION_INSTANCE_NUMBER>0){
				System.out.print("  - Creating relation instances:        ");
				for (int j=1; j<=1;j++){
					System.out.print(".");
					// Esto ha cambiado de la iteracin 3 a la 4
					//TermRelationInstance tri = new TermRelationInstance(
					//	"relationInstance1","termRelation1","instanceSet"+j,
					//	"instance of termRelation1",
					//	"instance1","instance2");
					TermRelation tr = new TermRelation("ontology"+i,null,"termRelation1",
																						 "concept1","concept2");
					TermRelationInstance tri = new TermRelationInstance(
						"relationInstance1",tr,"instanceSet"+j,
						"instance of termRelation1",
						"instance1","instance2");
					odeService.insertRelationInstance("ontology"+i,tri);
				}
				System.out.println();
			}
		}
	}

	/**
	 *	Creates the initial state for the non basic group of benchmarks
	 */
	public static void initialStateNonBasic (ODEService odeService) throws Exception
	{
		final int factor = benchmarkConstants.initial_state_factor;

		final int ONTOLOGY_NUMBER = 								factor;

		final int CONCEPT_NUMBER = 									factor;

		// Creates the ontologies one by one
		for (int i=0; i<=ONTOLOGY_NUMBER; i++){
			System.out.println("Creating ontology: ontology"+i);

			// Creates ontology
			try{
				Date now=Calendar.getInstance().getTime();
				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 "admin","admin",now,now);
/* MULTILINGUE				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 	 "admin","admin",0,now,now);*/
				odeService.createOntology(od);
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating ontology: "+e.getMessage());
			}

			// Creates concepts
			if (CONCEPT_NUMBER>0){
				System.out.print("  - Creating concepts:                  ");
				for (int j=0; j<=CONCEPT_NUMBER;j++){
					System.out.print(".");
					odeService.insertTerm("ontology"+i,"concept"+j,
																"concept "+j+" of ontology "+i,Term.CONCEPT);
				}
				System.out.println();
			}
		}
	}

	/**
	 *	Creates the minimal initial state for the non basic group of benchmarks
	 */
	public static void initialStateNonBasicMinimal (ODEService odeService) throws Exception
	{
		final int factor = benchmarkConstants.initial_state_factor;

		final int ONTOLOGY_NUMBER = 								factor;

		final int CONCEPT_NUMBER = 									factor;

		// Creates ontology0
		for (int i=0; i<=0; i++){
			System.out.println("Creating ontology: ontology"+i);

			// Creates ontology
			try{
				Date now=Calendar.getInstance().getTime();
				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 "admin","admin",now,now);
/* MULTILINGUE				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 	 "admin","admin",0,now,now);*/
				odeService.createOntology(od);
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating ontology: "+e.getMessage());
			}

			// Creates concepts
			if (CONCEPT_NUMBER>0){
				System.out.print("  - Creating concepts:                  ");
				for (int j=0; j<=CONCEPT_NUMBER;j++){
					System.out.print(".");
					odeService.insertTerm("ontology"+i,"concept"+j,
																"concept "+j+" of ontology "+i,Term.CONCEPT);
				}
				System.out.println();
			}
		}

		// Creates the rest of ontologies
		for (int i=1; i<=ONTOLOGY_NUMBER; i++){
			System.out.println("Creating ontology: ontology"+i);

			// Creates ontology
			try{
				Date now=Calendar.getInstance().getTime();
				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 "admin","admin",now,now);
/* MULTILINGUE				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 	 "admin","admin",0,now,now);*/
				odeService.createOntology(od);
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating ontology: "+e.getMessage());
			}

			// Creates concepts
			if (CONCEPT_NUMBER>0){
				System.out.print("  - Creating concepts:                  ");
				for (int j=1; j<=2;j++){
					System.out.print(".");
					odeService.insertTerm("ontology"+i,"concept"+j,
																"concept "+j+" of ontology "+i,Term.CONCEPT);
				}
				System.out.println();
			}
		}
	}

	/**
	 *	Creates the initial state for the complex group of benchmarks
	 */
	public static void initialStateComplex (ODEService odeService) throws Exception
	{
		final int factor = benchmarkConstants.initial_state_factor;

		final int ONTOLOGY_NUMBER = 								factor;

		final int CONCEPT_NUMBER = 									factor+1;
		final int CONSTANT_NUMBER = 								factor;
		final int GROUP_NUMBER = 										factor;
		final int TERM_RELATION_NUMBER = 						factor; // CONCEPT_NUMBER-1

		// Creates the ontologies one by one
		for (int i=0; i<=ONTOLOGY_NUMBER; i++){
			System.out.println("Creating ontology: ontology"+i);

			// Creates ontology
			try{
				Date now=Calendar.getInstance().getTime();
				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 "admin","admin",now,now);
/* MULTILINGUE				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 	 "admin","admin",0,now,now);*/
				odeService.createOntology(od);
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating ontology: "+e.getMessage());
			}

			// Creates concepts
			if (CONCEPT_NUMBER>0){
				System.out.print("  - Creating concepts:                  ");
				for (int j=1; j<=CONCEPT_NUMBER;j++){
					System.out.print(".");
					odeService.insertTerm("ontology"+i,"concept"+j,
																"concept "+j+" of ontology "+i,Term.CONCEPT);
				}
				System.out.println();
			}

			// Creates constants
			// Without this, the merging of ontologies OIL fails.
			if (CONSTANT_NUMBER>0){
				System.out.print("  - Creating constants:                 ");
				for (int j=1; j<=CONSTANT_NUMBER;j++){
					System.out.print(".");
					ConstantDescriptor cd = new ConstantDescriptor(
																				"constant"+j, "constant "+j+" of ontology "+i,
																				ValueTypes.STRING,"measurement unit "+j,
																				"value of constant"+j);
					odeService.insertConstant("ontology"+i,cd);
				}
				System.out.println();
			}

			// Creates groups
			// Without this, the merging of ontologies OIL fails.
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>=GROUP_NUMBER)){
				System.out.print("  - Creating groups:                    ");
				String[] concept_list = new String[1];
				for (int j=1; j<=GROUP_NUMBER;j++){
					System.out.print(".");
					concept_list[0] = "concept"+j;
					Group group = new Group("group"+j,"group "+j+" of ontology"+i,concept_list);
					odeService.addGroup("ontology"+i,group);
				}
				System.out.println();
			}

			// Creates term relations
			// Without this, the export to OIL fails.
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>TERM_RELATION_NUMBER)){
				System.out.print("  - Creating term relations:            ");
				for (int j=1; j<=TERM_RELATION_NUMBER;j++){
					System.out.print(".");
					TermRelation tr = new TermRelation("ontology"+i,null,"termRelation"+j,
																						 "concept"+j,"concept"+(j+1));
					odeService.insertTermRelation(tr);
				}
				System.out.println();
			}
		}
	}

	/**
	 *	Creates the initial state for all the benchmarks
	 */
	public static void initialStateCommonMinimal (ODEService odeService) throws Exception
	{
		final int factor = benchmarkConstants.initial_state_factor;

		final int ONTOLOGY_NUMBER = 								factor;

		final int CONCEPT_NUMBER = 									factor+1;
		final int TERM_REFERENCE_NUMBER = 					factor;
		final int ONTOLOGY_REFERENCE_NUMBER = 			factor;
		final int FORMULA_NUMBER = 									factor;
		final int SYNONYM_NUMBER = 									factor;
		final int CONSTANT_NUMBER = 								factor;
		final int CLASS_ATTRIBUTE_NUMBER = 					factor;
		final int CLASS_ATTRIBUTE_VALUE_NUMBER = 		CLASS_ATTRIBUTE_NUMBER;
		final int INSTANCE_ATTRIBUTE_NUMBER = 			factor;
		final int INSTANCE_ATTRIBUTE_VALUE_NUMBER = INSTANCE_ATTRIBUTE_NUMBER;
		final int TERM_RELATION_NUMBER = 						factor; // CONCEPT_NUMBER-1
		final int INSTANCE_SET_NUMBER = 						factor;
		final int INSTANCE_NUMBER = 								factor+1;
		final int RELATION_INSTANCE_NUMBER = 				factor;
		final int INSTANCE_VALUE_NUMBER = 					factor;
		final int GROUP_NUMBER = 										factor;

		// Creates ontology0
		for (int i=0; i<=0; i++){
			System.out.println("Creating ontology: ontology"+i);

			// Creates ontology
			try{
				Date now=Calendar.getInstance().getTime();
				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 "admin","admin",now,now);
/* MULTILINGUE				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 	 "admin","admin",0,now,now);*/
				odeService.createOntology(od);
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating ontology: "+e.getMessage());
			}

			// Creates concepts
			if (CONCEPT_NUMBER>0){
				for (int j=0; j<=CONCEPT_NUMBER;j++){
					System.out.print("\r  - Creating concepts:                  "+j);
					odeService.insertTerm("ontology"+i,"concept"+j,
																"concept "+j+" of ontology "+i,Term.CONCEPT);
				}
				System.out.println();
			}

			// Creates term references
			if ((TERM_REFERENCE_NUMBER>0) &&
					(CONCEPT_NUMBER>0) &&
					(TERM_REFERENCE_NUMBER<=CONCEPT_NUMBER)){
				for (int j=1; j<=TERM_REFERENCE_NUMBER;j++){
					System.out.print("\r  - Creating term references:           "+j);
					ReferenceDescriptor rd = new ReferenceDescriptor("termReference"+j,
																													 "reference "+j+" of concept"+j);
					odeService.addReference("ontology"+i,rd);
					odeService.relateReferenceToTerm("ontology"+i,"concept"+(j-1),"termReference"+j);
				}
				System.out.println();
			}

			// Creates ontology references
			if (ONTOLOGY_REFERENCE_NUMBER>0){
				for (int j=1; j<=ONTOLOGY_REFERENCE_NUMBER;j++){
					System.out.print("\r  - Creating ontology references:       "+j);
					ReferenceDescriptor rd = new ReferenceDescriptor("ontologyReference"+j,
																													 "reference "+j+" of ontology"+i);
					odeService.addReferenceToOntology("ontology"+i,rd);
				}
				System.out.println();
			}

			// Creates formulas
			if (FORMULA_NUMBER>0){
				for (int j=1; j<=FORMULA_NUMBER;j++){
					System.out.print("\r  - Creating formulas:                  "+j);
					FormulaDescriptor fd = new FormulaDescriptor(
														  "ontology"+i,"formula"+j, "formula "+j+" of ontology"+i,
															"Have friends and they will give you...",
															"person(?x), person(?y), friend(?x,?y) --> give(?y,?x)",
															Term.AXIOM);
					odeService.insertReasoningElement(fd);
					odeService.relateFormulaToTerm("ontology"+i,"concept"+(j-1),"formula"+j);
				}
				System.out.println();
			}

			// Creates synonyms
			if (SYNONYM_NUMBER>0){
				// Creates first synonyms for concept0
				for (int j=1; j<=SYNONYM_NUMBER;j++){
					System.out.print("\r  - Creating synonyms:                  "+j);
					SynonymAbbreviation sa = new SynonymAbbreviation(
																				"synonym"+j, "concept0", null,
																				TermTypes.SYNONYM, "synonym of concept0");
					odeService.addSynonymToTerm("ontology"+i,sa);
				}
				System.out.println();
				// Creates synonyms for the rest of concepts
				for (int j=1; j<=SYNONYM_NUMBER;j++){
					System.out.print("\r  - Creating synonyms(2):               "+j);
					SynonymAbbreviation sa = new SynonymAbbreviation(
																				"synonym1", "concept"+j, null,
																				TermTypes.SYNONYM, "synonym of concept"+j);
					odeService.addSynonymToTerm("ontology"+i,sa);
				}
				System.out.println();
			}

			// Creates constants
			if (CONSTANT_NUMBER>0){
				for (int j=1; j<=CONSTANT_NUMBER;j++){
					System.out.print("\r  - Creating constants:                 "+j);
					ConstantDescriptor cd = new ConstantDescriptor(
																				"constant"+j, "constant "+j+" of ontology "+i,
																				ValueTypes.STRING,"measurement unit "+j,
																				"value of constant"+j);
					odeService.insertConstant("ontology"+i,cd);
				}
				System.out.println();
			}

			// Creates class attributes
			if ((CLASS_ATTRIBUTE_NUMBER>0) && (CONCEPT_NUMBER>0)){
				// First creates class attributes for concept0
				for (int k=0; k<=CLASS_ATTRIBUTE_NUMBER; k++){
					System.out.print("\r  - Creating class attributes:          "+k);
					ClassAttributeDescriptor cad = new ClassAttributeDescriptor(
						"classAttribute"+k,"concept0",ValueTypes.STRING,"ontology"+i,
						"measurement unit "+k,"precision"+k,1,100000,
						"class attribute "+k+" of concept0");
					odeService.insertClassAttribute("ontology"+i,cad);
				}
				System.out.println();
				// Creates class attributes for concept1
				for (int k=0; k<=CLASS_ATTRIBUTE_NUMBER; k++){
					System.out.print("\r  - Creating class attributes(2):       "+k);
					ClassAttributeDescriptor cad = new ClassAttributeDescriptor(
						"classAttribute"+k,"concept1",ValueTypes.STRING,"ontology"+i,
						"measurement unit "+k,"precision"+k,1,100000,
						"class attribute "+k+" of concept1");
					odeService.insertClassAttribute("ontology"+i,cad);
				}
				System.out.println();
				// Creates class attributes for the rest of concepts
				for (int k=2; k<=CLASS_ATTRIBUTE_NUMBER; k++){
					System.out.print("\r  - Creating class attributes(3):       "+k);
					ClassAttributeDescriptor cad = new ClassAttributeDescriptor(
						"classAttribute1","concept"+k,ValueTypes.STRING,"ontology"+i,
						"measurement unit "+k,"precision"+k,1,100000,
						"class attribute 1 of concept"+k);
					odeService.insertClassAttribute("ontology"+i,cad);
				}
				System.out.println();
			}

			// Creates class attribute values for concept0
			if ((CONCEPT_NUMBER>0) &&
					(CLASS_ATTRIBUTE_NUMBER>0) &&
					(CLASS_ATTRIBUTE_NUMBER>=CLASS_ATTRIBUTE_VALUE_NUMBER)){
				// Creates class attribute values for classAttribute0
				for (int k=1; k<=CLASS_ATTRIBUTE_VALUE_NUMBER; k++){
					System.out.print("\r  - Creating class attribute values:    "+k);
					odeService.addValueToClassAttribute("ontology"+i,"classAttribute0",
																							"concept1","classAttributeValue"+k);
				}
				System.out.println();
				// Creates class attribute values for the rest class attributes
				for (int k=1; k<=CLASS_ATTRIBUTE_VALUE_NUMBER; k++){
					System.out.print("\r  - Creating class attribute values(2): "+k);
					odeService.addValueToClassAttribute("ontology"+i,"classAttribute"+k,
																							"concept1","classAttributeValue1");
				}
				System.out.println();
			}

			// Creates instance attributes
			if ((INSTANCE_ATTRIBUTE_NUMBER>0) && (CONCEPT_NUMBER>0)){
				// First creates instance attributes for concept0
				for (int k=0; k<=INSTANCE_ATTRIBUTE_NUMBER; k++){
					System.out.print("\r  - Creating instance attributes:       "+k);
					InstanceAttributeDescriptor iad = new InstanceAttributeDescriptor(
						"instanceAttribute"+k,"concept0",ValueTypes.STRING,"ontology"+i,
						"measurement unit "+k,"precision"+k,1,100000,
						"instance attribute "+k+" of concept0","100","1");
					odeService.insertInstanceAttribute("ontology"+i,iad);
				}
				System.out.println();
				// First creates instance attributes for concept1
				for (int k=0; k<=INSTANCE_ATTRIBUTE_NUMBER; k++){
					System.out.print("\r  - Creating instance attributes(2):    "+k);
					InstanceAttributeDescriptor iad = new InstanceAttributeDescriptor(
						"instanceAttribute"+k,"concept1",ValueTypes.STRING,"ontology"+i,
						"measurement unit "+k,"precision"+k,1,100000,
						"instance attribute "+k+" of concept1","100","1");
					odeService.insertInstanceAttribute("ontology"+i,iad);
				}
				System.out.println();
				// Creates the rest of instance attributes
				for (int k=2; k<=INSTANCE_ATTRIBUTE_NUMBER; k++){
					System.out.print("\r  - Creating instance attributes(3):    "+k);
					InstanceAttributeDescriptor iad = new InstanceAttributeDescriptor(
						"instanceAttribute1","concept"+k,ValueTypes.STRING,"ontology"+i,
						"measurement unit "+k,"precision"+k,1,100000,
						"instance attribute 1 of concept"+k,"100","1");
					odeService.insertInstanceAttribute("ontology"+i,iad);
				}
				System.out.println();
			}

			// Creates instance attribute values for concept0
			if ((CONCEPT_NUMBER>0) &&
					(INSTANCE_ATTRIBUTE_NUMBER>0) &&
					(INSTANCE_ATTRIBUTE_NUMBER>=INSTANCE_ATTRIBUTE_VALUE_NUMBER)){
				// Creates instance attribute values for instanceAttribute0
				for (int k=1; k<=INSTANCE_ATTRIBUTE_VALUE_NUMBER; k++){
					System.out.print("\r  - Creating instance attribute values: "+k);
					odeService.addValueToInstanceAttribute("ontology"+i,"instanceAttribute0",
																							   "concept1","instanceAttributeValue"+k);
				}
				System.out.println();
				// Creates instance attribute values for the rest of instance attributes
				for (int k=1; k<=INSTANCE_ATTRIBUTE_VALUE_NUMBER; k++){
					System.out.print("\r  - Creating instance attribute values: "+k);
					odeService.addValueToInstanceAttribute("ontology"+i,"instanceAttribute"+k,
																							   "concept1","instanceAttributeValue1");
				}
				System.out.println();
			}

			// Creates term relations
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>TERM_RELATION_NUMBER)){
				for (int j=1; j<=TERM_RELATION_NUMBER;j++){
					System.out.print("\r  - Creating term relations:            "+j);
					TermRelation tr = new TermRelation("ontology"+i,null,"termRelation"+j,
																						 "concept"+j,"concept"+(j+1));
					odeService.insertTermRelation(tr);
				}
				System.out.println();
			}

			// Creates instance sets
			if (INSTANCE_SET_NUMBER>0){
				for (int j=1; j<=INSTANCE_SET_NUMBER;j++){
					System.out.print("\r  - Creating instance sets:             "+j);
					odeService.insertTerm("ontology"+i,"instanceSet"+j,
																"instance set "+j+" of ontology"+i,TermTypes.INSTANCE_SET);
				}
				System.out.println();
			}

			// Creates instances
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>=INSTANCE_NUMBER) &&
					(INSTANCE_SET_NUMBER>0)){
				// First creates instances for instanceSet1
				for (int j=0; j<=INSTANCE_NUMBER;j++){
					System.out.print("\r  - Creating instances:                 "+j);
					Instance instance = new Instance("instance"+j,"concept"+j,"instanceSet1",
																					 "instance "+j+" of ontology"+i);
					odeService.insertInstance("ontology"+i,instance);
				}
				System.out.println();
				// Creates instances for the rest of instanceSets
				for (int j=2; j<=INSTANCE_SET_NUMBER;j++){
					System.out.print("\r  - Creating instances:                 "+(j*2));
					Instance instance = new Instance("instance1","concept1","instanceSet"+j,
																					 "instance 1 of ontology"+i);
					odeService.insertInstance("ontology"+i,instance);
					Instance instance2 = new Instance("instance2","concept2","instanceSet"+j,
																					 "instance 2 of ontology"+i);
					odeService.insertInstance("ontology"+i,instance2);
				}
				System.out.println();
			}

			// Creates instance values
			if (INSTANCE_VALUE_NUMBER>0){
				for (int j=1; j<=INSTANCE_VALUE_NUMBER;j++){
					System.out.print("\r  - Creating instance values:           "+j);
					odeService.addValueToInstance("ontology"+i,"instanceSet1","instance0",
						"concept1","instanceAttribute"+j,"instanceValue"+j);
				}
				System.out.println();
			}

			// Creates relation instances
			if (RELATION_INSTANCE_NUMBER>0){
				for (int j=1; j<=RELATION_INSTANCE_NUMBER;j++){
					System.out.print("\r  - Creating relation instances:        "+j);
					// Esto ha cambiado de la iteracin 3 a la 4
					//TermRelationInstance tri = new TermRelationInstance(
					//	"relationInstance1","termRelation1","instanceSet"+j,
					//	"instance of termRelation1",
					//	"instance1","instance2");
					TermRelation tr = new TermRelation("ontology"+i,null,"termRelation1",
																						 "concept1","concept2");
					TermRelationInstance tri = new TermRelationInstance(
						"relationInstance1",tr,"instanceSet"+j,
						"instance of termRelation1",
						"instance1","instance2");
					odeService.insertRelationInstance("ontology"+i,tri);
				}
				System.out.println();
			}

			// Creates groups
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>=GROUP_NUMBER)){
				String[] concept_list = new String[1];
				for (int j=1; j<=GROUP_NUMBER;j++){
					System.out.print("\r  - Creating groups:                    "+j);
					concept_list[0] = "concept"+j;
					Group group = new Group("group"+j,"group "+j+" of ontology"+i,concept_list);
					odeService.addGroup("ontology"+i,group);
				}
				System.out.println();
			}
		}

		// Creates the rest of ontologies
		for (int i=1; i<=ONTOLOGY_NUMBER; i++){
			System.out.println("Creating ontology: ontology"+i);

			// Creates ontology
			try{
				Date now=Calendar.getInstance().getTime();
				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 "admin","admin",now,now);
/* MULTILINGUE				OntologyDescriptor od = new OntologyDescriptor("ontology"+i,"webode://webode.dia.fi.upm.es/ontology"+i+"#","ontology "+i,
																										 	 "admin","admin",0,now,now);*/
				odeService.createOntology(od);
			}
			catch (WebODEException e){
				throw new Exception("ERROR creating ontology: "+e.getMessage());
			}

			// Creates concepts
			if (CONCEPT_NUMBER>0){
				for (int j=0; j<=2;j++){
					System.out.print("\r  - Creating concepts:                  "+j);
					odeService.insertTerm("ontology"+i,"concept"+j,
																"concept "+j+" of ontology "+i,Term.CONCEPT);
				}
				System.out.println();
			}

			// Creates term references
			if ((TERM_REFERENCE_NUMBER>0) &&
					(CONCEPT_NUMBER>0) &&
					(TERM_REFERENCE_NUMBER<=CONCEPT_NUMBER)){
				for (int j=1; j<=2;j++){
					System.out.print("\r  - Creating term references:           "+j);
					ReferenceDescriptor rd = new ReferenceDescriptor("termReference"+j,
																													 "reference "+j+" of concept"+j);
					odeService.addReference("ontology"+i,rd);
				}
				odeService.relateReferenceToTerm("ontology"+i,"concept2","termReference2");
				System.out.println();
			}

			// Creates ontology references
			if (ONTOLOGY_REFERENCE_NUMBER>0){
				for (int j=1; j<=1;j++){
					System.out.print("\r  - Creating ontology references:       "+j);
					ReferenceDescriptor rd = new ReferenceDescriptor("ontologyReference"+j,
																													 "reference "+j+" of ontology"+i);
					odeService.addReferenceToOntology("ontology"+i,rd);
				}
				System.out.println();
			}

			// Creates formulas
			if (FORMULA_NUMBER>0){
				for (int j=1; j<=2;j++){
					System.out.print("\r  - Creating formulas:                  "+j);
					FormulaDescriptor fd = new FormulaDescriptor(
														  "ontology"+i,"formula"+j, "formula "+j+" of ontology"+i,
															"Have friends and they will give you...",
															"person(?x), person(?y), friend(?x,?y) --> give(?y,?x)",
															Term.AXIOM);
					odeService.insertReasoningElement(fd);
				}

				odeService.relateFormulaToTerm("ontology"+i,"concept2","formula2");
				System.out.println();
			}

			// Creates constants
			if (CONSTANT_NUMBER>0){
				for (int j=1; j<=1;j++){
					System.out.print("\r  - Creating constants:                 "+j);
					ConstantDescriptor cd = new ConstantDescriptor(
																				"constant"+j, "constant "+j+" of ontology "+i,
																				ValueTypes.STRING,"measurement unit "+j,
																				"value of constant"+j);
					odeService.insertConstant("ontology"+i,cd);
				}
				System.out.println();
			}

			// Creates class attributes for concept0
			if ((CLASS_ATTRIBUTE_NUMBER>0) && (CONCEPT_NUMBER>0)){
				for (int k=0; k<=0; k++){
					System.out.print("\r  - Creating class attributes:          "+k);
					ClassAttributeDescriptor cad = new ClassAttributeDescriptor(
						"classAttribute"+k,"concept0",ValueTypes.STRING,"ontology"+i,
						"measurement unit "+k,"precision"+k,1,100000,
						"class attribute "+k+" of concept0");
					odeService.insertClassAttribute("ontology"+i,cad);
				}
				System.out.println();
			}

			// Creates instance attributes for concept0
			if ((INSTANCE_ATTRIBUTE_NUMBER>0) && (CONCEPT_NUMBER>0)){
				for (int k=0; k<=0; k++){
					System.out.print("\r  - Creating instance attributes:       "+k);
					InstanceAttributeDescriptor iad = new InstanceAttributeDescriptor(
						"instanceAttribute"+k,"concept0",ValueTypes.STRING,"ontology"+i,
						"measurement unit "+k,"precision"+k,1,100000,
						"instance attribute "+k+" of concept0","100","1");
					odeService.insertInstanceAttribute("ontology"+i,iad);
				}
				System.out.println();
			}

			// Creates term relations
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>TERM_RELATION_NUMBER)){
				for (int j=1; j<=1;j++){
					System.out.print("\r  - Creating term relations:            "+j);
					TermRelation tr = new TermRelation("ontology"+i,null,"termRelation"+j,
																						 "concept"+j,"concept"+(j+1));
					odeService.insertTermRelation(tr);
				}
				System.out.println();
			}

			// Creates instance sets
			if (INSTANCE_SET_NUMBER>0){
				for (int j=1; j<=1;j++){
					System.out.print("\r  - Creating instance sets:             "+j);
					odeService.insertTerm("ontology"+i,"instanceSet"+j,
																"instance set "+j+" of ontology"+i,TermTypes.INSTANCE_SET);
				}
				System.out.println();
			}

			// Creates instances for instanceSet1
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>=INSTANCE_NUMBER) &&
					(INSTANCE_SET_NUMBER>0)){
				for (int j=0; j<=2;j++){
					System.out.print("\r  - Creating instances:                 "+j);
					Instance instance = new Instance("instance"+j,"concept"+j,"instanceSet1",
																					 "instance "+j+" of ontology"+i);
					odeService.insertInstance("ontology"+i,instance);
				}
				System.out.println();
			}

			// Creates instance values
			if (INSTANCE_VALUE_NUMBER>0){
				for (int j=1; j<=1;j++){
					System.out.print("\r  - Creating instance values:           "+j);
					odeService.addValueToInstance("ontology"+i,"instanceSet1","instance0",
						"concept0","instanceAttribute0","instanceValue"+j);
				}
				System.out.println();
			}

			// Creates relation instances
			if (RELATION_INSTANCE_NUMBER>0){
				for (int j=1; j<=1;j++){
					System.out.print("\r  - Creating relation instances:        "+j);
					// Esto ha cambiado de la iteracin 3 a la 4
					//TermRelationInstance tri = new TermRelationInstance(
					//	"relationInstance1","termRelation1","instanceSet"+j,
					//	"instance of termRelation1",
					//	"instance1","instance2");
					TermRelation tr = new TermRelation("ontology"+i,null,"termRelation1",
																						 "concept1","concept2");
					TermRelationInstance tri = new TermRelationInstance(
						"relationInstance1",tr,"instanceSet"+j,
						"instance of termRelation1",
						"instance1","instance2");
					odeService.insertRelationInstance("ontology"+i,tri);
				}
				System.out.println();
			}

			// Creates groups
			if ((CONCEPT_NUMBER>0) &&
					(CONCEPT_NUMBER>=GROUP_NUMBER)){
				String[] concept_list = new String[1];
				for (int j=1; j<=1;j++){
					System.out.print("\r  - Creating groups:                    "+j);
					concept_list[0] = "concept"+j;
					Group group = new Group("group"+j,"group "+j+" of ontology"+i,concept_list);
					odeService.addGroup("ontology"+i,group);
				}
				System.out.println();
			}

		}
  }

	/* Esto de abajo es un intento de importar la ontologa "top level ontology of universals"
	 * a partir de llamadas al servicio xml-import
		String fichero = "d:\\rgarcia\\ontologias\\top level ontology of universals.xml";
			java.io.BufferedReader br = new java.io.BufferedReader(new java.io.FileReader(fichero));
		String input;

	// Set up a buffer
	boolean first = true;
	int line = 0;
	// 8k
	StringBuffer strBuffer = new StringBuffer (8096);

	  while ((input = br.readLine()) != null) {
//	while ((foo = sis.readLine (buffer, 0, buffer.length)) >= 0) {
	    String str;
	    str = new String (input);
	    if (str.startsWith ("------------")) {
				if (first) {
		   		first = false;
		    	continue;
				}
				else
		    	break;
	   	}
	    line++;

	    // Skip headers
	    if (line > 3) {
	    	if (str.length() > 1){
	    		if (line <10)
		    		System.out.println(str);
					strBuffer.append (str);
				}
			}
		}

//	sis.close();
    ImportService xmlImport = (ImportService) ms.getService("xml-import");

		xmlImport.loadInMemory (strBuffer);



    String newName = "top level ontology of universals";

            Date date = new Date();
            odeService.createOntology (
	        new OntologyDescriptor (newName, xmlImport.getOntologyDescription(),
	                           "admin", "admin",
				   date, date));

         xmlImport.setOntologyName (newName);
	          xmlImport.importReferences ();
                  xmlImport.importFormulas ();
                  xmlImport.importImportedTerms ();
                  xmlImport.importProperties ();
                  xmlImport.importConcepts ();
                  xmlImport.importGroups ();
                  xmlImport.importConstants ();
                  xmlImport.importRelationships ();

	 */

}
