package es.upm.fi.dia.ontology.webode.wpbs;

// Java stuff
import es.upm.fi.dia.ontology.webode.service.ODEService;
import es.upm.fi.dia.ontology.webode.service.Term;
import es.upm.fi.dia.ontology.webode.service.WebODEException;

/** Implements a set of benchmarks for testing ODE service.
 *	All the benchmarks in this class test non basic operations  
 *	which use other methods in the service.
 */
public class benchmarkNonBasic
{

	/**
	 * Opens several times ontologies.
	 *
	 *	The database must have OPEN_ONTOLOGIES ontologies named ONTOLOGY_NAMEx
	 */
	public static void benchmark2_01 (ODEService odeService) throws Exception
	{
		final int OPEN_ONTOLOGIES = benchmarkConstants.TEST_ONTOLOGIES;
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(OPEN_ONTOLOGIES);
		
		// Opens ontologies one by one
		System.out.println("Opening ontology: ");
		for (int i=1; i<=OPEN_ONTOLOGIES+benchmarkConstants.warming; i++){
			System.out.print("\r  "+ONTOLOGY_NAME+i);
				
			// Opens the ontology
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				odeService.openOntology(ONTOLOGY_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR opening ontology: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark2_01.txt");
	}
	
	/**
	 * Gets if several concepts are subclass of other concepts in an ontology.
	 *
	 *	The database must have a ontology named ONTOLOGY_NAME
	 *	with CHECK_SUBCLASS+1 concepts named CONCEPT_NAMEx
	 */
	public static void benchmark2_02 (ODEService odeService) throws Exception
	{
		final int CHECK_SUBCLASS = benchmarkConstants.TEST_CONCEPTS;
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology1";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(CHECK_SUBCLASS);
		
		// Checks concepts one by one
		System.out.println("Checking concept: ");
		for (int i=1; i<=CHECK_SUBCLASS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CONCEPT_NAME+i);
				
			// Checks concepts
			try{
				Term term1 = odeService.getTerm(ONTOLOGY_NAME,CONCEPT_NAME+i);
				Term term2 = odeService.getTerm(ONTOLOGY_NAME,CONCEPT_NAME+(i+1));
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				boolean b = odeService.isSubclassOf(ONTOLOGY_NAME,term1,term2);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR checking concepts: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark2_02.txt");
	}

	/**
	 * Gets if several concepts are subclass of other concepts in several ontologies.
	 *
	 *	The database must have CHECK_SUBCLASS ontologies named ONTOLOGY_NAMEx
	 *	with two concepts named CONCEPT_NAME1 and CONCEPT_NAME2
	 */
	public static void benchmark2_03 (ODEService odeService) throws Exception
	{
		final int CHECK_SUBCLASS = benchmarkConstants.TEST_CONCEPTS;
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(CHECK_SUBCLASS);
		
		// Checks concepts one by one
		System.out.println("Checking concept: ");
		for (int i=1; i<=CHECK_SUBCLASS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CONCEPT_NAME+"1"+"("+i+")");
				
			// Checks concepts
			try{
				Term term1 = odeService.getTerm(ONTOLOGY_NAME+i,CONCEPT_NAME+"1");
				Term term2 = odeService.getTerm(ONTOLOGY_NAME+i,CONCEPT_NAME+"2");
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				boolean b = odeService.isSubclassOf(ONTOLOGY_NAME+i,term1,term2);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR checking concepts: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark2_03.txt");
	}

	/**
	 *	Gets the inherited term relation of several concepts in an ontology.
	 *
	 *	The database must have at least one ontology named ONTOLOGY_NAME
	 *	with GET_INHERITED_TERM_RELATIONS concepts named CONCEPT_NAMEx
	 *	with inherited term relations
	 */
	public static void benchmark2_04 (ODEService odeService) throws Exception
	{
		final int GET_INHERITED_TERM_RELATIONS = benchmarkConstants.TEST_CONCEPTS;
		final String CONCEPT_NAME = "concept";
		final String ONTOLOGY_NAME = "ontology1";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_INHERITED_TERM_RELATIONS);
		
		// Gets inherited term relations one by one
		System.out.println("Getting inherited term relations of concept: ");
		for (int i=1; i<=GET_INHERITED_TERM_RELATIONS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CONCEPT_NAME+i);

			// Gets the inherited term relations
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				odeService.getInheritedTermRelations(ONTOLOGY_NAME,CONCEPT_NAME+i);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting inherited term relations: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark2_04.txt");
	}	

	/**
	 *	Gets the inherited term relation of several concepts in several ontologies.
	 *
	 *	The database must have at least GET_INHERITED_TERM_RELATIONS ontologies named ONTOLOGY_NAMEx
	 *	with a concept named CONCEPT_NAME
	 *	with inherited term relations
	 */
	public static void benchmark2_05 (ODEService odeService) throws Exception
	{
		final int GET_INHERITED_TERM_RELATIONS = benchmarkConstants.TEST_CONCEPTS;
		final String CONCEPT_NAME = "concept1";
		final String ONTOLOGY_NAME = "ontology";
		
		// Creates the timer
		benchmarkTimer timer = new benchmarkTimer(GET_INHERITED_TERM_RELATIONS);
		
		// Gets inherited term relations one by one
		System.out.println("Getting inherited term relations of concept: ");
		for (int i=1; i<=GET_INHERITED_TERM_RELATIONS+benchmarkConstants.warming; i++){
			System.out.print("\r  "+CONCEPT_NAME+"("+i+")");

			// Gets the inherited term relations
			try{
				if (i>benchmarkConstants.warming){
          timer.start();
        }
				odeService.getInheritedTermRelations(ONTOLOGY_NAME+i,CONCEPT_NAME);
				if (i>benchmarkConstants.warming){
          timer.stop();
        }
			}
			catch (WebODEException e){
				throw new Exception("ERROR getting inherited term relations: "+e.getMessage());
			}
		}	
		System.out.println("");
		
		benchmarkGeneral.save_timer_data(timer,"benchmark2_05.txt");
	}	

}